/*****************************************************************************
 *
 * NETSAINT.C - Core Program Code For NetSaint
 *
 * Program: NetSaint
 * Version: 0.0.5b4
 * License: GPL
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 *
 * First Written:   01-28-1999 (start of development for version 0.0.1)
 * Last Modified:   02-21-2000
 *
 * Description:
 *
 * NetSaint is a network monitoring tool that will check hosts and services
 * that you specify.  It has the ability to notify contacts via email, pager,
 * or other user-defined methods when a service or host goes down and
 * recovers.  Service and host monitoring is done through the use of external
 * plugins which can be developed independently of NetSaint.
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netsaint.h"

#define 	PROGRAM_VERSION		        "0.0.5b4"
#define         PROGRAM_MODIFICATION_DATE       "02-21-2000"

char		config_file[MAX_FILENAME_LENGTH];
char            hosts_config_file[MAX_FILENAME_LENGTH];
char		log_file[MAX_FILENAME_LENGTH];
char            status_file[MAX_FILENAME_LENGTH];
char            command_file[MAX_FILENAME_LENGTH];
char            temp_file[MAX_FILENAME_LENGTH];
char            comment_file[MAX_FILENAME_LENGTH];
char            log_archive_path[MAX_FILENAME_LENGTH];

char            macro_contact_name[MAX_CONTACTNAME_LENGTH];
char            macro_contact_alias[MAX_CONTACTNAME_LENGTH];
char	        macro_host_name[MAX_HOSTNAME_LENGTH];
char	        macro_host_alias[MAX_HOSTNAME_LENGTH];
char	        macro_host_address[MAX_ADDRESS_LENGTH];
char	        macro_service_description[MAX_SERVICEDESC_LENGTH];
char	        macro_service_state[MAX_STATE_LENGTH];
char	        macro_date_time[MAX_DATETIME_LENGTH];
char	        macro_output[MAX_PLUGINOUTPUT_LENGTH];
char            macro_contact_email[MAX_EMAIL_LENGTH];
char            macro_contact_pager[MAX_PAGER_LENGTH];
char            macro_summary[MAX_COMMANDLINE_LENGTH];
char            macro_admin_email[MAX_EMAIL_LENGTH];
char            macro_admin_pager[MAX_PAGER_LENGTH];
char            macro_host_state[MAX_STATE_LENGTH];
char            macro_state_type[MAX_STATETYPE_LENGTH];
char            macro_current_service_attempt[MAX_ATTEMPT_LENGTH];
char            macro_current_host_attempt[MAX_ATTEMPT_LENGTH];
char            macro_argv[MAX_COMMAND_ARGUMENTS][MAX_COMMANDLINE_LENGTH];

char            generic_summary[MAX_COMMANDLINE_LENGTH];

char            global_host_event_handler[MAX_COMMANDNAME_LENGTH];
char            global_service_event_handler[MAX_COMMANDNAME_LENGTH];

int		log_level=DEFAULT_LOG_LEVEL;
int		use_syslog=DEFAULT_USE_SYSLOG;
int		syslog_level=DEFAULT_SYSLOG_LEVEL;
int             log_notifications=DEFAULT_NOTIFICATION_LOGGING;
int             log_service_retries=DEFAULT_LOG_SERVICE_RETRIES;
int             log_host_retries=DEFAULT_LOG_HOST_RETRIES;
int             log_event_handlers=DEFAULT_LOG_EVENT_HANDLERS;

int             sleep_time=DEFAULT_SLEEP_TIME;
int             interval_length=DEFAULT_INTERVAL_LENGTH;
int             inter_check_delay_method=ICD_SMART;
double          inter_check_delay=DEFAULT_INTER_CHECK_DELAY;
int             interleave_factor_method=ILF_SMART;
int             interleave_factor=DEFAULT_INTERLEAVE_FACTOR;

int             max_child_process_time=DEFAULT_MAX_CHILD_PROCESS_TIME;
int             child_process_timed_out=FALSE;
int             total_orphaned_children=0;

int             zombie_cleanup_interval=DEFAULT_ZOMBIE_CLEANUP_INTERVAL;
int             command_check_interval=DEFAULT_COMMAND_CHECK_INTERVAL;
int             service_check_reaper_interval=DEFAULT_SERVICE_REAPER_INTERVAL;

int             check_external_commands=DEFAULT_CHECK_EXTERNAL_COMMANDS;

time_t          last_zombie_cleanup=0L;
time_t          last_command_check=0L;
time_t          last_log_rotation=0L;

int             use_agressive_host_checking=DEFAULT_AGRESSIVE_HOST_CHECKING;

int             log_rotation_method=LOG_ROTATION_NONE;

int             sigshutdown=FALSE;
int             sigrestart=FALSE;

int             restarting=FALSE;

int             verify_config=FALSE;
int             daemon_mode=FALSE;

int             current_comment_id=0;

int             ipc_message_queue=0;

int             max_parallel_service_checks=DEFAULT_MAX_PARALLEL_SERVICE_CHECKS;
int             currently_running_service_checks=0;

time_t          program_start=0L;
int             program_mode=ACTIVE_MODE;
time_t          last_mode_change=0L;

service_message svc_msg;

contact		*contact_list=NULL;
contactgroup	*contactgroup_list=NULL;
host		*host_list=NULL;
hostgroup	*hostgroup_list=NULL;
service		*service_list=NULL;
timed_event     *event_list_high=NULL;
timed_event     *event_list_low=NULL;
notification    *notification_list=NULL;
command         *command_list=NULL;
timeperiod      *timeperiod_list=NULL;





int main(int argc, char **argv){
	int result;
	int error;
	char buffer[MAX_INPUT_BUFFER];

	error=FALSE;

	/* make sure we have the correct number of command line arguments */
	if(argc<2||argc>3)
		error=TRUE;

	/* we're starting monitoring... */
	if(argc==2){

		/* get the name of the main configuration file */
		strncpy(config_file,argv[1],sizeof(config_file));
		config_file[sizeof(config_file)-1]='\x0';
	        }

	/* we're either just verifying the configuration of starting in daemon mode... */
	else if(argc==3){

		/* check command line options... */
		if(!strcmp(argv[1],"-d"))
			daemon_mode=TRUE;
		else if(!strcmp(argv[1],"-v"))
			verify_config=TRUE;
		else
			error=TRUE;

		/* get the name of the main configuration file */
		strncpy(config_file,argv[2],sizeof(config_file));
		config_file[sizeof(config_file)-1]='\x0';
	        }

	if(daemon_mode==FALSE){
		printf("\nNetSaint %s\n",PROGRAM_VERSION);
		printf("Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: %s\n",PROGRAM_MODIFICATION_DATE);
		printf("License: GPL\n\n");
	        }

	/* if there are no command line options (or if we encountered an error), print usage */
	if(error==TRUE){
		printf("Usage: %s [-v | -d] <main_config_file>\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf("\n");
		printf("  -v  Verify information in the configuration files.  NetSaint will *not* start\n");
		printf("      monitoring when you supply this option.\n");
		printf("\n");
		printf("  -d  Run NetSaint in daemon mode.\n");
		printf("\n");
		printf("Visit the NetSaint website at http://www.netsaint.org for information on known\n");
		printf("bugs, new releases, and online documentation.\n");
		printf("\n");
		return ERROR;
		}

	/* make sure the config file uses an absolute path */
	if(config_file[0]!='/'){

		/* save the name of the config file */
		strncpy(buffer,config_file,sizeof(buffer)-1);
		buffer[sizeof(buffer)-1]='\x0';

		/* get absolute path of current working directory */
		strcpy(config_file,"");
		getcwd(config_file,sizeof(config_file)-1);

		/* append a forward slash */
		strncat(config_file,"/",sizeof(config_file)-2);
		config_file[sizeof(config_file)-1]='\x0';

		/* append the config file to the path */
		strncat(config_file,buffer,sizeof(config_file)-strlen(config_file)-1);
		config_file[sizeof(config_file)-1]='\x0';
	        }

	do{

		/* reset program variables */
		reset_variables();

		/* get program (re)start time */
		program_start=time(NULL);

		/* read in the configuration files */
		result=read_main_config(config_file);

		/* there was a problem reading the config files */
		if(result!=OK){

			/* if the config filename looks fishy, warn the user */
			if(!strstr(config_file,"netsaint.cfg")){
				printf("\nNote: Make sure you are specifying the name of the *main* configuration file\n");
				printf("      on the command line (usually 'netsaint.cfg').\n\n");
		                }
		        }

		/* there were no errors detected when reading the config files... */
		else{

			/* if we are just checking the config... */
			if(verify_config==TRUE){

				/* run the pre-fligh check to make sure things look okay... */
				result=pre_flight_check();

				if(result==OK)
					printf("\nThings look okay - No obvious problems were detected during the pre-flight check\n");
				else
					printf("\n****> Whoa! One or more problems were detected while running the pre-flight check <****\n");

				printf("\nDone verifying configuration.\n");
		                }

			/* else start monitoring... */
			else{

				snprintf(buffer,sizeof(buffer),"NetSaint %s starting... (PID=%d)\n",PROGRAM_VERSION,(int)getpid());
				buffer[sizeof(buffer)-1]='\x0';
				write_to_log(buffer);
				write_to_syslog(buffer);

				/* remove the old service status log if present */
				remove(status_file);

				/* run the pre-flight check to make sure everything looks okay*/
				result=pre_flight_check();

				/* the config data looks okay, so start to monitor... */
				if(result==OK){

					/* initialize IPC message queue */
					ipc_message_queue=init_message_queue();
					if(ipc_message_queue==-1){
						snprintf(buffer,sizeof(buffer),"Error: Could not initialize IPC message queue!\n");
						buffer[sizeof(buffer)-1]='\x0';
						write_to_log(buffer);
						write_to_syslog(buffer);
						return ERROR;
					        }

				        /* handle signals (interrupts) */
					setup_sighandler();

				        /* calculate the inter-check delay to use when initially scheduling services */
					calculate_inter_check_delay();

					/* calculate interleave factor to use for spreading out service checks */
					calculate_interleave_factor();

				        /* initialize the event timing loop */
					init_timing_loop();

				        /* create the initial status log */
					create_initial_status_log();

					/* enter daemon mode (unless we're restarting...) */
					if(daemon_mode==TRUE && sigrestart==FALSE){
#ifdef DEBUG2
						printf("$0: Cannot enter daemon mode with DEBUG2 option enabled.\n");
#else
						daemon_init();

						snprintf(buffer,sizeof(buffer),"Finished daemonizing... (New PID=%d)\n",(int)getpid());
						buffer[sizeof(buffer)-1]='\x0';
						write_to_log(buffer);
						write_to_syslog(buffer);
#endif
					        }

					/* enter the proper program mode */
					set_program_mode(program_mode);

					/* clean up the comment file */
					clean_comment_file();

					/* reset the restart flag */
					sigrestart=FALSE;

				        /* start monitoring all services */
					event_execution_loop();

					/* looks like we're shutting down... */
					if(sigrestart==FALSE){

						/* make sure lock file has been removed - it may not have been if we received a shutdown command */
						if(daemon_mode==TRUE)
							unlink(LOCKFILE);

						snprintf(buffer,sizeof(buffer),"Successfully shutdown... (PID=%d)\n",(int)getpid());
						buffer[sizeof(buffer)-1]='\x0';
						write_to_log(buffer);
						write_to_syslog(buffer);
					        }
				        }

				/* there was a problem running the pre-flight check */
				else{
					snprintf(buffer,sizeof(buffer),"Error: A problem was detected while running the pre-flight check\n");
					buffer[sizeof(buffer)-1]='\x0';
					write_to_log(buffer);
					write_to_syslog(buffer);
					snprintf(buffer,sizeof(buffer),"Bailing out due to errors...\n");
					buffer[sizeof(buffer)-1]='\x0';
					write_to_log(buffer);
					write_to_syslog(buffer);				        
				        }
	                        }
		        }

		/* clean up after ourselves */
		cleanup();

	        }while(sigrestart==TRUE);


	return OK;
	}




/******************************************************************/
/************ EVENT SCHEDULING/HANDLING FUNCTIONS *****************/
/******************************************************************/


/* calculate the inter check delay to use when initially scheduling service checks */
void calculate_inter_check_delay(void){
	service *temp_service;
	int total_services=0;
	unsigned long check_interval_total=0L;
	double average_check_interval=0.0L;
	double average_inter_check_delay=0.0L;

#ifdef DEBUG0
	printf("calculate_inter_check_delay() start\n");
#endif

	/* how should we determine the inter-check delay to use? */
	switch(inter_check_delay_method){

	case ICD_NONE:

		/* don't spread checks out - useful for testing parallelization code */
		inter_check_delay=0.0;
		break;

	case ICD_DUMB:

		/* be dumb and just schedule checks 1 second apart */
		inter_check_delay=1.0;
		break;

	case ICD_SMART:
	default:

		/* be smart and calculate the best delay to use to minimize CPU load... */

		for(temp_service=service_list,total_services=0;temp_service!=NULL;temp_service=temp_service->next,total_services++)
			check_interval_total+=temp_service->check_interval;

		if(total_services==0 || check_interval_total==0)
			return;

		/* adjust the check interval total to correspond to the interval length */
		check_interval_total=(check_interval_total*interval_length);

		/* calculate the average check interval for services */
		average_check_interval=(double)((double)check_interval_total/(double)total_services);

		/* calculate the average inter check delay (in seconds) needed to evenly space the service checks out */
		average_inter_check_delay=(double)(average_check_interval/(double)total_services);

		/* set the global inter check delay value */
		inter_check_delay=average_inter_check_delay;

#ifdef DEBUG1
		printf("\tTotal service checks:    %d\n",total_services);
		printf("\tCheck interval total:    %lu\n",check_interval_total);
		printf("\tAverage check interval:  %0.2f sec\n",average_check_interval);
		printf("\tInter-check delay:       %0.2f sec\n",inter_check_delay);
#endif
	        }


#ifdef DEBUG0
	printf("calculate_inter_check_delay() end\n");
#endif

	return;
        }


/* calculate the interleave factor for spreading out service checks */
void calculate_interleave_factor(void){
	int total_services=0;
	int total_hosts=0;
	host *temp_host=NULL;
	service *temp_service=NULL;

#ifdef DEBUG0
	printf("calculate_interleave_factor() start\n");
#endif

	/* how should we determine the interleave factor? */
	switch(interleave_factor_method){

	case ILF_USER:

		/* the user supplied a value, so don't do any calculation */
		break;


	case ILF_SMART:
	default:


		/* count the number of service we have */
		for(total_services=0,temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next,total_services++);

		/* count the number of hosts we have */
		for(total_hosts=0,temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next,total_hosts++);

		/* protect against a divide by zero problem - shouldn't happen, but just in case... */
		if(total_hosts==0)
			total_hosts=1;

		interleave_factor=(int)(ceil((double)total_services/(double)total_hosts));
#ifdef DEBUG1
		printf("\tTotal service checks:    %d\n",total_services);
		printf("\tTotal hosts:             %d\n",total_hosts);
		printf("\tInterleave factor:       %d\n",interleave_factor);
#endif
	        }

#ifdef DEBUG0
	printf("calculate_interleave_factor() end\n");
#endif

	return;
        }


/* initialize the event timing loop before we start monitoring */
void init_timing_loop(void){
	service *temp_service;
	time_t current_time;
	timed_event *new_event;
	time_t run_time;
	int total_services=0;
	int total_interleave_blocks=0;
	int current_interleave_block=1;
	int interleave_block_index=0;
	int mult_factor;

#ifdef DEBUG0
	printf("init_timing_loop() start\n");
#endif

	/* get the time right now */
	time(&current_time);

	/* count total number of services */
	for(total_services=0,temp_service=service_list;temp_service!=NULL;total_services++,temp_service=temp_service->next);

	/* calculate number of interleave blocks */
	total_interleave_blocks=(int)ceil((double)total_services/(double)interleave_factor);

	/* add all service checks as separate events (with interleaving) */
	for(current_interleave_block=1,temp_service=service_list;temp_service!=NULL;current_interleave_block++){

		for(interleave_block_index=0;interleave_block_index<interleave_factor;interleave_block_index++,temp_service=temp_service->next){

			if(temp_service==NULL)
				break;

			/*mult_factor=interleave_block_index+current_interleave_block+(interleave_factor*interleave_block_index);*/
			mult_factor=current_interleave_block+(interleave_block_index*total_interleave_blocks);

			/* set the next check time for the service */
			temp_service->next_check=current_time+(mult_factor*inter_check_delay);

			/* create a new event */
			new_event=malloc(sizeof(timed_event));
			if(new_event!=NULL){
				new_event->event_type=EVENT_SERVICE_CHECK;
				new_event->event_data=(void *)temp_service;
				new_event->run_time=temp_service->next_check;

				/* service checks really are recurring, but are rescheduled manually - not automatically */
				new_event->recurring=FALSE;

				schedule_event(new_event,&event_list_low);
		                }
		        }

	        }

	/* add a zombie process cleanup event */
	new_event=malloc(sizeof(timed_event));
	if(new_event!=NULL){
		new_event->event_type=EVENT_ZOMBIE_CLEANUP;
		new_event->event_data=(void *)NULL;
		new_event->run_time=current_time;
		new_event->recurring=TRUE;
		schedule_event(new_event,&event_list_high);
	        }

	/* add a service check reaper event */
	new_event=malloc(sizeof(timed_event));
	if(new_event!=NULL){
		new_event->event_type=EVENT_SERVICE_REAPER;
		new_event->event_data=(void *)NULL;
		new_event->run_time=current_time;
		new_event->recurring=TRUE;
		schedule_event(new_event,&event_list_high);
	        }

	/* add an external command check event if needed */
	if(check_external_commands){
		new_event=malloc(sizeof(timed_event));
		if(new_event!=NULL){
			new_event->event_type=EVENT_COMMAND_CHECK;
			new_event->event_data=(void *)NULL;
			new_event->run_time=current_time;
			new_event->recurring=TRUE;
			schedule_event(new_event,&event_list_high);
		        }
	        }

	/* add a log rotation event if necessary */
	if(log_rotation_method!=LOG_ROTATION_NONE){

		/* get the next time to run the log rotation */
		get_next_log_rotation_time(&run_time);

		new_event=malloc(sizeof(timed_event));
		if(new_event!=NULL){
			new_event->event_type=EVENT_LOG_ROTATION;
			new_event->event_data=(void *)NULL;
			new_event->run_time=run_time;
			new_event->recurring=TRUE;
			schedule_event(new_event,&event_list_high);
		        }
	        }

#ifdef DEBUG0
	printf("init_timing_loop() end\n");
#endif

	return;
        }


/* schedule an event in order of execution time */
void schedule_event(timed_event *event,timed_event **event_list){
	timed_event *temp_event;
	timed_event *first_event;
	service *temp_service;
	time_t run_time;

#ifdef DEBUG0
	printf("schedule_event() start\n");
#endif

	/* if this event is a service check... */
	if(event->event_type==EVENT_SERVICE_CHECK){
		temp_service=(service *)event->event_data;
		event->run_time=temp_service->next_check;
	        }

	/* if this is a zombie cleanup event ... */
	else if(event->event_type==EVENT_ZOMBIE_CLEANUP)
		event->run_time=event->run_time+(zombie_cleanup_interval*interval_length);

	/* if this is an external command check event ... */
	else if(event->event_type==EVENT_COMMAND_CHECK)
		event->run_time=event->run_time+(command_check_interval*interval_length);

	/* if this is a log rotation event... */
	else if(event->event_type==EVENT_LOG_ROTATION){
		get_next_log_rotation_time(&run_time);
		event->run_time=run_time;
	        }

	/* if this is a service check reaper event... */
	else if(event->event_type==EVENT_SERVICE_REAPER)
		event->run_time=event->run_time+service_check_reaper_interval;

	/* if this is a scheduled program shutdown or restart, we already know the time... */

	event->next=NULL;

	first_event=*event_list;

	/* add the event to the head of the list if there are no other events */
	if(*event_list==NULL)
		*event_list=event;

        /* add event to head of the list if it should be executed first */
	else if(event->run_time < first_event->run_time){
	        event->next=*event_list;
		*event_list=event;
	        }

	/* else place the event according to next execution time */
	else{

		temp_event=*event_list;
		while(temp_event!=NULL){

			if(temp_event->next==NULL){
				temp_event->next=event;
				break;
			        }
			
			if(event->run_time < temp_event->next->run_time){
				event->next=temp_event->next;
				temp_event->next=event;
				break;
			        }

			temp_event=temp_event->next;
		        }
	        }

#ifdef DEBUG0
	printf("schedule_event() end\n");
#endif

	return;
        }



/* remove an event from the queue */
void remove_event(timed_event *event,timed_event **event_list){
	timed_event *temp_event;

#ifdef DEBUG0
	printf("remove_event() start\n");
#endif

	if(*event_list==NULL)
		return;

	if(*event_list==event)
		*event_list=event->next;

	else{

		for(temp_event=*event_list;temp_event!=NULL;temp_event=temp_event->next){
			if(temp_event->next==event){
				temp_event->next=temp_event->next->next;
				event->next=NULL;
				break;
			        }
		        }
	        }

#ifdef DEBUG0
	printf("remove_event() end\n");
#endif

	return;
        }



/* this is the main event handler loop */
int event_execution_loop(void){
	timed_event *temp_event;
	time_t last_time;
	time_t current_time;

#ifdef DEBUG0
	printf("event_execution_loop() start\n");
#endif

	time(&last_time);

	while(1){

		/* see if we should exit or restart (a signal was encountered) */
		if(sigshutdown==TRUE || sigrestart==TRUE)
			return OK;

		/* if we don't have any events to handle, exit */
		if(event_list_high==NULL && event_list_low==NULL){
#ifdef DEBUG1
			printf("There aren't any events that need to be handled!\n");
#endif
			return OK;
	                }

		/* get the current time */
		time(&current_time);

		/* hey, wait a second...  we traveled back in time! */
		if(current_time<last_time)
			compensate_for_system_time_change((unsigned long)last_time,(unsigned long)current_time);

		/* else if the time advanced by at least an hour (mabye its DST)... */
		else if((current_time-last_time)>=3600)
			compensate_for_system_time_change((unsigned long)last_time,(unsigned long)current_time);

		/* keep track of the last time */
		last_time=current_time;

#ifdef DEBUG3
		printf("\n");
		printf("*** Event Check Loop ***\n");
		printf("\tCurrent time: %s",ctime(&current_time));
		if(event_list_high!=NULL)
			printf("\tNext High Priority Event Time: %s",ctime(&event_list_high->run_time));
		else
			printf("\tNo high priority events are scheduled...\n");
		if(event_list_low!=NULL)
			printf("\tNext Low Priority Event Time:  %s",ctime(&event_list_low->run_time));
		else
			printf("\tNo low priority events are scheduled...\n");
#endif

		/* handle high priority events */
		if(event_list_high!=NULL && (current_time>=event_list_high->run_time)){

			/* remove the first event from the timing loop */
			temp_event=event_list_high;
			event_list_high=event_list_high->next;

			/* handle the event */
			handle_timed_event(temp_event);

			/* reschedule the event if necessary */
			if(temp_event->recurring==TRUE)
				schedule_event(temp_event,&event_list_high);

			/* else free memory associated with the event */
			else
				free(temp_event);
		        }

		/* handle low priority events */
		else if(event_list_low!=NULL && (current_time>=event_list_low->run_time)){

			/* run the event except if it is a service check and we already are maxed out on number or parallel service checks... */
			if(!(event_list_low->event_type==EVENT_SERVICE_CHECK && currently_running_service_checks>=max_parallel_service_checks)){

				/* remove the first event from the timing loop */
				temp_event=event_list_low;
				event_list_low=event_list_low->next;

				/* handle the event */
				handle_timed_event(temp_event);

				/* reschedule the event if necessary */
				if(temp_event->recurring==TRUE)
					schedule_event(temp_event,&event_list_low);

				/* else free memory associated with the event */
				else
					free(temp_event);
			        }

			/* else wait a second so we don't hog the CPU... */
			else
				sleep(1);
		        }

		/* wait for a while */
		else
		        sleep((unsigned int)sleep_time);
	        }

#ifdef DEBUG0
	printf("event_execution_loop() end\n");
#endif

	return OK;
	}



/* handles a timed event */
int handle_timed_event(timed_event *event){
	service *temp_service;
	char temp_buffer[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("handle_timed_event() start\n");
#endif

#ifdef DEBUG3
	printf("*** Event Details ***\n");
	printf("\tEvent type: %d ",event->event_type);

	if(event->event_type==EVENT_SERVICE_CHECK){
		printf("(service check)\n");
		temp_service=(service *)event->event_data;
		printf("\t\tService Description: %s\n",temp_service->description);
		printf("\t\tAssociated Host:     %s\n",temp_service->host_name);
	        }

	else if(event->event_type==EVENT_ZOMBIE_CLEANUP)
		printf("(zombie cleanup)\n");

	else if(event->event_type==EVENT_COMMAND_CHECK)
		printf("(external command check)\n");

	else if(event->event_type==EVENT_LOG_ROTATION)
		printf("(log file rotation)\n");

	else if(event->event_type==EVENT_PROGRAM_SHUTDOWN)
		printf("(program shutdown)\n");

	else if(event->event_type==EVENT_PROGRAM_RESTART)
		printf("(program restart)\n");

	else if(event->event_type==EVENT_ENTER_ACTIVE_MODE)
		printf("(enter active mode)\n");

	else if(event->event_type==EVENT_ENTER_STANDBY_MODE)
		printf("(enter standby mode)\n");

	else if(event->event_type==EVENT_SERVICE_REAPER)
		printf("(service check reaper)\n");

	printf("\tEvent time: %s",ctime(&event->run_time));
#endif
		
	/* how should we handle the event? */
	switch(event->event_type){

	case EVENT_SERVICE_CHECK:

		/* run  a service check */
		temp_service=(service *)event->event_data;
		run_service_check(temp_service);

		break;

	case EVENT_ZOMBIE_CLEANUP:

		/* free orphaned child processes */
		clear_zombies();
		break;

	case EVENT_COMMAND_CHECK:

		/* check for external commands */
		check_for_external_commands();
		break;

	case EVENT_LOG_ROTATION:

		/* rotate the log file */
		rotate_log_file();
		break;

	case EVENT_PROGRAM_SHUTDOWN:
		
		/* set the shutdown flag */
		sigshutdown=TRUE;

		/* log the shutdown */
		snprintf(temp_buffer,sizeof(temp_buffer),"PROGRAM_SHUTDOWN event encountered, shutting down...\n");
		temp_buffer[sizeof(temp_buffer)-1]='\x0';
		write_to_log(temp_buffer);
		write_to_syslog(temp_buffer);
		break;

	case EVENT_PROGRAM_RESTART:

		/* set the restart flag */
		sigrestart=TRUE;

		/* log the restart */
		snprintf(temp_buffer,sizeof(temp_buffer),"PROGRAM_RESTART event encountered, restarting...\n");
		temp_buffer[sizeof(temp_buffer)-1]='\x0';
		write_to_log(temp_buffer);
		write_to_syslog(temp_buffer);
		break;

	case EVENT_ENTER_ACTIVE_MODE:

		/* enter active mode */
		set_program_mode(ACTIVE_MODE);
		break;

	case EVENT_ENTER_STANDBY_MODE:

		/* enter standy mode */
		set_program_mode(STANDBY_MODE);
		break;

	case EVENT_SERVICE_REAPER:

		/* reap service check results */
		reap_service_checks();
		break;

	default:

		break;
	        }

#ifdef DEBUG0
	printf("handle_timed_event() end\n");
#endif

	return OK;
        }



/* attempts to compensate for a change in the system time */
void compensate_for_system_time_change(unsigned long last_time,unsigned long current_time){
	char buffer[MAX_INPUT_BUFFER];
	unsigned long time_difference;
	timed_event *temp_event;
	service *temp_service;
	host *temp_host;

#ifdef DEBUG0
	printf("compensate_for_system_time_change() start\n");
#endif

	/* we moved back in time... */
	if(last_time>current_time)
		time_difference=last_time-current_time;

	/* we moved into the future... */
	else
		time_difference=current_time-last_time;

	/* log the time change */
	snprintf(buffer,sizeof(buffer),"A system time change of %lu seconds (%s in time) has been detected.  Compensating...\n",time_difference,(last_time>current_time)?"backwards":"forwards");
	buffer[sizeof(buffer)-1]='\x0';
	write_to_log(buffer);
	write_to_syslog(buffer);

	/* adjust the next run time for all high priority timed events */
	for(temp_event=event_list_high;temp_event!=NULL;temp_event=temp_event->next){

		/* we moved back in time... */
		if(last_time>current_time){

			/* we can't precede the UNIX epoch */
			if(time_difference>(unsigned long)temp_event->run_time)
				temp_event->run_time=(time_t)0;
			else
				temp_event->run_time=(time_t)(temp_event->run_time-(time_t)time_difference);
		        }

		/* we moved into the future... */
		else
			temp_event->run_time=(time_t)(temp_event->run_time+(time_t)time_difference);
	        }

	/* adjust the next run time for all low priority timed events */
	for(temp_event=event_list_low;temp_event!=NULL;temp_event=temp_event->next){

		/* we moved back in time... */
		if(last_time>current_time){

			/* we can't precede the UNIX epoch */
			if(time_difference>(unsigned long)temp_event->run_time)
				temp_event->run_time=(time_t)0;
			else
				temp_event->run_time=(time_t)(temp_event->run_time-(time_t)time_difference);
		        }

		/* we moved into the future... */
		else
			temp_event->run_time=(time_t)(temp_event->run_time+(time_t)time_difference);
	        }

	/* adjust the last notification time for all services */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		if(temp_service->last_notification==(time_t)0)
			continue;

		/* we moved back in time... */
		if(last_time>current_time){

			/* we can't precede the UNIX epoch */
			if(time_difference>(unsigned long)temp_service->last_notification)
				temp_service->last_notification=(time_t)0;
			else
				temp_service->last_notification=(time_t)(temp_service->last_notification-(time_t)time_difference);
		        }

		/* we moved into the future... */
		else
			temp_service->last_notification=(time_t)(temp_service->last_notification+(time_t)time_difference);
	        }

	/* adjust the last notification time for all hosts */
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		if(temp_host->last_host_notification==(time_t)0L)
			continue;

		/* we moved back in time... */
		if(last_time>current_time){

			/* we can't precede the UNIX epoch */
			if(time_difference>(unsigned long)temp_host->last_host_notification)
				temp_host->last_host_notification=(time_t)0;
			else
				temp_host->last_host_notification=(time_t)(temp_host->last_host_notification-(time_t)time_difference);
		        }

		/* we moved into the future... */
		else
			temp_host->last_host_notification=(time_t)(temp_host->last_host_notification+(time_t)time_difference);
	        }

#ifdef DEBUG0
	printf("compensate_for_system_time_change() end\n");
#endif

	return;
        }



/* attempts to clean up orphaned child processes that got left behind */
int clear_zombies(void){
	int cleared_zombies=0;
	char temp_buffer[MAX_INPUT_BUFFER];
	pid_t wait_result;
	int status;

#ifdef DEBUG0
	printf("clear_zombies() start\n");
#endif

#ifdef DEBUG1
	printf("\tTotal orphaned children: %d\n",total_orphaned_children);
#endif

	/* update last zombie cleanup time */
	last_zombie_cleanup=time(NULL);

	/* if we have child processes to kill... */
	if(total_orphaned_children>0){

		/* catch all zombie processes that were cleared... */
		while(1){

			/* try to catch a child process that has exited, but don't hang around.. */
			wait_result=waitpid(-1,&status,WNOHANG | WUNTRACED);

			/* there weren't any child processes that have exited, so break from the loop */
			if((int)wait_result==0)
				break;

			/* we freed a zombie! */
			else if((int)wait_result>0){

				cleared_zombies++;

				snprintf(temp_buffer,sizeof(temp_buffer),"Zombie cleanup cleared an orphaned child process with PID=%d!",(int)wait_result);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				write_to_log(temp_buffer);
			        }
	                }
	        }

	/* figure out how many orphaned child processes are still out there... */
	total_orphaned_children-=cleared_zombies;

	/* update the status log with new program information */
	update_status_log_with_program();

#ifdef DEBUG1
	printf("\tTotal cleared zombies: %d\n",cleared_zombies);
	printf("\tOutstanding orphaned children:   %d\n",total_orphaned_children);
#endif

#ifdef DEBUG0
	printf("clear_zombies() end\n");
#endif
	return cleared_zombies;
        }
