/*****************************************************************************
 *
 * OBJECTS.C - Object addition and search functions for NetSaint
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified:   02-18-2000
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netsaint.h"

extern contact		*contact_list;
extern contactgroup	*contactgroup_list;
extern host		*host_list;
extern hostgroup	*hostgroup_list;
extern service		*service_list;
extern timed_event      *event_list;
extern notification     *notification_list;
extern command          *command_list;
extern timeperiod       *timeperiod_list;



/******************************************************************/
/**************** OBJECT ADDITION FUNCTIONS ***********************/
/******************************************************************/


/* add a new timeperiod to the list in memory */
int add_timeperiod(char *name,char *value,int line){
	timeperiod *new_timeperiod;
	timeperiod *temp_timeperiod;
	timeperiod *last_timeperiod;
	timerange *new_timerange;
	char *day_range_start_buffer;
	char *range_buffer;
	char temp_period_buffer[MAX_TIMEPERIOD_LENGTH];
	char *temp_day_range_ptr;
	char *temp_range_ptr;
	char *temp_time_ptr;
	char *temp_buffer;
	char *temp_time_buffer;
	char *temp_alias;
	int hours;
	int minutes;
	unsigned long range_start_time;
	unsigned long range_end_time;
	int day;

#ifdef DEBUG0
	printf("add_timeperiod() start\n");
#endif

	/* make sure there isn't a timeperiod by this name added already */
	temp_timeperiod=NULL;
	temp_timeperiod=find_timeperiod(name,temp_timeperiod);
	if(temp_timeperiod!=NULL){
		printf("Error: Duplicate instance of timeperiod '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check the timeperiod name length */
	if(strlen(name)>MAX_TIMEPERIODNAME_LENGTH-1){
		printf("Error: Timeperiod name '%s' too long - Line %d\n",name,line);
		return ERROR;
	        }

	/* get the alias */
	temp_alias=my_strtok(value,";");
	if(temp_alias==NULL){
		printf("Error: Invalid alias/description for timeperiod - Line %d\n",line);
		return ERROR;
		}
	if(strlen(temp_alias)>MAX_TIMEPERIODALIAS_LENGTH-1){
		printf("Error: Timeperiod alias too long - Line %d\n",line);
		return ERROR;
		}

	/* get the time period day ranges */
	temp_buffer=my_strtok(NULL,"\n");
	if(temp_buffer==NULL){
		printf("Error: Invalid day ranges specified for timeperiod - Line %d\n",line);
		return ERROR;
	        }
	if(strlen(temp_buffer)>MAX_TIMEPERIOD_LENGTH-1){
		printf("Error: Timeperiod day ranges too long - Line %d\n",line);
		return ERROR;
	        }
	strcpy(temp_period_buffer,temp_buffer);
	strip(temp_period_buffer);

	/* allocate memory for the new timeperiod */
	new_timeperiod=malloc(sizeof(timeperiod));
	if(new_timeperiod==NULL){
		printf("Error: Could not allocate memory for timeperiod - Line %d\n",line);
		return ERROR;
	        }

	strip(name);
	strip(temp_alias);

	strcpy(new_timeperiod->name,name);
	strcpy(new_timeperiod->alias,temp_alias);

	/* initialize the time ranges for each day in the time period to NULL */
	for(day=0;day<7;day++)
		new_timeperiod->days[day]=NULL;

	/* parse time ranges for each day in the time period */
	day=0;
	for(temp_buffer=my_strtok(temp_period_buffer,";");temp_buffer!=NULL;temp_buffer=my_strtok(NULL,";"),day++){

		if(day>=7)
			break;

#ifdef DEBUG1
		printf("\nDay #%d\n",day);
#endif

		/* no time range found for today, range already initialized to NULL, so continue */
		if(!strcmp(temp_buffer,""))
			continue;

		temp_day_range_ptr=temp_buffer;
		for(day_range_start_buffer=my_strsep(&temp_day_range_ptr,",");day_range_start_buffer!=NULL;day_range_start_buffer=my_strsep(&temp_day_range_ptr,",")){

			temp_range_ptr=day_range_start_buffer;
			range_buffer=my_strsep(&temp_range_ptr,"-");
			if(range_buffer==NULL){
				printf("Error: Invalid timeperiod range for day #%d - Line %d\n",day,line);
				return ERROR;
			        }

			temp_time_ptr=range_buffer;
			temp_time_buffer=my_strsep(&temp_time_ptr,":");
			if(temp_time_buffer==NULL){
				printf("Error: Invalid timeperiod range for day #%d - Line %d\n",day,line);
				return ERROR;
			        }
			hours=atoi(temp_time_buffer);
			temp_time_buffer=my_strsep(&temp_time_ptr,":");
			if(temp_time_buffer==NULL){
				printf("Error: Invalid timeperiod range for day #%d - Line %d\n",day,line);
				return ERROR;
			        }
			minutes=atoi(temp_time_buffer);

			/* calculate the range start time in seconds */
			range_start_time=(unsigned long)((minutes*60)+(hours*60*60));
			
			range_buffer=my_strsep(&temp_range_ptr,"-");
			if(range_buffer==NULL){
				printf("Error: Invalid timeperiod range for day #%d - Line %d\n",day,line);
				return ERROR;
			        }

			temp_time_ptr=range_buffer;
			temp_time_buffer=my_strsep(&temp_time_ptr,":");
			if(temp_time_buffer==NULL){
				printf("Error: Invalid timeperiod range for day #%d - Line %d\n",day,line);
				return ERROR;
			        }
			hours=atoi(temp_time_buffer);

			temp_time_buffer=my_strsep(&temp_time_ptr,":");
			if(temp_time_buffer==NULL){
				printf("Error: Invalid timeperiod range for day #%d - Line %d\n",day,line);
				return ERROR;
			        }
			minutes=atoi(temp_time_buffer);

			/* calculate the range end time in seconds */
			range_end_time=(unsigned long)((minutes*60)+(hours*60*60));

			/* allocate memory for the new time range */
			new_timerange=malloc(sizeof(timerange));
			if(new_timerange==NULL){
				printf("Error: Could not allocate memory for time range - Line %d\n",line);
				return ERROR;
			        }

			new_timerange->range_start=range_start_time;
			new_timerange->range_end=range_end_time;

			/* add the new time range to the head of the range list for this day */
			new_timerange->next=new_timeperiod->days[day];
			new_timeperiod->days[day]=new_timerange;

#ifdef DEBUG1
			printf("\tTime Range: %lu - %lu\n",new_timerange->range_start,new_timerange->range_end);
#endif

		        }
		
	        }

	/* add new timeperiod to command list, sorted by name */
	last_timeperiod=timeperiod_list;
	for(temp_timeperiod=timeperiod_list;temp_timeperiod!=NULL;temp_timeperiod=temp_timeperiod->next){
		if(strcmp(new_timeperiod->name,temp_timeperiod->name)<0){
			new_timeperiod->next=temp_timeperiod;
			if(temp_timeperiod==timeperiod_list)
				timeperiod_list=new_timeperiod;
			else
				last_timeperiod->next=new_timeperiod;
			break;
		        }
		else
			last_timeperiod=temp_timeperiod;
	        }
	if(timeperiod_list==NULL){
		new_timeperiod->next=NULL;
		timeperiod_list=new_timeperiod;
	        }
	else if(temp_timeperiod==NULL){
		new_timeperiod->next=NULL;
		last_timeperiod->next=new_timeperiod;
	        }
		
#ifdef DEBUG1
	printf("\tName:         %s\n",new_timeperiod->name);
	printf("\tAlias:        %s\n",new_timeperiod->alias);
#endif
#ifdef DEBUG0
	printf("add_timeperiod() end\n");
#endif

	return OK;
        }


/* add a new host to the list in memory */
int add_host(char *name,char *value,int line){
	host *temp_host;
	host *new_host;
	host *last_host;
	char *temp_alias;
        char *temp_address;
	char *temp_host_check;
	char *temp_event_handler;
	char *temp_buffer;
	char *temp_parent_host;
	char *temp_notification_period;
	int temp_max_attempts;
	int temp_notify_up;
	int temp_notify_down;
	int temp_notify_unreachable;
	int notification_interval;

#ifdef DEBUG0
	printf("add_host() start\n");
#endif

	/* make sure there isn't a host by this name already added */
	temp_host=NULL;
	temp_host=find_host(name,temp_host);
	if(temp_host!=NULL){
		printf("Error: Duplicate instance of host name '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check name length ... */
	if(strlen(name)>MAX_HOSTNAME_LENGTH-1){
		printf("Error: Host name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* grab the host alias name */
	temp_alias=my_strtok(value,";");

	/* if no alias, use the host name */
	if(temp_alias==NULL)
		temp_alias=name;

	/* check the alias length */
	if(strlen(temp_alias)>MAX_HOSTNAME_LENGTH-1){
		printf("Error: Host alias '%s' too long - Line %d\n",temp_alias,line);
		return ERROR;
		}

	/* grab the host address */
	temp_address=my_strtok(NULL,";");

	
	/* if no address, return error */
	if(temp_address==NULL){
		printf("Error: No host address specified - Line %d\n",line);
		return ERROR;
		}

	if(strlen(temp_address)>MAX_ADDRESS_LENGTH-1){
		printf("Error: Host address '%s' too long - Line %d\n",temp_address,line);
		return ERROR;
		}

	/* get the parent host (if any) - no parent host means this host is on the local net */
	temp_parent_host=my_strtok(NULL,";");
	if(temp_parent_host==NULL)
		temp_parent_host="";

	/* check the parent host name length */
	if(strlen(temp_parent_host)>MAX_HOSTNAME_LENGTH-1){
		printf("Error: Parent host name '%s' too long - Line %d\n",temp_parent_host,line);
		return ERROR;
	        }

	/* grab the host check command */
	temp_host_check=my_strtok(NULL,";");
	
	/* no command means we won't check the host */
	if(temp_host_check==NULL)
		temp_host_check="";

	/* check the length of the host check command */
	if(strlen(temp_host_check)>MAX_COMMANDNAME_LENGTH){
		printf("Error: Host check command '%s' too long - Line %d\n",temp_host_check,line);
		return ERROR;
	        }

	/* get the max number of check attempts */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid max_attempts value for host - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_max_attempts=atoi(temp_buffer);
	if(temp_max_attempts<=0){
		printf("Error: Invalid max_attempts value for host - Line %d\n",line);
		return ERROR;
		}

	/* get the notification interval */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: NULL notification_interval value for host - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	notification_interval=atoi(temp_buffer);
	if(notification_interval<1){
		printf("Error: Invalid notification_interval value for host - Line %d\n",line);
		return ERROR;
		}

	/* get the notification timeperiod name */
	temp_notification_period=my_strtok(NULL,";");
	if(temp_notification_period==NULL)
		temp_notification_period="";
	if(strlen(temp_notification_period)>MAX_TIMEPERIODNAME_LENGTH-1){
		printf("Error: Host notification timeperiod name too long - Line %d\n",line);
		return ERROR;
		}

	/* get the notifcation on recovery option */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_on_recovery value for host - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_up=atoi(temp_buffer);
	if(temp_notify_up<0){
		printf("Error: Invalid notify_on_recovery value for host - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_up>1)
		temp_notify_up=1;

	/* get the notifcation on down option */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_on_down value for host - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_down=atoi(temp_buffer);
	if(temp_notify_down<0){
		printf("Error: Invalid notify_on_down value for host - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_down>1)
		temp_notify_down=1;

	/* get the notifcation on unreachable option */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_on_unreachable value for host - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_unreachable=atoi(temp_buffer);
	if(temp_notify_unreachable<0){
		printf("Error: Invalid notify_on_unreachable value for host - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_unreachable>1)
		temp_notify_unreachable=1;

	/* get the command name to run when the host state changes */
	temp_event_handler=my_strtok(NULL,";");
	if(temp_event_handler==NULL)
		temp_event_handler="";
	if(strlen(temp_event_handler)>MAX_COMMANDNAME_LENGTH-1){
		printf("Error: Host event handler command name line too long - Line %d\n",line);
		return ERROR;
		}

	/* allocate memory for a new host */
	new_host=malloc(sizeof(host));
	if(new_host==NULL){
		printf("Error: Could not allocate memory for host '%s'\n",name);
		return ERROR;
		}


	strip(name);
	strip(temp_alias);
	strip(temp_address);
	strip(temp_parent_host);
	strip(temp_host_check);
	strip(temp_event_handler);
	strip(temp_notification_period);

	strcpy(new_host->name,name);
	strcpy(new_host->alias,temp_alias);
	strcpy(new_host->address,temp_address);
	strcpy(new_host->parent_host,temp_parent_host);
	strcpy(new_host->host_check_command,temp_host_check);
	strcpy(new_host->event_handler,temp_event_handler);
	strcpy(new_host->notification_period,temp_notification_period);
	strcpy(new_host->plugin_output,"");
	new_host->status=HOST_UP;
	new_host->last_host_notification=0L;
	new_host->next_host_notification=0L;
	new_host->notification_interval=notification_interval;
	new_host->notify_on_recovery=temp_notify_up;
	new_host->notify_on_down=temp_notify_down;
	new_host->notify_on_unreachable=temp_notify_unreachable;
	new_host->notifications_disabled=FALSE;
	new_host->last_check=0L;
	new_host->max_attempts=temp_max_attempts;
	new_host->current_attempt=0;
	new_host->last_state_change=0L;
	new_host->time_up=0L;
	new_host->time_down=0L;
	new_host->time_unreachable=0L;

	/* add new host to host list, sorted by host name */
	last_host=host_list;
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
		if(strcmp(new_host->name,temp_host->name)<0){
			new_host->next=temp_host;
			if(temp_host==host_list)
				host_list=new_host;
			else
				last_host->next=new_host;
			break;
		        }
		else
			last_host=temp_host;
	        }
	if(host_list==NULL){
		new_host->next=NULL;
		host_list=new_host;
	        }
	else if(temp_host==NULL){
		new_host->next=NULL;
		last_host->next=new_host;
	        }
		

#ifdef DEBUG1
	printf("\tHost Name:                %s\n",new_host->name);
	printf("\tHost Alias:               %s\n",new_host->alias);
        printf("\tHost Address:             %s\n",new_host->address);
	printf("\tParent Host:              %s\n",new_host->parent_host);
	printf("\tHost Check Command:       %s\n",new_host->host_check_command);
	printf("\tMax. Check Attempts:      %d\n",new_host->max_attempts);
	printf("\tHost Event Handler:       %s\n",new_host->event_handler);
	printf("\tNotify On Down:           %s\n",(new_host->notify_on_down==1)?"yes":"no");
	printf("\tNotify On Unreachable:    %s\n",(new_host->notify_on_unreachable==1)?"yes":"no");
	printf("\tNotify On Recovery:       %s\n",(new_host->notify_on_recovery==1)?"yes":"no");
	printf("\tNotification Interval:    %d\n",new_host->notification_interval);
	printf("\tNotification Time Period: %s\n",new_host->notification_period);
#endif
#ifdef DEBUG0
	printf("add_host() end\n");
#endif

	return OK;
	}


/* add a new host group to the list in memory */
int add_hostgroup(char *name,char *value,int line){
	hostgroup *temp_hostgroup;
	hostgroup *new_hostgroup;
	hostgroup *last_hostgroup;
	char *temp_contacts;
	char *temp_host;
	char *temp_alias;
	int result;

#ifdef DEBUG0
	printf("add_hostgroup() start\n");
#endif

	/* make sure a hostgroup by this name hasn't been added already */
	temp_hostgroup=NULL;
	temp_hostgroup=find_hostgroup(name,temp_hostgroup);
	if(temp_hostgroup!=NULL){
		printf("Error: Duplicate instance of hostgroup '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check the length of the hostgroup name */
	if(strlen(name)>MAX_HOSTGROUPNAME_LENGTH-1){
		printf("Error: Hostgroup name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* get the group alias */
	temp_alias=my_strtok(value,";");
	if(temp_alias==NULL){
		printf("Error: No alias specified for hostgroup '%s' - Line %d\n",name,line);
		return ERROR;
	        }

	/* check the length of the group alias */
	if(strlen(temp_alias)>MAX_HOSTGROUPALIAS_LENGTH-1){
		printf("Error: Alias name '%s' too long - Line %d\n",temp_alias,line);
		return ERROR;
	        }

	/* get the contact group */
	temp_contacts=my_strtok(NULL,";");
	if(temp_contacts==NULL){
		printf("Error: No contact groups specified for hostgroup '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check the length of the contact groups */
	if(strlen(temp_contacts)>MAX_CONTACTGROUPS_LENGTH-1){
		printf("Error: Contact group(s) '%s' too long - Line %d\n",temp_contacts,line);
		return ERROR;
		}


	/* allocate memory */
	new_hostgroup=malloc(sizeof(hostgroup));
	if(new_hostgroup==NULL){
		printf("Error: Could not allocate memory for hostgroup '%s'\n",name);
		return ERROR;
		}

	strip(name);
	strip(temp_contacts);
	strip(temp_alias);

	strcpy(new_hostgroup->group_name,name);
	strcpy(new_hostgroup->alias,temp_alias);
	strcpy(new_hostgroup->contact_groups,temp_contacts);

	new_hostgroup->members=NULL;

	/* add new hostgroup to hostgroup list, sorted by hostgroup name */
	last_hostgroup=hostgroup_list;
	for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
		if(strcmp(new_hostgroup->group_name,temp_hostgroup->group_name)<0){
			new_hostgroup->next=temp_hostgroup;
			if(temp_hostgroup==hostgroup_list)
				hostgroup_list=new_hostgroup;
			else
				last_hostgroup->next=new_hostgroup;
			break;
		        }
		else
			last_hostgroup=temp_hostgroup;
	        }
	if(hostgroup_list==NULL){
		new_hostgroup->next=NULL;
		hostgroup_list=new_hostgroup;
	        }
	else if(temp_hostgroup==NULL){
		new_hostgroup->next=NULL;
		last_hostgroup->next=new_hostgroup;
	        }
		
#ifdef DEBUG1
	printf("\tGroup name:     %s\n",new_hostgroup->group_name);
	printf("\tAlias:          %s\n",new_hostgroup->alias);
	printf("\tContact groups: %s\n",new_hostgroup->contact_groups);
#endif


	/* get the hosts */
	temp_host=my_strtok(NULL,",");
	while(temp_host!=NULL){

		/* add the hostgroup member */
		result=add_hostgroupmember(temp_host,new_hostgroup,line);
		if(result!=OK)
			return ERROR;

		temp_host=my_strtok(NULL,",");
		}

#ifdef DEBUG0
	printf("add_hostgroup() end\n");
#endif

	return OK;
	}


/* add a new member to a host group */
int add_hostgroupmember(char *name,hostgroup *hostgroupptr,int line){
	hostgroupmember *new_member;

#ifdef DEBUG0
	printf("add_hostgroupmember() start\n");
#endif

	/* check the length of the host name */
	if(strlen(name)>MAX_HOSTNAME_LENGTH-1){
		printf("Error: Host name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* allocate memory for a new member */
	new_member=malloc(sizeof(hostgroupmember));
	if(new_member==NULL){
		printf("Error: Could not allocate memory for host '%s' - Line %d\n",name,line);
		return ERROR;
	        }

	strcpy(new_member->host_name,name);
	strip(new_member->host_name);
	
	/* add the new member to the head of the member list */
	new_member->next=hostgroupptr->members;
	hostgroupptr->members=new_member;

#ifdef DEBUG0
	printf("add_hostgroupmember() end\n");
#endif

	return OK;
        }


/* add a new contact to the list in memory */
int add_contact(char *name,char *value,int line){
	contact *temp_contact;
	contact *new_contact;
	contact *last_contact;
	commandsmember *new_commandsmember;
	char *temp_service_commands;
	char *temp_host_commands;
	char *temp_command;
	char temp_svc_commands_buffer[MAX_CONTACTCOMMANDS_LENGTH];
	char temp_host_commands_buffer[MAX_CONTACTCOMMANDS_LENGTH];
	char *temp_email;
	char *temp_pager;
	char *temp_alias;
	char *temp_svc_notification_period;
	char *temp_host_notification_period;
	int temp_notify_service_recovery;
	int temp_notify_service_critical;
	int temp_notify_service_warning;
	int temp_notify_host_recovery;
	int temp_notify_host_down;
	int temp_notify_host_unreachable;
	char *temp_buffer;

#ifdef DEBUG0
	printf("add_contact() start\n");
#endif

	/* make sure there isn't a contact by this name already added */
	temp_contact=NULL;
	temp_contact=find_contact(name,temp_contact);
	if(temp_contact!=NULL){
		printf("Error: Duplicate instance of contact '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check name length ... */
	if(strlen(name)>MAX_CONTACTNAME_LENGTH-1){
		printf("Error: Contact name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* get the alias */
	temp_alias=my_strtok(value,";");

	/* if no alias, use name */
	if(temp_alias==NULL)
		temp_alias=name;

	/* check alias length ... */
	if(strlen(temp_alias)>MAX_CONTACTALIAS_LENGTH-1){
		printf("Error: Contact alias '%s' too long - Line %d\n",temp_alias,line);
		return ERROR;
		}

	/* get the service notification timeperiod name */
	temp_svc_notification_period=my_strtok(NULL,";");
	if(temp_svc_notification_period==NULL)
		temp_svc_notification_period="";
	if(strlen(temp_svc_notification_period)>MAX_TIMEPERIODNAME_LENGTH-1){
		printf("Error: Contact service notification timeperiod name too long - Line %d\n",line);
		return ERROR;
		}

	/* get the host notification timeperiod name */
	temp_host_notification_period=my_strtok(NULL,";");
	if(temp_host_notification_period==NULL)
		temp_host_notification_period="";
	if(strlen(temp_host_notification_period)>MAX_TIMEPERIODNAME_LENGTH-1){
		printf("Error: Contact host notification timeperiod name too long - Line %d\n",line);
		return ERROR;
		}

	/* get the notifcation on recovery option for services */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_service_recovery value for contact - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_service_recovery=atoi(temp_buffer);
	if(temp_notify_service_recovery<0){
		printf("Error: Invalid notify_service_recovery value for contact - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_service_recovery>1)
		temp_notify_service_recovery=1;

	/* get the notifcation on critical option for services */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_service_critical value for contact - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_service_critical=atoi(temp_buffer);
	if(temp_notify_service_critical<0){
		printf("Error: Invalid notify_service_critical value for contact - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_service_critical>1)
		temp_notify_service_critical=1;

	/* get the notifcation on warning option for services */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_service_warning value for contact - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_service_warning=atoi(temp_buffer);
	if(temp_notify_service_warning<0){
		printf("Error: Invalid notify_service_warning value for contact - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_service_warning>1)
		temp_notify_service_warning=1;

	/* get the notifcation on recovery option for hosts */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_host_recovery value for contact - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_host_recovery=atoi(temp_buffer);
	if(temp_notify_host_recovery<0){
		printf("Error: Invalid notify_host_recovery value for contact - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_host_recovery>1)
		temp_notify_host_recovery=1;

	/* get the notifcation on down option for hosts */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_host_down value for contact - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_host_down=atoi(temp_buffer);
	if(temp_notify_host_down<0){
		printf("Error: Invalid notify_host_down value for contact - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_host_down>1)
		temp_notify_host_down=1;

	/* get the notifcation on unreachable option for hosts */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_host_unreachable value for contact - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_host_unreachable=atoi(temp_buffer);
	if(temp_notify_host_unreachable<0){
		printf("Error: Invalid notify_host_unreachable value for contact - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_host_unreachable>1)
		temp_notify_host_unreachable=1;

	/* get the service notification commands */
	temp_service_commands=my_strtok(NULL,";");
	
	/* check service commands length */
	if(strlen(temp_service_commands)>MAX_CONTACTCOMMANDS_LENGTH-1){
		printf("Error: Service notification commands for contact '%s' is too long - Line %d\n",name,line);
		return ERROR;
	        }
	strncpy(temp_svc_commands_buffer,temp_service_commands,sizeof(temp_svc_commands_buffer)-1);
	temp_svc_commands_buffer[sizeof(temp_svc_commands_buffer)-1]='\x0';


	/* get the host notification commands */
	temp_host_commands=my_strtok(NULL,";");

	/* check host commands length */
	if(strlen(temp_host_commands)>MAX_CONTACTCOMMANDS_LENGTH-1){
		printf("Error: Host notification commands for contact '%s' is too long - Line %d\n",name,line);
		return ERROR;
	        }
	strncpy(temp_host_commands_buffer,temp_host_commands,sizeof(temp_host_commands_buffer)-1);
	temp_host_commands_buffer[sizeof(temp_host_commands_buffer)-1]='\x0';


	/* grab the email address */
	temp_email=my_strtok(NULL,";");

	if(temp_email==NULL)
		temp_email="";

	/* check the email address length */
	if(strlen(temp_email)>MAX_EMAIL_LENGTH-1){
		printf("Error: Email address '%s' too long - Line %d\n",temp_email,line);
		return ERROR;
		}

	/* grab the pager number */
	temp_pager=my_strtok(NULL,";");

	if(temp_pager==NULL)
		temp_pager="";

	if(strlen(temp_pager)>MAX_PAGER_LENGTH-1){
		printf("Error: Pager number '%s' too long - Line %d\n",temp_pager,line);
		return ERROR;
		}

	/* error if the contact has no email AND no pager */
	if(!strcmp(temp_pager,"")&&!strcmp(temp_email,"")){
		printf("Error: Contact '%s' has no contact information - Line %d\n",name,line);
		return ERROR;
	        }

	/* allocate memory for a new contact */
	new_contact=malloc(sizeof(contact));
	if(new_contact==NULL){
		printf("Error: Could not allocate memory for contact '%s'\n",name);
		return ERROR;
		}

	strip(name);
	strip(temp_email);
	strip(temp_pager);
	strip(temp_alias);
	strip(temp_service_commands);
	strip(temp_host_commands);
	strip(temp_svc_notification_period);
	strip(temp_host_notification_period);
	strcpy(new_contact->name,name);
	strcpy(new_contact->email,temp_email);
	strcpy(new_contact->pager,temp_pager);
	strcpy(new_contact->alias,temp_alias);
	strcpy(new_contact->service_notification_period,temp_svc_notification_period);
	strcpy(new_contact->host_notification_period,temp_host_notification_period);
	new_contact->notify_on_service_recovery=temp_notify_service_recovery;
	new_contact->notify_on_service_critical=temp_notify_service_critical;
	new_contact->notify_on_service_warning=temp_notify_service_warning;
	new_contact->notify_on_host_recovery=temp_notify_host_recovery;
	new_contact->notify_on_host_down=temp_notify_host_down;
	new_contact->notify_on_host_unreachable=temp_notify_host_unreachable;

	/* add service notification commands */
	new_contact->service_notification_commands=NULL;
	for(temp_command=my_strtok(temp_svc_commands_buffer,",");temp_command!=NULL;temp_command=my_strtok(NULL,",")){
		new_commandsmember=malloc(sizeof(commandsmember));
		if(new_commandsmember!=NULL){
			new_commandsmember->next=new_contact->service_notification_commands;
			new_contact->service_notification_commands=new_commandsmember;
			strncpy(new_commandsmember->command,temp_command,MAX_COMMANDNAME_LENGTH);
			new_commandsmember->command[MAX_COMMANDNAME_LENGTH-1]='\x0';
		        }
	        }

	/* add host notification commands */
	new_contact->host_notification_commands=NULL;
	for(temp_command=my_strtok(temp_host_commands_buffer,",");temp_command!=NULL;temp_command=my_strtok(NULL,",")){
		new_commandsmember=malloc(sizeof(commandsmember));
		if(new_commandsmember!=NULL){
			new_commandsmember->next=new_contact->host_notification_commands;
			new_contact->host_notification_commands=new_commandsmember;
			strncpy(new_commandsmember->command,temp_command,MAX_COMMANDNAME_LENGTH);
			new_commandsmember->command[MAX_COMMANDNAME_LENGTH-1]='\x0';
		        }
	        }

	/* add new contact to contact list, sorted by contact name */
	last_contact=contact_list;
	for(temp_contact=contact_list;temp_contact!=NULL;temp_contact=temp_contact->next){
		if(strcmp(new_contact->name,temp_contact->name)<0){
			new_contact->next=temp_contact;
			if(temp_contact==contact_list)
				contact_list=new_contact;
			else
				last_contact->next=new_contact;
			break;
		        }
		else
			last_contact=temp_contact;
	        }
	if(contact_list==NULL){
		new_contact->next=NULL;
		contact_list=new_contact;
	        }
	else if(temp_contact==NULL){
		new_contact->next=NULL;
		last_contact->next=new_contact;
	        }

		
#ifdef DEBUG1
	printf("\tContact Name:                  %s\n",new_contact->name);
	printf("\tContact Alias:                 %s\n",new_contact->alias);
	printf("\tContact Email Address:         %s\n",new_contact->email);
	printf("\tContact Pager Address/Number:  %s\n",new_contact->pager);
	printf("\tSvc Notification Time Period:  %s\n",new_contact->service_notification_period);
	printf("\tHost Notification Time Period: %s\n",new_contact->host_notification_period);
#endif
#ifdef DEBUG0
	printf("add_contact() end\n");
#endif

	return OK;
	}


/* add a new contact group to the list in memory */
int add_contactgroup(char *name,char *value,int line){
	contactgroup *temp_contactgroup;
	contactgroup *new_contactgroup;
	contactgroup *last_contactgroup;
	char *temp_contact;
	char *temp_alias;
	int result;

#ifdef DEBUG0
	printf("add_contactgroup() start\n");
#endif

	/* make sure there isn't a contactgroup by this name added already */
	temp_contactgroup=NULL;
	temp_contactgroup=find_contactgroup(name,temp_contactgroup);
	if(temp_contactgroup!=NULL){
		printf("Error: Duplicate instance of contactgroup '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check the length of the contactgroup name */
	if(strlen(name)>MAX_CONTACTGROUPNAME_LENGTH-1){
		printf("Error: Contactgroup name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* get the group alias */
	temp_alias=my_strtok(value,";");
	if(temp_alias==NULL){
		printf("Error: No alias specified for contact group '%s' - Line %d\n",name,line);
		return ERROR;
	        }

	/* check the length of the group alias */
	if(strlen(temp_alias)>MAX_CONTACTGROUPALIAS_LENGTH-1){
		printf("Error: Alias name '%s' too long - Line %d\n",temp_alias,line);
		return ERROR;
	        }

	/* allocate memory for a new contactgroup entry */
	new_contactgroup=malloc(sizeof(contactgroup));
	if(new_contactgroup==NULL){
		printf("Error: Could not allocate memory for contactgroup '%s' - Line %d\n",name,line);
		return ERROR;
			}

	strip(name);
	strip(temp_alias);

	strcpy(new_contactgroup->group_name,name);
	strcpy(new_contactgroup->alias,temp_alias);

	new_contactgroup->members=NULL;

	/* add new contactgroup to contactgroup list, sorted by contactgroup name */
	last_contactgroup=contactgroup_list;
	for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){
		if(strcmp(new_contactgroup->group_name,temp_contactgroup->group_name)<0){
			new_contactgroup->next=temp_contactgroup;
			if(temp_contactgroup==contactgroup_list)
				contactgroup_list=new_contactgroup;
			else
				last_contactgroup->next=new_contactgroup;
			break;
		        }
		else
			last_contactgroup=temp_contactgroup;
	        }
	if(contactgroup_list==NULL){
		new_contactgroup->next=NULL;
		contactgroup_list=new_contactgroup;
	        }
	else if(temp_contactgroup==NULL){
		new_contactgroup->next=NULL;
		last_contactgroup->next=new_contactgroup;
	        }
		

#ifdef DEBUG1
	printf("\tGroup name:   %s\n",new_contactgroup->group_name);
	printf("\tAlias:        %s\n",new_contactgroup->alias);
#endif

	/* get the first contact */
	temp_contact=my_strtok(NULL,",");
	while(temp_contact!=NULL){

		/* add the contactgroup member */
		result=add_contactgroupmember(temp_contact,new_contactgroup,line);
		if(result!=OK)
			return ERROR;

		/* get the next contact */
		temp_contact=my_strtok(NULL,",");
		}

#ifdef DEBUG0
	printf("add_contactgroup() end\n");
#endif

	return OK;
	}


/* add a new member to a contact group */
int add_contactgroupmember(char *name,contactgroup *contactgroupptr,int line){
	contactgroupmember *new_member;

#ifdef DEBUG0
	printf("add_contactgroupmember() start\n");
#endif

	/* check the length of the contact name */
	if(strlen(name)>MAX_CONTACTNAME_LENGTH-1){
		printf("Error: Contact name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* allocate memory for a new member */
	new_member=malloc(sizeof(contactgroupmember));
	if(new_member==NULL){
		printf("Error: Could not allocate memory for contact '%s' - Line %d\n",name,line);
		return ERROR;
	        }

	strcpy(new_member->contact_name,name);
	strip(new_member->contact_name);
	
	/* add the new member to the head of the member list */
	new_member->next=contactgroupptr->members;
	contactgroupptr->members=new_member;

#ifdef DEBUG0
	printf("add_contactgroupmember() end\n");
#endif

	return OK;
        }


/* add a new service to the list in memory */
int add_service(char *name,char *value,int line){
	service *temp_service;
	service *new_service;
	service *last_service;
	char *temp_command;
	char *temp_event_handler;
	char *temp_description;
	char *temp_buffer;
	char *temp_notification_period;
	char *temp_check_period;
	char *temp_contactgroups;
	int temp_max_attempts;
	int temp_check_interval;
	int temp_retry_interval;
	int temp_notify_up;
	int temp_notify_down;
	int temp_notify_warning;
	int temp_notification_interval;

#ifdef DEBUG0
	printf("add_service() start\n");
#endif

	/* check the host name length */
	if(strlen(name)>MAX_HOSTNAME_LENGTH-1){
		printf("Error: Host name '%s' too long - Line %d\n",name,line);
		return ERROR;
		}

	/* get the service description */
	temp_description=my_strtok(value,";");
	if(temp_description==NULL){
		printf("Error: Service description missing - Line %d\n",line);
		return ERROR;
		}
	if(strlen(temp_description)>MAX_SERVICEDESC_LENGTH-1){
		printf("Error: Service description too long - Line %d\n",line);
		return ERROR;
		}

	/* make sure there isn't a service by this name added already */
	temp_service=NULL;
	temp_service=find_service(name,temp_description,temp_service);
	if(temp_service!=NULL){
		printf("Error: Duplicate instance of service '%s' for host '%s' - Line %d\n",temp_description,name,line);
		return ERROR;
		}

	/* get the check timeperiod name */
	temp_check_period=my_strtok(NULL,";");
	if(temp_check_period==NULL)
		temp_check_period="";
	if(strlen(temp_check_period)>MAX_TIMEPERIODNAME_LENGTH-1){
		printf("Error: Service check timeperiod name too long - Line %d\n",line);
		return ERROR;
		}

	/* get the max number of tries to check service status */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid max_attempts value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_max_attempts=atoi(temp_buffer);
	if(temp_max_attempts<=0){
		printf("Error: Invalid max_attempts value for service - Line %d\n",line);
		return ERROR;
		}

	/* get the check interval */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid check_interval value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_check_interval=atoi(temp_buffer);
	if(temp_check_interval<=0){
		printf("Error: Invalid check_interval value for service - Line %d\n",line);
		return ERROR;
		}

	/* get the retry interval */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid retry_interval value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_retry_interval=atoi(temp_buffer);
	if(temp_retry_interval<=0){
		printf("Error: Invalid retry_interval value for service - Line %d\n",line);
		return ERROR;
		}

	/* get the contact groups to be notified */
	temp_contactgroups=my_strtok(NULL,";");
	if(temp_contactgroups==NULL){
		printf("Error: No contact groups specified for service - Line %d\n",line);
		return ERROR;
		}
	if(strlen(temp_contactgroups)>MAX_CONTACTGROUPS_LENGTH-1){
		printf("Error: Contact group(s) '%s' too long - Line %d\n",temp_contactgroups,line);
		return ERROR;
		}

	/* get the notifcation interval */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notification_interval value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notification_interval=atoi(temp_buffer);
	/* if notification interval = 0, never notify a second time about service being down */
	if(temp_notification_interval<0){
		printf("Error: Invalid notification_interval value for service - Line %d\n",line);
		return ERROR;
		}

	/* get the notification timeperiod name */
	temp_notification_period=my_strtok(NULL,";");
	if(temp_notification_period==NULL)
		temp_notification_period="";
	if(strlen(temp_notification_period)>MAX_TIMEPERIODNAME_LENGTH-1){
		printf("Error: Service notification timeperiod name too long - Line %d\n",line);
		return ERROR;
		}

	/* get the notifcation on recovery option */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_on_recovery value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_up=atoi(temp_buffer);
	if(temp_notify_up<0){
		printf("Error: Invalid notify_on_recovery value for service - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_up>1)
		temp_notify_up=1;

	/* get the notifcation on critical (down) option */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_on_critical value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_down=atoi(temp_buffer);
	if(temp_notify_down<0){
		printf("Error: Invalid notify_on_critical value for service - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_down>1)
		temp_notify_down=1;

	/* get the notifcation on warning option */
	temp_buffer=my_strtok(NULL,";");
	if(temp_buffer==NULL){
		printf("Error: Invalid notify_on_warning value for service - Line %d\n",line);
		return ERROR;
		}
	strip(temp_buffer);
	temp_notify_warning=atoi(temp_buffer);
	if(temp_notify_warning<0){
		printf("Error: Invalid notify_on_warning value for service - Line %d\n",line);
		return ERROR;
		}
	if(temp_notify_warning>1)
		temp_notify_warning=1;

	/* get the command name to run when the service state changes */
	temp_event_handler=my_strtok(NULL,";");
	if(temp_event_handler==NULL)
		temp_event_handler="";
	if(strlen(temp_event_handler)>MAX_COMMANDNAME_LENGTH-1){
		printf("Error: Service event handler command name line too long - Line %d\n",line);
		return ERROR;
		}

	/* get the command name to run when checking this service */
	temp_command=my_strtok(NULL,";");
	if(temp_command==NULL){
		printf("Error: Command name is NULL - Line %d\n",line);
		return ERROR;
		}
	if(strlen(temp_command)>MAX_COMMANDLINE_LENGTH-1){
		printf("Error: Command line too long - Line %d\n",line);
		return ERROR;
		}

	/* allocate memory */
	new_service=malloc(sizeof(service));
	if(new_service==NULL){
		printf("Error: Could not allocate memory for service - Line %d\n",line);
		return ERROR;
		}

	strip(name);
	strip(temp_command);
	strip(temp_event_handler);
	strip(temp_description);
	strip(temp_notification_period);
	strip(temp_check_period);
	strip(temp_contactgroups);

	strcpy(new_service->host_name,name);
	strcpy(new_service->description,temp_description);
	strcpy(new_service->command,temp_command);
	strcpy(new_service->event_handler,temp_event_handler);
	strcpy(new_service->notification_period,temp_notification_period);
	strcpy(new_service->check_period,temp_check_period);
	strcpy(new_service->plugin_output,"");
	new_service->check_interval=temp_check_interval;
	new_service->retry_interval=temp_retry_interval;
	new_service->max_attempts=temp_max_attempts;
	new_service->current_attempt=1;
	new_service->current_state=STATE_OK;
	new_service->last_state=STATE_OK;
	new_service->last_hard_state=STATE_OK;
	new_service->state_type=SOFT_STATE;
	new_service->host_problem_at_last_check=FALSE;
	new_service->next_check=0L;
	new_service->last_check=0L;
	new_service->disabled=FALSE;
	strcpy(new_service->contact_groups,temp_contactgroups);
	new_service->notification_interval=temp_notification_interval;
	new_service->last_notification=0L;
	new_service->next_notification=0L;
	new_service->notify_on_warning=temp_notify_warning;
	new_service->notify_on_critical=temp_notify_down;
	new_service->notify_on_recovery=temp_notify_up;
	new_service->notifications_disabled=FALSE;
	new_service->last_state_change=0L;
	new_service->time_ok=0L;
	new_service->time_unknown=0L;
	new_service->time_warning=0L;
	new_service->time_critical=0L;

	/* add new service to service list, sorted by host name */
	last_service=service_list;
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
		if(strcmp(new_service->host_name,temp_service->host_name)<0){
			new_service->next=temp_service;
			if(temp_service==service_list)
				service_list=new_service;
			else
				last_service->next=new_service;
			break;
		        }
		else
			last_service=temp_service;
	        }
	if(service_list==NULL){
		new_service->next=NULL;
		service_list=new_service;
	        }
	else if(temp_service==NULL){
		new_service->next=NULL;
		last_service->next=new_service;
	        }
		

#ifdef DEBUG1
	printf("\tHost:                     %s\n",new_service->host_name);
	printf("\tDescription:              %s\n",new_service->description);
	printf("\tCommand:                  %s\n",new_service->command);
	printf("\tCheck Interval:           %d\n",new_service->check_interval);
	printf("\tRetry Interval:           %d\n",new_service->retry_interval);
	printf("\tMax attempts:             %d\n",new_service->max_attempts);
	printf("\tNotification Interval:    %d\n",new_service->notification_interval);
	printf("\tNotification Time Period: %s\n",new_service->notification_period);
	printf("\tNotify On Warning:        %s\n",(new_service->notify_on_warning==1)?"yes":"no");
	printf("\tNotify On Critical:       %s\n",(new_service->notify_on_critical==1)?"yes":"no");
	printf("\tNotify On Recovery:       %s\n",(new_service->notify_on_recovery==1)?"yes":"no");
#endif
#ifdef DEBUG0
	printf("add_service() end\n");
#endif

	return OK;
	}


/* add a new command to the list in memory */
int add_command(char *name,char *value,int line){
	command *new_command;
	command *temp_command;
	command *last_command;

#ifdef DEBUG0
	printf("add_command() start\n");
#endif

	/* make sure there isn't a command by this name added already */
	temp_command=NULL;
	temp_command=find_command(name,temp_command);
	if(temp_command!=NULL){
		printf("Error: Duplicate instance of command '%s' - Line %d\n",name,line);
		return ERROR;
		}

	/* check the command name length */
	if(strlen(name)>MAX_COMMANDNAME_LENGTH-1){
		printf("Error: Command name '%s' too long - Line %d\n",name,line);
		return ERROR;
	        }

	/* check the command line length */
	if(strlen(value)>MAX_COMMANDLINE_LENGTH-1){
		printf("Error: Command line '%s' too long - Line %d\n",value,line);
		return ERROR;
	        }

	/* allocate memory for the new command */
	new_command=malloc(sizeof(command));
	if(new_command==NULL){
		printf("Error: Could not allocate memory for command - Line %d\n",line);
		return ERROR;
	        }

	strip(name);
	strip(value);

	strcpy(new_command->name,name);
	strcpy(new_command->command_line,value);

	/* add new command to command list, sorted by command name */
	last_command=command_list;
	for(temp_command=command_list;temp_command!=NULL;temp_command=temp_command->next){
		if(strcmp(new_command->name,temp_command->name)<0){
			new_command->next=temp_command;
			if(temp_command==command_list)
				command_list=new_command;
			else
				last_command->next=new_command;
			break;
		        }
		else
			last_command=temp_command;
	        }
	if(command_list==NULL){
		new_command->next=NULL;
		command_list=new_command;
	        }
	else if(temp_command==NULL){
		new_command->next=NULL;
		last_command->next=new_command;
	        }
		
#ifdef DEBUG1
	printf("\tName:         %s\n",new_command->name);
	printf("\tCommand Line: %s\n",new_command->command_line);
#endif
#ifdef DEBUG0
	printf("add_command() end\n");
#endif

	return OK;
        }


/* add a new notification to the list in memory */
int add_notification(contact *cntct){
	notification *new_notification;
	notification *temp_notification;

#ifdef DEBUG0
	printf("add_notification() start\n");
#endif
#ifdef DEBUG1
	printf("\tAdd contact '%s'\n",cntct->name);
#endif

	/* don't add anything if this contact is already on the notification list */
	temp_notification=find_notification(cntct->name);
	if(temp_notification!=NULL)
		return OK;

	/* allocate memory for a new contact in the notification list */
	new_notification=malloc(sizeof(notification));
	if(new_notification==NULL)
		return ERROR;

	/* fill in the contact info */
	new_notification->contact=cntct;

	/* add new notification to head of list */
	new_notification->next=notification_list;
	notification_list=new_notification;

#ifdef DEBUG0
	printf("add_notification() end\n");
#endif

	return OK;
        }





/******************************************************************/
/******************** OBJECT SEARCH FUNCTIONS *********************/
/******************************************************************/

/* given a timeperiod name and a starting point, find a timeperiod from the list in memory */
timeperiod * find_timeperiod(char *name,timeperiod *timeptr){
	timeperiod *temp_timeperiod;

#ifdef DEBUG0
	printf("find_timeperiod() start\n");
#endif

	if(timeptr==NULL)
		temp_timeperiod=timeperiod_list;
	else
		temp_timeperiod=timeptr->next;
	while(temp_timeperiod!=NULL){

		/* we found a matching host */
       	        if(!strcmp(temp_timeperiod->name,name))
		       return temp_timeperiod;

		temp_timeperiod=temp_timeperiod->next;
		}

#ifdef DEBUG0
	printf("find_timeperiod() end\n");
#endif

	/* we couldn't find a matching timeperiod */
	return NULL;
	}


/* given a host name and a starting point, find a host from the list in memory */
host * find_host(char *name,host *hostptr){
	host *temp_host;

#ifdef DEBUG0
	printf("find_host() start\n");
#endif

	if(hostptr==NULL)
		temp_host=host_list;
	else
		temp_host=hostptr->next;
	while(temp_host!=NULL){

		/* we found a matching host */
       	        if(!strcmp(temp_host->name,name))
		       return temp_host;

		temp_host=temp_host->next;
		}

#ifdef DEBUG0
	printf("find_host() end\n");
#endif

	/* we couldn't find a matching host */
	return NULL;
	}


/* given a name and starting point, find a hostgroup from the list in memory */
hostgroup * find_hostgroup(char *name,hostgroup *hostgroupptr){
	hostgroup *temp_hostgroup;

#ifdef DEBUG0
	printf("find_hostgroup() start\n");
#endif

	if(hostgroupptr==NULL)
		temp_hostgroup=hostgroup_list;
	else
		temp_hostgroup=hostgroupptr->next;
	while(temp_hostgroup!=NULL){

		/* we found a matching hostgroup */
		if(!strcmp(temp_hostgroup->group_name,name))
			return temp_hostgroup;

		temp_hostgroup=temp_hostgroup->next;
		}

#ifdef DEBUG0
	printf("find_hostgroup() end\n");
#endif

	/* we couldn't find a matching hostgroup */
	return NULL;
	}


/* given a host name and a starting point, find a member of a host group */
hostgroupmember * find_hostgroupmember(char *name,hostgroup *hostgroupptr,hostgroupmember *memberptr){
	hostgroupmember *temp_member;

#ifdef DEBUG0
	printf("find_hostgroupmember() start\n");
#endif

	if(memberptr==NULL)
		temp_member=hostgroupptr->members;
	else
		temp_member=memberptr->next;
	while(temp_member!=NULL){

		/* we found a match */
		if(!strcmp(temp_member->host_name,name))
			return temp_member;

		temp_member=temp_member->next;
	        }
	

#ifdef DEBUG0
	printf("find_hostgroupmember() end\n");
#endif

	/* we couldn't find a matching member */
	return NULL;
        }


/* given a name and a starting point, find a contact from the list in memory */
contact * find_contact(char *name,contact *contactptr){
	contact *temp_contact;

#ifdef DEBUG0
	printf("find_contact() start\n");
#endif

	if(contactptr==NULL)
		temp_contact=contact_list;
	else
		temp_contact=contactptr->next;
	while(temp_contact!=NULL){

		/* we found a matching contact */
		if(!strcmp(temp_contact->name,name))
			return temp_contact;

		temp_contact=temp_contact->next;
		}

#ifdef DEBUG0
	printf("find_contact() end\n");
#endif

	/* we couldn't find a matching contact */
	return NULL;
	}


/* given a group name and a starting point, find a contact group from the list in memory */
contactgroup * find_contactgroup(char *name,contactgroup *contactgroupptr){
	contactgroup *temp_contactgroup;

#ifdef DEBUG0
	printf("find_contactgroup() start\n");
#endif

	if(contactgroupptr==NULL)
		temp_contactgroup=contactgroup_list;
	else
		temp_contactgroup=contactgroupptr->next;
	while(temp_contactgroup!=NULL){

		/* we found a match */
		if(!strcmp(temp_contactgroup->group_name,name))
			return temp_contactgroup;

		temp_contactgroup=temp_contactgroup->next;
		}


#ifdef DEBUG0
	printf("find_contactgroup() end\n");
#endif

	/* we couldn't find a matching contactgroup */
	return NULL;
	}


/* given a contact name and a starting point, find the corresponding member of a contact group */
contactgroupmember * find_contactgroupmember(char *name,contactgroup *contactgroupptr,contactgroupmember *memberptr){
	contactgroupmember *temp_member;

#ifdef DEBUG0
	printf("find_contactgroupmember() start\n");
#endif

	if(memberptr==NULL)
		temp_member=contactgroupptr->members;
	else
		temp_member=memberptr->next;
	while(temp_member!=NULL){

		/* we found a match */
		if(!strcmp(temp_member->contact_name,name))
			return temp_member;

		temp_member=temp_member->next;
	        }
	

#ifdef DEBUG0
	printf("find_contactgroupmember() end\n");
#endif

	/* we couldn't find a matching member */
	return NULL;
        }


/* given a command name, find a command from the list in memory */
command * find_command(char *name,command *commandptr){
	command *temp_command;

#ifdef DEBUG0
	printf("find_command() start\n");
#endif

	if(commandptr==NULL)
		temp_command=command_list;
	else
		temp_command=commandptr->next;
	while(temp_command!=NULL){

		/* we found a match */
		if(!strcmp(name,temp_command->name))
			return temp_command;

		temp_command=temp_command->next;
	        }

#ifdef DEBUG0
	printf("find_command() end\n");
#endif

	/* we couldn't find a matching command */
	return NULL;
        }



/* given a host/service name and a starting point, find a service from the list in memory */
service * find_service(char *host_name,char *svc_desc,service *svcptr){
	service *temp_service;

#ifdef DEBUG0
	printf("find_service() start\n");
#endif

	if(svcptr==NULL)
		temp_service=service_list;
	else
		temp_service=svcptr->next;
	while(temp_service!=NULL){

		/* we found a matching host name and service description */
       	        if(!strcmp(temp_service->host_name,host_name) && !strcmp(temp_service->description,svc_desc))
		       return temp_service;

		temp_service=temp_service->next;
		}

#ifdef DEBUG0
	printf("find_service() end\n");
#endif

	/* we couldn't find a matching service */
	return NULL;
	}



/* given a contact name, find the notification entry for them for the list in memory */
notification * find_notification(char *contact_name){
	notification *temp_notification;

#ifdef DEBUG0
	printf("find_notification() start\n");
#endif
	
	temp_notification=notification_list;
	while(temp_notification!=NULL){
		if(!strcmp(contact_name,temp_notification->contact->name))
		  return temp_notification;
		temp_notification=temp_notification->next;
	        }

#ifdef DEBUG0
	printf("find_notification() end\n");
#endif

	/* we couldn't find the contact in the notification list */
	return NULL;
        }
	



/******************************************************************/
/********************* OBJECT QUERY FUNCTIONS *********************/
/******************************************************************/

/* determines whether or not a specific host is an immediate child of another host */
int is_immediate_child_of_host(host *parent_host,host *child_host){

	if(parent_host==NULL){
		if(!strcmp(child_host->parent_host,""))
			return TRUE;
		else
			return FALSE;
	        }

	else if(!strcmp(child_host->parent_host,parent_host->name))
		return TRUE;

	return FALSE;
	}


