/***********************************************************************
 *
 * NetSaint CGI Utilities
 * 
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 01-10-2000
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ***********************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "../common/locations.h"

#include "cgiutils.h"
#include "popen.h"

char            main_config_file[MAX_FILENAME_LENGTH];
char            log_file[MAX_FILENAME_LENGTH];
char            log_archive_path[MAX_FILENAME_LENGTH];
char            status_file[MAX_FILENAME_LENGTH];
char            command_file[MAX_FILENAME_LENGTH];
char            comment_file[MAX_FILENAME_LENGTH];

char            physical_html_path[MAX_FILENAME_LENGTH];
char            physical_images_path[MAX_FILENAME_LENGTH];
char            url_html_path[MAX_FILENAME_LENGTH];
char            url_docs_path[MAX_FILENAME_LENGTH];
char            url_images_path[MAX_FILENAME_LENGTH];
char            url_logo_images_path[MAX_FILENAME_LENGTH];
char            url_cgibin_path[MAX_FILENAME_LENGTH];

char            html_table_bg_color[MAX_COLOR_LENGTH];
char            html_table_fg_color[MAX_COLOR_LENGTH];
char            html_table_bg_odd_color[MAX_COLOR_LENGTH];
char            html_table_bg_even_color[MAX_COLOR_LENGTH];

char            netsaint_check_command[MAX_INPUT_BUFFER];

char            netsaint_process_info[MAX_INPUT_BUFFER];
int             netsaint_process_state=STATE_OK;

time_t          program_start=0L;
int             program_mode=ACTIVE_MODE;
int             daemon_mode=FALSE;
time_t          last_mode_change=0L;

time_t          last_zombie_cleanup=0L;
int             orphaned_child_processes=0;

time_t          last_command_check=0L;

int             check_external_commands=0;

int             log_rotation_method=LOG_ROTATION_NONE;
time_t          last_log_rotation=0L;

time_t          this_scheduled_log_rotation=0L;
time_t          last_scheduled_log_rotation=0L;
time_t          next_scheduled_log_rotation=0L;

int             use_authentication=TRUE;

int             suppress_alert_window=FALSE;

int             interval_length=60;

int             hosts_have_been_read=FALSE;
int             hostgroups_have_been_read=FALSE;
int             contacts_have_been_read=FALSE;
int             contactgroups_have_been_read=FALSE;
int             services_have_been_read=FALSE;
int             timeperiods_have_been_read=FALSE;
int             commands_have_been_read=FALSE;

int             refresh_rate=DEFAULT_REFRESH_RATE;


hostgroup       *hostgroup_list=NULL;
hoststatus      *hoststatus_list=NULL;
host            *host_list=NULL;
servicestatus   *servicestatus_list=NULL;
service         *service_list=NULL;
hostextinfo     *hostextinfo_list=NULL;
comment         *comment_list=NULL;
contactgroup    *contactgroup_list=NULL;
command         *command_list=NULL;
timeperiod      *timeperiod_list=NULL;
contact         *contact_list=NULL;

char            *my_strtok_buffer=NULL;
char            *original_my_strtok_buffer=NULL;

char encoded_url_string[MAX_INPUT_BUFFER];

#ifdef HAVE_TZNAME
extern char     *tzname[2];
#endif




/**********************************************************
 ***************** CLEANUP FUNCTIONS **********************
 **********************************************************/

/* reset all variables used by the CGIs */
void reset_cgi_vars(void){

	strcpy(main_config_file,"");

	strcpy(url_html_path,"");
	strcpy(url_docs_path,"");
	strcpy(url_images_path,"");
	strcpy(url_cgibin_path,"");

	strcpy(log_file,"");
	strcpy(log_archive_path,DEFAULT_LOG_ARCHIVE_PATH);
	if(log_archive_path[strlen(log_archive_path)-1]!='/' && strlen(log_archive_path)<sizeof(log_archive_path)-2)
		strcat(log_archive_path,"/");
	strcpy(status_file,"");
	strcpy(command_file,DEFAULT_COMMAND_FILE);
	strcpy(comment_file,DEFAULT_COMMENT_FILE);

	strcpy(netsaint_check_command,"");
	strcpy(netsaint_process_info,"");
	netsaint_process_state=STATE_OK;

	hostgroup_list=NULL;
	hoststatus_list=NULL;
	servicestatus_list=NULL;
	hostextinfo_list=NULL;
	comment_list=NULL;
	contactgroup_list=NULL;
	command_list=NULL;
	timeperiod_list=NULL;
	contact_list=NULL;

	log_rotation_method=LOG_ROTATION_NONE;

	use_authentication=TRUE;

	suppress_alert_window=FALSE;

	interval_length=60;

	refresh_rate=DEFAULT_REFRESH_RATE;

	strcpy(html_table_bg_color,HTML_TABLE_BG_COLOR);
	strcpy(html_table_fg_color,HTML_TABLE_FG_COLOR);
	strcpy(html_table_bg_odd_color,HTML_TABLE_BG_ODD_COLOR);
	strcpy(html_table_bg_even_color,HTML_TABLE_BG_EVEN_COLOR);

	return;
        }


/* free all memory allocated by the CGIs */
void free_memory(void){

	free_hostgroup_memory();
	free_host_status_memory();
	free_host_memory();
	free_service_status_memory();
	free_service_memory();
	free_comment_memory();
	free_contactgroup_memory();
	free_command_memory();
	free_timeperiod_memory();
	free_contact_memory();

	return;
        }


/* free memory allocated to the hostgroup list */
void free_hostgroup_memory(void){
	hostgroup *this_hostgroup;
	hostgroup *next_hostgroup;

	/* free memory for the hostgroup list */
	this_hostgroup=hostgroup_list;
	while(this_hostgroup!=NULL){
		next_hostgroup=this_hostgroup->next;
		free(this_hostgroup);
		this_hostgroup=next_hostgroup;
		}

	return;
        }


/* free memory allocated to the contactgroup list */
void free_contactgroup_memory(void){
	contactgroup *this_contactgroup;
	contactgroup *next_contactgroup;

	/* free memory for the contactgroup list */
	this_contactgroup=contactgroup_list;
	while(this_contactgroup!=NULL){
		next_contactgroup=this_contactgroup->next;
		free(this_contactgroup);
		this_contactgroup=next_contactgroup;
		}

	return;
        }


/* free memory allocated to the host status list */
void free_host_status_memory(void){
	hoststatus *this_hoststatus;
	hoststatus *next_hoststatus;

	/* free memory for the host status list */
	this_hoststatus=hoststatus_list;
	while(this_hoststatus!=NULL){
		next_hoststatus=this_hoststatus->next;
		free(this_hoststatus);
		this_hoststatus=next_hoststatus;
	        }

	return;
        }


/* free memory allocated to the host list */
void free_host_memory(void){
	host *this_host;
	host *next_host;

	/* free memory for the host list */
	this_host=host_list;
	while(this_host!=NULL){
		next_host=this_host->next;
		free(this_host);
		this_host=next_host;
	        }

	return;
        }


/* free memory allocated to the service list */
void free_service_memory(void){
	service *this_service;
	service *next_service;

	/* free memory for the service list */
	this_service=service_list;
	while(this_service!=NULL){
		next_service=this_service->next;
		free(this_service);
		this_service=next_service;
	        }

	return;
        }


/* free memory allocated to the status list */
void free_service_status_memory(void){
	servicestatus *this_svcstatus;
	servicestatus *next_svcstatus;

	/* free memory for the service status list */
	this_svcstatus=servicestatus_list;
	while(this_svcstatus!=NULL){
		next_svcstatus=this_svcstatus->next;
		free(this_svcstatus);
		this_svcstatus=next_svcstatus;
	        }

	return;
        }


/* free memory allocated to the comment list */
void free_comment_memory(void){
	comment *this_comment;
	comment *next_comment;

	/* free memory for the comment list */
	this_comment=comment_list;
	while(this_comment!=NULL){
		next_comment=this_comment->next;
		free(this_comment);
		this_comment=next_comment;
	        }

	return;
        }


/* free memory allocated to the command list */
void free_command_memory(void){
	command *this_command;
	command *next_command;

	/* free memory for the command list */
	this_command=command_list;
	while(this_command!=NULL){
		next_command=this_command->next;
		free(this_command);
		this_command=next_command;
	        }

	return;
        }


/* free memory allocated to the timeperiod list */
void free_timeperiod_memory(void){
	timeperiod *this_timeperiod;
	timeperiod *next_timeperiod;

	/* free memory for the timeperiod list */
	this_timeperiod=timeperiod_list;
	while(this_timeperiod!=NULL){
		next_timeperiod=this_timeperiod->next;
		free(this_timeperiod);
		this_timeperiod=next_timeperiod;
		}

	return;
        }


/* free memory allocated to the contact list */
void free_contact_memory(void){
	contact *this_contact;
	contact *next_contact;

	/* free memory for the contact list */
	this_contact=contact_list;
	while(this_contact!=NULL){
		next_contact=this_contact->next;
		free(this_contact);
		this_contact=next_contact;
		}

	return;
        }



/**********************************************************
 *************** CONFIG FILE FUNCTIONS ********************
 **********************************************************/


/* read the CGI configuration file */
int read_cgi_config_file(char *filename){
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	FILE *fp;
	hostextinfo *new_hostextinfo;
	hostextinfo *last_hostextinfo;
	hostextinfo *temp_hostextinfo;

	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

	        if(feof(fp))
		        break;

		if(input_buffer==NULL)
		        continue;

		if(strstr(input_buffer,"main_config_file=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			if(temp_buffer!=NULL){
				strncpy(main_config_file,temp_buffer,sizeof(main_config_file));
				main_config_file[sizeof(main_config_file)-1]='\x0';
				strip(main_config_file);
			        }
		        }

		else if((strstr(input_buffer,"use_authentication=")==input_buffer)){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\n");
			if(temp_buffer==NULL)
				use_authentication=TRUE;
			else if(atoi(temp_buffer)==0)
				use_authentication=FALSE;
			else
				use_authentication=TRUE;
		        }

		else if(strstr(input_buffer,"netsaint_check_command=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			if(temp_buffer!=NULL){
				strncpy(netsaint_check_command,temp_buffer,sizeof(netsaint_check_command));
				netsaint_check_command[sizeof(netsaint_check_command)-1]='\x0';
				strip(netsaint_check_command);
			        }
		        }

		else if(strstr(input_buffer,"hostextinfo[")==input_buffer){

			/* add the host extended info entry to the list in memory */
			new_hostextinfo=(hostextinfo *)malloc(sizeof(hostextinfo));
			if(new_hostextinfo!=NULL){
				
				temp_buffer=my_strtok(input_buffer,"[");
				temp_buffer=my_strtok(NULL,"]");
				strncpy(new_hostextinfo->host_name,(temp_buffer==NULL)?"":temp_buffer,MAX_HOSTNAME_LENGTH);
				new_hostextinfo->host_name[MAX_HOSTNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_hostextinfo->notes_url,(temp_buffer==NULL)?"":temp_buffer+1,MAX_INPUT_BUFFER);
				new_hostextinfo->notes_url[MAX_INPUT_BUFFER-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_hostextinfo->icon_image,(temp_buffer==NULL)?"":temp_buffer,MAX_FILENAME_LENGTH);
				new_hostextinfo->icon_image[MAX_FILENAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_hostextinfo->vrml_image,(temp_buffer==NULL)?"":temp_buffer,MAX_FILENAME_LENGTH);
				new_hostextinfo->vrml_image[MAX_FILENAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_hostextinfo->gd2_icon_image,(temp_buffer==NULL)?"":temp_buffer,MAX_FILENAME_LENGTH);
				new_hostextinfo->gd2_icon_image[MAX_FILENAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_hostextinfo->icon_image_alt,(temp_buffer==NULL)?"":temp_buffer,MAX_INPUT_BUFFER);
				new_hostextinfo->icon_image_alt[MAX_INPUT_BUFFER-1]='\x0';

				/* add new host extended info entry to list, sorted by host name */
				last_hostextinfo=hostextinfo_list;
				for(temp_hostextinfo=hostextinfo_list;temp_hostextinfo!=NULL;temp_hostextinfo=temp_hostextinfo->next){
					if(strcmp(new_hostextinfo->host_name,temp_hostextinfo->host_name)<0){
						new_hostextinfo->next=temp_hostextinfo;
						if(temp_hostextinfo==hostextinfo_list)
							hostextinfo_list=new_hostextinfo;
						else
							last_hostextinfo->next=new_hostextinfo;
						break;
					        }
					else
						last_hostextinfo=temp_hostextinfo;
				        }
				if(hostextinfo_list==NULL){
					new_hostextinfo->next=NULL;
					hostextinfo_list=new_hostextinfo;
				        }
				else if(temp_hostextinfo==NULL){
					new_hostextinfo->next=NULL;
					last_hostextinfo->next=new_hostextinfo;
				        }

			        }
		        }

		else if(strstr(input_buffer,"html_table_bg_color=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(html_table_bg_color,(temp_buffer==NULL)?"":temp_buffer,sizeof(html_table_bg_color));
			html_table_bg_color[sizeof(html_table_bg_color)-1]='\x0';
			strip(html_table_bg_color);
		        }

		else if(strstr(input_buffer,"html_table_fg_color=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(html_table_fg_color,(temp_buffer==NULL)?"":temp_buffer,sizeof(html_table_fg_color));
			html_table_fg_color[sizeof(html_table_fg_color)-1]='\x0';
			strip(html_table_fg_color);
		        }

		else if(strstr(input_buffer,"html_table_bg_odd_color=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(html_table_bg_odd_color,(temp_buffer==NULL)?"":temp_buffer,sizeof(html_table_bg_odd_color));
			html_table_bg_odd_color[sizeof(html_table_bg_odd_color)-1]='\x0';
			strip(html_table_bg_color);
		        }

		else if(strstr(input_buffer,"html_table_bg_even_color=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(html_table_bg_even_color,(temp_buffer==NULL)?"":temp_buffer,sizeof(html_table_bg_even_color));
			html_table_bg_even_color[sizeof(html_table_bg_even_color)-1]='\x0';
			strip(html_table_bg_even_color);
		        }

		else if(strstr(input_buffer,"suppress_alert_window=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			suppress_alert_window=(atoi((temp_buffer==NULL)?"0":temp_buffer)==0)?FALSE:TRUE;
		        }

		else if(strstr(input_buffer,"refresh_rate=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			refresh_rate=atoi((temp_buffer==NULL)?"60":temp_buffer);
		        }

		else if(strstr(input_buffer,"physical_html_path=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\n");
			strncpy(physical_html_path,(temp_buffer==NULL)?"":temp_buffer,sizeof(physical_html_path));
			physical_html_path[sizeof(physical_html_path)-1]='\x0';
			strip(physical_html_path);
			if(physical_html_path[strlen(physical_html_path)-1]!='/' && (strlen(physical_html_path) < sizeof(physical_html_path)-1))
				strcat(physical_html_path,"/");
			snprintf(physical_images_path,sizeof(physical_images_path),"%simages/",physical_html_path);
			physical_images_path[sizeof(physical_images_path)-1]='\x0';
		        }

		else if(strstr(input_buffer,"url_html_path=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\n");
			strncpy(url_html_path,(temp_buffer==NULL)?"":temp_buffer,sizeof(url_html_path));
			url_html_path[sizeof(url_html_path)-1]='\x0';
			strip(url_html_path);
			if(url_html_path[strlen(url_html_path)-1]!='/' && (strlen(url_html_path) < sizeof(url_html_path)-1))
				strcat(url_html_path,"/");
			snprintf(url_docs_path,sizeof(url_docs_path),"%sdocs/",url_html_path);
			url_docs_path[sizeof(url_docs_path)-1]='\x0';
			snprintf(url_images_path,sizeof(url_images_path),"%simages/",url_html_path);
			url_images_path[sizeof(url_images_path)-1]='\x0';
			snprintf(url_logo_images_path,sizeof(url_logo_images_path),"%slogos/",url_images_path);
			url_logo_images_path[sizeof(url_logo_images_path)-1]='\x0';
		        }

		else if(strstr(input_buffer,"url_cgibin_path=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(url_cgibin_path,(temp_buffer==NULL)?"":temp_buffer,sizeof(url_cgibin_path));
			url_cgibin_path[sizeof(url_cgibin_path)-1]='\x0';
			strip(url_cgibin_path);
			if(url_cgibin_path[strlen(url_cgibin_path)-1]!='/' && (strlen(url_cgibin_path) < sizeof(url_cgibin_path)-1))
				strcat(url_cgibin_path,"/");
		        }

 	        }

	fclose(fp);

	if(!strcmp(main_config_file,""))
		return ERROR;
	else
		return OK;
        }



/* read the main configuration file */
int read_main_config_file(char *filename){
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	FILE *fp;

	
	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

	        if(feof(fp))
		        break;

		if(input_buffer==NULL)
		        continue;

		if(strstr(input_buffer,"interval_length=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			interval_length=(temp_buffer==NULL)?60:atoi(temp_buffer);
		        }

		else if(strstr(input_buffer,"log_file=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(log_file,(temp_buffer==NULL)?"":temp_buffer,sizeof(log_file));
			log_file[sizeof(log_file)-1]='\x0';
			strip(log_file);
		        }

		else if(strstr(input_buffer,"log_archive_path=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\n");
			strncpy(log_archive_path,(temp_buffer==NULL)?"":temp_buffer,sizeof(log_archive_path));
			log_archive_path[sizeof(log_archive_path)-1]='\x0';
			strip(physical_html_path);
			if(log_archive_path[strlen(log_archive_path)-1]!='/' && (strlen(log_archive_path) < sizeof(log_archive_path)-1))
				strcat(log_archive_path,"/");
		        }

		else if(strstr(input_buffer,"log_rotation_method=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			if(temp_buffer==NULL)
				log_rotation_method=LOG_ROTATION_NONE;
			else if(!strcmp(temp_buffer,"h"))
				log_rotation_method=LOG_ROTATION_HOURLY;
			else if(!strcmp(temp_buffer,"d"))
				log_rotation_method=LOG_ROTATION_DAILY;
			else if(!strcmp(temp_buffer,"w"))
				log_rotation_method=LOG_ROTATION_WEEKLY;
			else if(!strcmp(temp_buffer,"m"))
				log_rotation_method=LOG_ROTATION_MONTHLY;
		        }

		else if(strstr(input_buffer,"status_file=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(status_file,(temp_buffer==NULL)?"":temp_buffer,sizeof(status_file));
			status_file[sizeof(status_file)-1]='\x0';
			strip(status_file);
		        }
	
		else if(strstr(input_buffer,"command_file=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(command_file,(temp_buffer==NULL)?"":temp_buffer,sizeof(command_file));
			command_file[sizeof(command_file)-1]='\x0';
			strip(command_file);
		        }

		else if(strstr(input_buffer,"comment_file=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(comment_file,(temp_buffer==NULL)?"":temp_buffer,sizeof(comment_file));
			comment_file[sizeof(comment_file)-1]='\x0';
			strip(comment_file);
		        }

		else if(strstr(input_buffer,"check_external_commands=")==input_buffer){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			check_external_commands=(temp_buffer==NULL)?0:atoi(temp_buffer);
		        }
               }

	fclose(fp);
	return OK;
        }



/* read all host configuration files */
int read_all_host_config_files(int options){
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	char config_file[MAX_FILENAME_LENGTH];
	FILE *fp;

	/* open the main config file */
	fp=fopen(main_config_file,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

	        if(feof(fp))
		        break;

		if(input_buffer==NULL)
		        continue;

		/* we found a host config file entry */
		if(strstr(input_buffer,"cfg_file=")==input_buffer){

			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(config_file,(temp_buffer==NULL)?"":temp_buffer,sizeof(config_file));
			config_file[sizeof(config_file)-1]='\x0';
			strip(config_file);

			/* read configuration info from this hosts config file... */
			read_host_config_file(config_file,options);
		        }
	        }

	fclose(fp);

	return OK;
        }


/* read a host configuration file */
int read_host_config_file(char *filename,int options){
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	hostgroup *new_hostgroup;
	hostgroup *last_hostgroup;
	hostgroup *temp_hostgroup;
	contactgroup *new_contactgroup;
	contactgroup *last_contactgroup;
	contactgroup *temp_contactgroup;
	host *new_host;
	host *last_host;
	host *temp_host;
	service *new_service;
	service *last_service;
	service *temp_service;
	command *new_command;
	command *last_command;
	command *temp_command;
	timeperiod *new_timeperiod;
	timeperiod *last_timeperiod;
	timeperiod *temp_timeperiod;
	contact *new_contact;
	contact *last_contact;
	contact *temp_contact;
	FILE *fp;


	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

	        if(feof(fp))
		        break;

		if(input_buffer==NULL)
		        continue;

		/* if the hostgroup identifier is found at the start of the line... */
		if((strstr(input_buffer,"hostgroup[")==input_buffer) && (options & READ_HOSTGROUPS) && hostgroups_have_been_read==FALSE){

			/* add the hostgroup entry to the list in memory */
			new_hostgroup=(hostgroup *)malloc(sizeof(hostgroup));
			if(new_hostgroup!=NULL){
				
				temp_buffer=my_strtok(input_buffer,"[");
				temp_buffer=my_strtok(NULL,"]");
				strncpy(new_hostgroup->group_name,(temp_buffer==NULL)?"":temp_buffer,MAX_HOSTGROUPNAME_LENGTH);
				new_hostgroup->group_name[MAX_HOSTGROUPNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_hostgroup->group_alias,(temp_buffer==NULL)?"":temp_buffer+1,MAX_HOSTGROUPALIAS_LENGTH);
				new_hostgroup->group_alias[MAX_HOSTGROUPALIAS_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_hostgroup->contact_members,(temp_buffer==NULL)?"":temp_buffer,MAX_CONTACTMEMBERS_LENGTH);
				new_hostgroup->contact_members[MAX_CONTACTMEMBERS_LENGTH-1]='\x0';
				strip(new_hostgroup->contact_members);
				temp_buffer=my_strtok(NULL,"\x0");
				strncpy(new_hostgroup->members,(temp_buffer==NULL)?"":temp_buffer,MAX_HOSTGROUPMEMBERS_LENGTH);
				new_hostgroup->members[MAX_HOSTGROUPMEMBERS_LENGTH-1]='\x0';
				strip(new_hostgroup->members);

				/* add new hostgroup to hostgroup list, sorted by hostgroup name */
				last_hostgroup=hostgroup_list;
				for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
					if(strcmp(new_hostgroup->group_name,temp_hostgroup->group_name)<0){
						new_hostgroup->next=temp_hostgroup;
						if(temp_hostgroup==hostgroup_list)
							hostgroup_list=new_hostgroup;
						else
							last_hostgroup->next=new_hostgroup;
						break;
					        }
					else
						last_hostgroup=temp_hostgroup;
				        }
				if(hostgroup_list==NULL){
					new_hostgroup->next=NULL;
					hostgroup_list=new_hostgroup;
				        }
				else if(temp_hostgroup==NULL){
					new_hostgroup->next=NULL;
					last_hostgroup->next=new_hostgroup;
				        }

			        }
		        }

		/* we found a host definition... */
		else if((strstr(input_buffer,"host[")==input_buffer) && (options & READ_HOSTS) && hosts_have_been_read==FALSE){

			/* add the new host to the list in memory */
			new_host=(host *)malloc(sizeof(host));
			if(new_host!=NULL){
				
				temp_buffer=my_strtok(input_buffer,"[");
				temp_buffer=my_strtok(NULL,"]");
				strncpy(new_host->name,(temp_buffer==NULL)?"":temp_buffer,MAX_HOSTNAME_LENGTH);
				new_host->name[MAX_HOSTNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_host->alias,(temp_buffer==NULL)?"":temp_buffer+1,MAX_HOSTNAME_LENGTH);
				new_host->alias[MAX_HOSTNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_host->address,(temp_buffer==NULL)?"":temp_buffer,MAX_ADDRESS_LENGTH);
				new_host->address[MAX_ADDRESS_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_host->parent_host,(temp_buffer==NULL)?"":temp_buffer,MAX_HOSTNAME_LENGTH);
				new_host->parent_host[MAX_HOSTNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_host->host_check_command,(temp_buffer==NULL)?"":temp_buffer,MAX_COMMANDNAME_LENGTH);
				new_host->host_check_command[MAX_COMMANDNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				new_host->max_attempts=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_host->notification_interval=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_host->notification_period,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODNAME_LENGTH);
				new_host->notification_period[MAX_TIMEPERIODNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				new_host->notify_on_recovery=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_host->notify_on_down=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_host->notify_on_unreachable=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,"\x0");
				strncpy(new_host->event_handler,(temp_buffer==NULL)?"":temp_buffer,MAX_COMMANDNAME_LENGTH);
				new_host->event_handler[MAX_COMMANDNAME_LENGTH-1]='\x0';
				strip(new_host->event_handler);

				new_host->number_of_immediate_children=0;
				new_host->max_child_drawing_width=0;

				/* add new host to host list, sorted by host name */
				last_host=host_list;
				for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
					if(strcmp(new_host->name,temp_host->name)<0){
						new_host->next=temp_host;
						if(temp_host==host_list)
							host_list=new_host;
						else
							last_host->next=new_host;
						break;
					        }
					else
						last_host=temp_host;
				        }
				if(host_list==NULL){
					new_host->next=NULL;
					host_list=new_host;
				        }
				else if(temp_host==NULL){
					new_host->next=NULL;
					last_host->next=new_host;
				        }
			         }
		        }

		/* we found a service definition... */
		else if(strstr(input_buffer,"service[")==input_buffer && (options & READ_SERVICES) && services_have_been_read==FALSE){

			/* add the new service to the list in memory */
			new_service=(service *)malloc(sizeof(service));
			if(new_service!=NULL){
				
				temp_buffer=my_strtok(input_buffer,"[");
				temp_buffer=my_strtok(NULL,"]");
				strncpy(new_service->host_name,(temp_buffer==NULL)?"":temp_buffer,MAX_HOSTNAME_LENGTH);
				new_service->host_name[MAX_HOSTNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_service->description,(temp_buffer==NULL)?"":temp_buffer+1,MAX_SERVICEDESC_LENGTH);
				new_service->description[MAX_SERVICEDESC_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_service->check_period,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODNAME_LENGTH);
				new_service->check_period[MAX_TIMEPERIODNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				new_service->max_attempts=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_service->check_interval=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_service->retry_interval=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_service->contact_members,(temp_buffer==NULL)?"":temp_buffer,MAX_CONTACTMEMBERS_LENGTH);
				new_service->contact_members[MAX_CONTACTMEMBERS_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				new_service->notification_interval=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_service->notification_period,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODNAME_LENGTH);
				new_service->notification_period[MAX_TIMEPERIODNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				new_service->notify_on_recovery=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_service->notify_on_critical=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_service->notify_on_warning=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_service->event_handler,(temp_buffer==NULL)?"":temp_buffer,MAX_COMMANDNAME_LENGTH);
				new_service->event_handler[MAX_COMMANDNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,"\x0");
				strncpy(new_service->command,(temp_buffer==NULL)?"":temp_buffer,MAX_COMMANDLINE_LENGTH);
				new_service->command[MAX_COMMANDLINE_LENGTH-1]='\x0';

				/* add new service to service list, sorted by service name */
				last_service=service_list;
				for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
					if(strcmp(new_service->host_name,temp_service->host_name)<0){
						new_service->next=temp_service;
						if(temp_service==service_list)
							service_list=new_service;
						else
							last_service->next=new_service;
						break;
					        }
					else
						last_service=temp_service;
				        }
				if(service_list==NULL){
					new_service->next=NULL;
					service_list=new_service;
				        }
				else if(temp_service==NULL){
					new_service->next=NULL;
					last_service->next=new_service;
				        }
			         }
		        }

		/* we found a contact group definition... */
		else if(strstr(input_buffer,"contactgroup[")==input_buffer && (options & READ_CONTACTGROUPS) && contactgroups_have_been_read==FALSE){

			/* add the contactgroup entry to the list in memory */
			new_contactgroup=(contactgroup *)malloc(sizeof(contactgroup));
			if(new_contactgroup!=NULL){
				
				temp_buffer=my_strtok(input_buffer,"[");
				temp_buffer=my_strtok(NULL,"]");
				strncpy(new_contactgroup->group_name,(temp_buffer==NULL)?"":temp_buffer,MAX_CONTACTGROUPNAME_LENGTH);
				new_contactgroup->group_name[MAX_CONTACTGROUPNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_contactgroup->group_alias,(temp_buffer==NULL)?"":temp_buffer+1,MAX_CONTACTGROUPALIAS_LENGTH);
				new_contactgroup->group_alias[MAX_CONTACTGROUPALIAS_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,"\x0");
				strncpy(new_contactgroup->members,(temp_buffer==NULL)?"":temp_buffer,MAX_CONTACTMEMBERS_LENGTH);
				new_contactgroup->members[MAX_CONTACTMEMBERS_LENGTH-1]='\x0';
				strip(new_contactgroup->members);

				/* add new contactgroup to contactgroup list, sorted by group name */
				last_contactgroup=contactgroup_list;
				for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){
					if(strcmp(new_contactgroup->group_name,temp_contactgroup->group_name)<0){
						new_contactgroup->next=temp_contactgroup;
						if(temp_contactgroup==contactgroup_list)
							contactgroup_list=new_contactgroup;
						else
							last_contactgroup->next=new_contactgroup;
						break;
					        }
					else
						last_contactgroup=temp_contactgroup;
				        }
				if(contactgroup_list==NULL){
					new_contactgroup->next=NULL;
					contactgroup_list=new_contactgroup;
				        }
				else if(temp_contactgroup==NULL){
					new_contactgroup->next=NULL;
					last_contactgroup->next=new_contactgroup;
				        }

			        }
		        }


		/* we found a contact definition... */
		else if(strstr(input_buffer,"contact[")==input_buffer && (options & READ_CONTACTS) && contacts_have_been_read==FALSE){

			/* add the contact entry to the list in memory */
			new_contact=(contact *)malloc(sizeof(contact));
			if(new_contact!=NULL){
				
				temp_buffer=my_strtok(input_buffer,"[");
				temp_buffer=my_strtok(NULL,"]");
				strncpy(new_contact->name,(temp_buffer==NULL)?"":temp_buffer,MAX_CONTACTNAME_LENGTH);
				new_contact->name[MAX_CONTACTNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,"=");
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_contact->alias,(temp_buffer==NULL)?"":temp_buffer,MAX_CONTACTALIAS_LENGTH);
				new_contact->alias[MAX_CONTACTALIAS_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_contact->service_notification_period,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODNAME_LENGTH);
				new_contact->service_notification_period[MAX_TIMEPERIODNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_contact->host_notification_period,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODNAME_LENGTH);
				new_contact->host_notification_period[MAX_TIMEPERIODNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				new_contact->notify_on_service_warning=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_contact->notify_on_service_critical=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_contact->notify_on_service_recovery=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_contact->notify_on_host_down=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_contact->notify_on_host_unreachable=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_contact->notify_on_host_recovery=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_contact->commands,(temp_buffer==NULL)?"":temp_buffer,MAX_CONTACTCOMMANDS_LENGTH);
				new_contact->commands[MAX_CONTACTCOMMANDS_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_contact->host_commands,(temp_buffer==NULL)?"":temp_buffer,MAX_CONTACTCOMMANDS_LENGTH);
				new_contact->host_commands[MAX_CONTACTCOMMANDS_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_contact->email,(temp_buffer==NULL)?"":temp_buffer,MAX_EMAIL_LENGTH);
				new_contact->email[MAX_EMAIL_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,"\x0");
				strncpy(new_contact->pager,(temp_buffer==NULL)?"":temp_buffer,MAX_PAGER_LENGTH);
				new_contact->pager[MAX_PAGER_LENGTH-1]='\x0';

				/* add new contact to contact list, sorted by contact name */
				last_contact=contact_list;
				for(temp_contact=contact_list;temp_contact!=NULL;temp_contact=temp_contact->next){
					if(strcmp(new_contact->name,temp_contact->name)<0){
						new_contact->next=temp_contact;
						if(temp_contact==contact_list)
							contact_list=new_contact;
						else
							last_contact->next=new_contact;
						break;
					        }
					else
						last_contact=temp_contact;
				        }
				if(contact_list==NULL){
					new_contact->next=NULL;
					contact_list=new_contact;
				        }
				else if(temp_contact==NULL){
					new_contact->next=NULL;
					last_contact->next=new_contact;
				        }
			        }
		        }


		/* we found a command definition... */
		else if(strstr(input_buffer,"command[")==input_buffer && (options & READ_COMMANDS) && commands_have_been_read==FALSE){

			/* add the command entry to the list in memory */
			new_command=(command *)malloc(sizeof(command));
			if(new_command!=NULL){
				
				temp_buffer=my_strtok(input_buffer,"[");
				temp_buffer=my_strtok(NULL,"]");
				strncpy(new_command->name,(temp_buffer==NULL)?"":temp_buffer,MAX_COMMANDNAME_LENGTH);
				new_command->name[MAX_COMMANDNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,"=");
				temp_buffer=my_strtok(NULL,"\x0");
				strncpy(new_command->command_line,(temp_buffer==NULL)?"":temp_buffer,MAX_COMMANDLINE_LENGTH);
				new_command->command_line[MAX_COMMANDLINE_LENGTH-1]='\x0';

				/* add new command to command list, sorted by command name */
				last_command=command_list;
				for(temp_command=command_list;temp_command!=NULL;temp_command=temp_command->next){
					if(strcmp(new_command->name,temp_command->name)<0){
						new_command->next=temp_command;
						if(temp_command==command_list)
							command_list=new_command;
						else
							last_command->next=new_command;
						break;
					        }
					else
						last_command=temp_command;
				        }
				if(command_list==NULL){
					new_command->next=NULL;
					command_list=new_command;
				        }
				else if(temp_command==NULL){
					new_command->next=NULL;
					last_command->next=new_command;
				        }
			        }
		        }


		/* we found a timeperiod definition... */
		else if(strstr(input_buffer,"timeperiod[")==input_buffer && (options & READ_TIMEPERIODS) && timeperiods_have_been_read==FALSE){

			/* add the timeperiod entry to the list in memory */
			new_timeperiod=(timeperiod *)malloc(sizeof(timeperiod));
			if(new_timeperiod!=NULL){
				
				temp_buffer=my_strtok(input_buffer,"[");
				temp_buffer=my_strtok(NULL,"]");
				strncpy(new_timeperiod->name,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODNAME_LENGTH);
				new_timeperiod->name[MAX_TIMEPERIODNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,"=");
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_timeperiod->alias,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODALIAS_LENGTH);
				new_timeperiod->alias[MAX_TIMEPERIODALIAS_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_timeperiod->sun_range,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODRANGE_LENGTH);
				new_timeperiod->sun_range[MAX_TIMEPERIODRANGE_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_timeperiod->mon_range,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODRANGE_LENGTH);
				new_timeperiod->mon_range[MAX_TIMEPERIODRANGE_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_timeperiod->tue_range,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODRANGE_LENGTH);
				new_timeperiod->tue_range[MAX_TIMEPERIODRANGE_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_timeperiod->wed_range,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODRANGE_LENGTH);
				new_timeperiod->wed_range[MAX_TIMEPERIODRANGE_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_timeperiod->thu_range,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODRANGE_LENGTH);
				new_timeperiod->thu_range[MAX_TIMEPERIODRANGE_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_timeperiod->fri_range,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODRANGE_LENGTH);
				new_timeperiod->fri_range[MAX_TIMEPERIODRANGE_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,"\x0");
				strncpy(new_timeperiod->sat_range,(temp_buffer==NULL)?"":temp_buffer,MAX_TIMEPERIODRANGE_LENGTH);
				new_timeperiod->sat_range[MAX_TIMEPERIODRANGE_LENGTH-1]='\x0';

				/* add new timeperiod to timeperiod list, sorted by timeperiod name */
				last_timeperiod=timeperiod_list;
				for(temp_timeperiod=timeperiod_list;temp_timeperiod!=NULL;temp_timeperiod=temp_timeperiod->next){
					if(strcmp(new_timeperiod->name,temp_timeperiod->name)<0){
						new_timeperiod->next=temp_timeperiod;
						if(temp_timeperiod==timeperiod_list)
							timeperiod_list=new_timeperiod;
						else
							last_timeperiod->next=new_timeperiod;
						break;
					        }
					else
						last_timeperiod=temp_timeperiod;
				        }
				if(timeperiod_list==NULL){
					new_timeperiod->next=NULL;
					timeperiod_list=new_timeperiod;
				        }
				else if(temp_timeperiod==NULL){
					new_timeperiod->next=NULL;
					last_timeperiod->next=new_timeperiod;
				        }
			        }
		        }

	        }

	fclose(fp);

	/* mark what items we've read in... */
	if(options & READ_HOSTS)
		hosts_have_been_read=TRUE;
	if(options & READ_HOSTGROUPS)
		hostgroups_have_been_read=TRUE;
	if(options & READ_CONTACTS)
		contacts_have_been_read=TRUE;
	if(options & READ_CONTACTGROUPS)
		contactgroups_have_been_read=TRUE;
	if(options & READ_SERVICES)
		services_have_been_read=TRUE;
	if(options & READ_TIMEPERIODS)
		timeperiods_have_been_read=TRUE;
	if(options & READ_COMMANDS)
		commands_have_been_read=TRUE;

	return OK;
        }



/* read the contents of the status log */
int read_status_log(char *filename){
	char input_buffer[MAX_INPUT_BUFFER];
	char status[MAX_STATUS_LENGTH];
	char *temp_buffer;
	FILE *fp;
	servicestatus *new_svcstatus;
	servicestatus *last_svcstatus;
	servicestatus *temp_svcstatus;
	hoststatus *new_hoststatus;
	hoststatus *last_hoststatus;
	hoststatus *temp_hoststatus;


	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

	        if(feof(fp))
		        break;

		if(input_buffer==NULL)
		        continue;

		/* program status information... */
		if(strstr(input_buffer,"] PROGRAM;")){
		
			temp_buffer=my_strtok(input_buffer,";");

			temp_buffer=my_strtok(NULL,";");
			program_start=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);

			temp_buffer=my_strtok(NULL,";");
			if(atoi((temp_buffer==NULL)?0:temp_buffer))
				daemon_mode=TRUE;
			else
				daemon_mode=FALSE;

			temp_buffer=my_strtok(NULL,";");
			if(!strcmp(temp_buffer,"STANDBY"))
				program_mode=STANDBY_MODE;
			else
				program_mode=ACTIVE_MODE;

			temp_buffer=my_strtok(NULL,";");
			last_mode_change=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);

			temp_buffer=my_strtok(NULL,";");
			last_zombie_cleanup=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);

			temp_buffer=my_strtok(NULL,";");
			orphaned_child_processes=(temp_buffer==NULL)?0:atoi(temp_buffer);

			temp_buffer=my_strtok(NULL,";");
			last_command_check=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);

			temp_buffer=my_strtok(NULL,";");
			last_log_rotation=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);

		        }

		/* host status information... */
		else if(strstr(input_buffer,"] HOST;")){
		
			/* allocate memory for new host status */
			new_hoststatus=(hoststatus *)malloc(sizeof(hoststatus));
			if(new_hoststatus!=NULL){

				temp_buffer=my_strtok(input_buffer,"]");
				new_hoststatus->last_check=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer+1,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_hoststatus->host_name,(temp_buffer==NULL)?"":temp_buffer,MAX_HOSTNAME_LENGTH);
				new_hoststatus->host_name[MAX_HOSTNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(status,(temp_buffer==NULL)?"":temp_buffer,sizeof(status));
				status[sizeof(status)-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				new_hoststatus->last_state_change=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				new_hoststatus->time_up=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				new_hoststatus->time_down=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				new_hoststatus->time_unreachable=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				new_hoststatus->last_notification=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				new_hoststatus->notifications_enabled=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,"\n");
				strncpy(new_hoststatus->information,(temp_buffer==NULL)?"":temp_buffer,MAX_HOSTINFO_LENGTH);
				new_hoststatus->information[MAX_HOSTINFO_LENGTH-1]='\x0';

				if(!strcmp(status,"DOWN"))
					new_hoststatus->status=HOST_DOWN;
				else if(!strcmp(status,"UNREACHABLE"))
					new_hoststatus->status=HOST_UNREACHABLE;
				else if(!strcmp(status,"PENDING"))
					new_hoststatus->status=HOST_PENDING;
				else
					new_hoststatus->status=HOST_UP;

				/* add new host status to list, sorted by host name */
				last_hoststatus=hoststatus_list;
				for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next){
					if(strcmp(new_hoststatus->host_name,temp_hoststatus->host_name)<0){
						new_hoststatus->next=temp_hoststatus;
						if(temp_hoststatus==hoststatus_list)
							hoststatus_list=new_hoststatus;
						else
							last_hoststatus->next=new_hoststatus;
						break;
					        }
					else
						last_hoststatus=temp_hoststatus;
				        }
				if(hoststatus_list==NULL){
					new_hoststatus->next=NULL;
					hoststatus_list=new_hoststatus;
				        }
				else if(temp_hoststatus==NULL){
					new_hoststatus->next=NULL;
					last_hoststatus->next=new_hoststatus;
				        }
			        }
		        }

		/* service status information... */
		else if(strstr(input_buffer,"] SERVICE;")){

			/* allocate memory for a new service status entry */
			new_svcstatus=(servicestatus *)malloc(sizeof(servicestatus));
			if(new_svcstatus!=NULL){

				temp_buffer=my_strtok(input_buffer,"]");
				new_svcstatus->last_check=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer+1,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_svcstatus->host_name,(temp_buffer==NULL)?"":temp_buffer,MAX_HOSTNAME_LENGTH);
				new_svcstatus->host_name[MAX_HOSTNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_svcstatus->description,(temp_buffer==NULL)?"":temp_buffer,MAX_SERVICEDESC_LENGTH);
				new_svcstatus->description[MAX_SERVICEDESC_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(status,(temp_buffer==NULL)?"":temp_buffer,sizeof(status));
				status[sizeof(status)-1]='\x0';
				if(!strcmp(status,"WARNING"))
					new_svcstatus->status=SERVICE_WARNING;
				else if(!strcmp(status,"UNKNOWN"))
					new_svcstatus->status=SERVICE_UNKNOWN;
				else if(!strcmp(status,"CRITICAL"))
					new_svcstatus->status=SERVICE_CRITICAL;
				else if(!strcmp(status,"RECOVERY"))
					new_svcstatus->status=SERVICE_RECOVERY;
				else if(!strcmp(status,"HOST DOWN"))
					new_svcstatus->status=SERVICE_HOST_DOWN;
				else if(!strcmp(status,"UNREACHABLE"))
					new_svcstatus->status=SERVICE_UNREACHABLE;
				else if(!strcmp(status,"PENDING"))
					new_svcstatus->status=SERVICE_PENDING;
				else
					new_svcstatus->status=SERVICE_OK;
				temp_buffer=my_strtok(NULL,"/");
				new_svcstatus->current_attempt=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_svcstatus->max_attempts=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_svcstatus->state_type=(!strcmp((temp_buffer==NULL)?"":temp_buffer,"SOFT"))?SOFT_STATE:HARD_STATE;
				temp_buffer=my_strtok(NULL,";");
				new_svcstatus->next_check=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				new_svcstatus->enabled=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_svcstatus->last_state_change=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				strncpy(status,(temp_buffer==NULL)?"":temp_buffer,sizeof(status));
				status[sizeof(status)-1]='\x0';
				if(!strcmp(status,"WARNING"))
					new_svcstatus->last_hard_state=SERVICE_WARNING;
				else if(!strcmp(status,"UNKNOWN"))
					new_svcstatus->last_hard_state=SERVICE_UNKNOWN;
				else if(!strcmp(status,"CRITICAL"))
					new_svcstatus->last_hard_state=SERVICE_CRITICAL;
				else
					new_svcstatus->last_hard_state=SERVICE_OK;
				temp_buffer=my_strtok(NULL,";");
				new_svcstatus->time_ok=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				new_svcstatus->time_unknown=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				new_svcstatus->time_warning=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				new_svcstatus->time_critical=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				new_svcstatus->last_notification=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				new_svcstatus->notifications_enabled=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,"\n");
				strncpy(new_svcstatus->information,(temp_buffer==NULL)?"":temp_buffer,MAX_SERVICEINFO_LENGTH);
				new_svcstatus->information[MAX_SERVICEINFO_LENGTH-1]='\x0';


				/* add new service status to front of list */
				last_svcstatus=servicestatus_list;
				for(temp_svcstatus=servicestatus_list;temp_svcstatus!=NULL;temp_svcstatus=temp_svcstatus->next){
					if(strcmp(new_svcstatus->host_name,temp_svcstatus->host_name)<0){
						new_svcstatus->next=temp_svcstatus;
						if(temp_svcstatus==servicestatus_list)
							servicestatus_list=new_svcstatus;
						else
							last_svcstatus->next=new_svcstatus;
						break;
					        }
					else
						last_svcstatus=temp_svcstatus;
				        }
				if(servicestatus_list==NULL){
					new_svcstatus->next=NULL;
					servicestatus_list=new_svcstatus;
				        }
				else if(temp_svcstatus==NULL){
					new_svcstatus->next=NULL;
					last_svcstatus->next=new_svcstatus;
				        }
			        }
		        }


	        }

	fclose(fp);

	return OK;
        }


/* read the comment file */
int read_comment_file(char *filename){
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	FILE *fp;
	comment *new_comment;
	comment *last_comment;
	comment *temp_comment;
	int is_persistent=0;

	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

	        if(feof(fp))
		        break;

		if(input_buffer==NULL)
		        continue;


		/* host comment... */
		if(strstr(input_buffer,"] HOST_COMMENT;")){
		
			/* allocate memory for new comment */
			new_comment=(comment *)malloc(sizeof(comment));
			if(new_comment!=NULL){

				temp_buffer=my_strtok(input_buffer,"[");
				temp_buffer=my_strtok(NULL,"]");
				new_comment->entry_time=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				temp_buffer=my_strtok(NULL,";");
				new_comment->comment_id=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_comment->host_name,(temp_buffer==NULL)?"":temp_buffer,MAX_HOSTNAME_LENGTH);
				new_comment->host_name[MAX_HOSTNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				is_persistent=(temp_buffer==NULL)?0:atoi(temp_buffer);
				new_comment->persistent=(is_persistent)?TRUE:FALSE;
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_comment->author,(temp_buffer==NULL)?"":temp_buffer,MAX_USERNAME_LENGTH);
				new_comment->author[MAX_USERNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,"\n");
				strncpy(new_comment->comment_data,(temp_buffer==NULL)?"":temp_buffer,MAX_COMMENT_LENGTH);
				new_comment->comment_data[MAX_COMMENT_LENGTH-1]='\x0';

				strcpy(new_comment->service_description,"");
				new_comment->comment_type=HOST_COMMENT;

				/* add new comment to comment list, sorted by comment id */
				last_comment=comment_list;
				for(temp_comment=comment_list;temp_comment!=NULL;temp_comment=temp_comment->next){
					if(new_comment->comment_id<temp_comment->comment_id){
						new_comment->next=temp_comment;
						if(temp_comment==comment_list)
							comment_list=new_comment;
						else
							last_comment->next=new_comment;
						break;
					        }
					else
						last_comment=temp_comment;
				        }
				if(comment_list==NULL){
					new_comment->next=NULL;
					comment_list=new_comment;
				        }
				else if(temp_comment==NULL){
					new_comment->next=NULL;
					last_comment->next=new_comment;
				        }
			        }
		        }


		/* service comment... */
		else if(strstr(input_buffer,"] SERVICE_COMMENT;")){
		
			/* allocate memory for new comment */
			new_comment=(comment *)malloc(sizeof(comment));
			if(new_comment!=NULL){

				temp_buffer=my_strtok(input_buffer,"[");
				temp_buffer=my_strtok(NULL,"]");
				new_comment->entry_time=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);
				temp_buffer=my_strtok(NULL,";");
				temp_buffer=my_strtok(NULL,";");
				new_comment->comment_id=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_comment->host_name,(temp_buffer==NULL)?"":temp_buffer,MAX_HOSTNAME_LENGTH);
				new_comment->host_name[MAX_HOSTNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_comment->service_description,(temp_buffer==NULL)?"":temp_buffer,MAX_SERVICEDESC_LENGTH);
				new_comment->service_description[MAX_SERVICEDESC_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,";");
				is_persistent=(temp_buffer==NULL)?0:atoi(temp_buffer);
				new_comment->persistent=(is_persistent)?TRUE:FALSE;
				temp_buffer=my_strtok(NULL,";");
				strncpy(new_comment->author,(temp_buffer==NULL)?"":temp_buffer,MAX_USERNAME_LENGTH);
				new_comment->author[MAX_USERNAME_LENGTH-1]='\x0';
				temp_buffer=my_strtok(NULL,"\n");
				strncpy(new_comment->comment_data,(temp_buffer==NULL)?"":temp_buffer,MAX_COMMENT_LENGTH);
				new_comment->comment_data[MAX_COMMENT_LENGTH-1]='\x0';

				new_comment->comment_type=SERVICE_COMMENT;

				/* add new comment to comment list, sorted by comment id */
				last_comment=comment_list;
				for(temp_comment=comment_list;temp_comment!=NULL;temp_comment=temp_comment->next){
					if(new_comment->comment_id<temp_comment->comment_id){
						new_comment->next=temp_comment;
						if(temp_comment==comment_list)
							comment_list=new_comment;
						else
							last_comment->next=new_comment;
						break;
					        }
					else
						last_comment=temp_comment;
				        }
				if(comment_list==NULL){
					new_comment->next=NULL;
					comment_list=new_comment;
				        }
				else if(temp_comment==NULL){
					new_comment->next=NULL;
					last_comment->next=new_comment;
				        }
			        }
		        }
	        }

	fclose(fp);

	return OK;
        }



/**********************************************************
 ************* OBJECT UTILITY FUNCTIONS *******************
 **********************************************************/

/* gets the total number of hosts in a certain state */
int get_host_count(host *hst, int type){
	hoststatus *temp_status;
	int count=0;

	for(temp_status=hoststatus_list;temp_status!=NULL;temp_status=temp_status->next){
		if(temp_status->status & type){
			if(hst==NULL)
				count++;
			else if(!strcmp(hst->name,temp_status->host_name))
				count++;
		        }
	        }

	return count;
        }


/* gets the total number of services of a certain state */
int get_service_count(host *hst, int type){
	servicestatus *temp_status;
	int count=0;

	for(temp_status=servicestatus_list;temp_status!=NULL;temp_status=temp_status->next){
		if(temp_status->status & type){
			if(hst==NULL)
				count++;
			else if(!strcmp(hst->name,temp_status->host_name))
				count++;
		        }
	        }

	return count;
        }


/*  tests wether a host is a member of a particular hostgroup */
int is_member_of_hostgroup(hostgroup *hg, host *hst){
	char members[MAX_HOSTGROUPMEMBERS_LENGTH];
	char *temp_buffer;

	if(hg==NULL)
		return FALSE;

	strncpy(members,hg->members,sizeof(members));
	members[sizeof(members)-1]='\x0';

	for(temp_buffer=(char *)my_strtok(members,",");temp_buffer!=NULL;temp_buffer=(char *)my_strtok(NULL,","))
		if(!strcmp(temp_buffer,hst->name))
			return TRUE;

	return FALSE;
        }


/*  tests wether a contact is a member of a particular contactgroup */
int is_member_of_contactgroup(contactgroup *cg, char *cntct_name){
	char members[MAX_CONTACTMEMBERS_LENGTH];
	char *temp_buffer;

	if(cg==NULL)
		return FALSE;

	/* get the list of contacts in this host group */
	strncpy(members,cg->members,sizeof(members));
	members[sizeof(members)-1]='\x0';

	/* see if the user is in the list of contacts */
	for(temp_buffer=(char *)my_strtok(members,",");temp_buffer!=NULL;temp_buffer=(char *)my_strtok(NULL,","))
		if(!strcmp(temp_buffer,cntct_name))
			return TRUE;

	return FALSE;
        }


/*  tests wether a contact is a member of a particular hostgroup */
int is_contact_member_of_hostgroup(hostgroup *hg, char *cntct_name){
	char members[MAX_CONTACTMEMBERS_LENGTH];
	contactgroup *temp_contactgroup;
	char *temp_buffer;

	if(hg==NULL)
		return FALSE;

	/* get the list of contact groups for this host group */
	strncpy(members,hg->contact_members,sizeof(members));
	members[sizeof(members)-1]='\x0';

	/* see if the contact is a member of any of the contact groups */
	for(temp_buffer=(char *)my_strtok(members,",");temp_buffer!=NULL;temp_buffer=(char *)my_strtok(NULL,",")){
		temp_contactgroup=find_contactgroup(temp_buffer);
		if(is_member_of_contactgroup(temp_contactgroup,cntct_name)==TRUE)
			return TRUE;
	        }

	return FALSE;
        }



/*  tests wether a contact is a contact for a particular host */
int is_contact_for_host(host *hst, char *cntct_name){
	hostgroup *temp_hostgroup;

	/* else check all hostgroups to see if the host is a member... */
	for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){

		/* the host is a part of this hostgroup... */
		if(is_member_of_hostgroup(temp_hostgroup,hst)==TRUE){
			
			/* the user is a contact member for this hostgroup, so they have rights... */
			if(is_contact_member_of_hostgroup(temp_hostgroup,cntct_name)==TRUE)
				return TRUE;
		        }
	        }

	return FALSE;
        }


/*  tests wether a contact is a contact for a particular service */
int is_contact_for_service(service *svc, char *cntct_name){
	char members[MAX_CONTACTMEMBERS_LENGTH];
	contactgroup *temp_contactgroup;
	char *temp_buffer;

	if(svc==NULL)
		return FALSE;

	strncpy(members,svc->contact_members,sizeof(members));
	members[sizeof(members)-1]='\x0';

	/* search through the list of contact groups for this host */
	for(temp_buffer=(char *)my_strtok(members,",");temp_buffer!=NULL;temp_buffer=my_strtok(NULL,",")){

		/* find the contact group object */
		temp_contactgroup=find_contactgroup(temp_buffer);

		/* the user is a member of this contact group */
		if(is_member_of_contactgroup(temp_contactgroup,cntct_name)==TRUE)
			return TRUE;
	        }

	return FALSE;
        }



/* determines whether or not a specific host is an immediate child of another host */
int is_immediate_child_of_host(host *parent_host,host *child_host){

	if(parent_host==NULL){
		if(!strcmp(child_host->parent_host,""))
			return 1;
		else
			return 0;
	        }

	else if(!strcmp(child_host->parent_host,parent_host->name))
		return 1;

	return 0;
	}


/* returns a count of the immediate children for a given host */
int number_of_immediate_children(host *hst){
	int children=0;
	host *temp_host;

	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
		if(is_immediate_child_of_host(hst,temp_host))
			children++;
		}

	return children;
	}


/* returns a count of the total children for a given host */
int number_of_children(host *hst){
	int children=0;
	host *temp_host;

	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
		if(is_immediate_child_of_host(hst,temp_host))
			children+=number_of_children(temp_host)+1;
		}

	return children;
	}


/* get the max depth of children hosts for a given host */
int get_max_host_child_depth(host *hst){
	int max_depth=0;
	int temp_depth=0;
	host *temp_host;

	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
		if(is_immediate_child_of_host(hst,temp_host)){
			temp_depth=get_max_host_child_depth(temp_host);
			if(temp_depth>max_depth)
				max_depth=temp_depth;
		        }
	        }

	return max_depth+1;
        }


/* determines whether or not a specific host is an immediate parent of another host */
int is_immediate_parent_of_host(host *child_host,host *parent_host){

	if(child_host==NULL || parent_host==NULL)
		return 0;

	else if(!strcmp(child_host->parent_host,parent_host->name))
		return 1;

	return 0;
	}


/* get the total number of parent hosts for a given host */
int number_of_parents(host *hst){
	int parents=0;
	host *temp_host;

	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
		if(is_immediate_parent_of_host(hst,temp_host)){
			parents+=number_of_parents(temp_host)+1;
			break;
		        }
	        }

	return parents;
        }



/* get the number of comments associated wth a particular host */
int number_of_host_comments(host *hst){
	comment *temp_comment;
	int total_comments=0;

	if(hst==NULL)
		return 0;

	for(temp_comment=comment_list;temp_comment!=NULL;temp_comment=temp_comment->next){
		if(temp_comment->comment_type==HOST_COMMENT && !strcmp(temp_comment->host_name,hst->name))
			total_comments++;
	        }

	return total_comments;
        }


/* get the number of comments associated wth a particular service */
int number_of_service_comments(service *svc){
	comment *temp_comment;
	int total_comments=0;

	if(svc==NULL)
		return 0;

	for(temp_comment=comment_list;temp_comment!=NULL;temp_comment=temp_comment->next){
		if(temp_comment->comment_type==SERVICE_COMMENT && !strcmp(temp_comment->host_name,svc->host_name) && !strcmp(temp_comment->service_description,svc->description))
			total_comments++;
	        }

	return total_comments;
        }



/**********************************************************
 *************** MISC UTILITY FUNCTIONS *******************
 **********************************************************/


/* recursively strips newline characters, spaces, and carriage returns from the end of a string */
void strip(char *buffer){
	int x;

	x=(int)strlen(buffer);

	if(!x)
		return;
	else
		x--;

	if(buffer[x]==' ' || buffer[x]=='\n' || buffer[x]=='\r' || buffer[x]=='\t' || buffer[x]==13)
		buffer[x]='\x0';

	else
		return;

	strip(buffer);

	return;
        }


/* reads a line of text from a file and strips it clean */
char * read_raw_line(char *buffer, int maxlength, FILE *fp){
	char *result;

	/* clear the buffer */
	strcpy(buffer,"");

	/* read in one line of text */
	result=fgets(buffer,maxlength-1,fp);

	/* if we were able to read something, strip extra characters from the end */
	if(result){
		buffer[maxlength-1]='\x0';
		strip(buffer);
	        }

	/* return the number of characters read */
	return result;
        }


/* read a line of text from a file, skipping comments and empty lines */
char * read_line(char *buffer, int maxlength, FILE *fp){
	char *result;
	int keep_line=TRUE;

	while((result=read_raw_line(buffer,maxlength,fp))){

		if(buffer[0]=='#')
			keep_line=FALSE;
		else if(buffer[0]=='\r')
			keep_line=FALSE;
		else if(buffer[0]=='\n')
			keep_line=FALSE;
		else if(buffer[0]=='\x0')
			keep_line=FALSE;
		else
			keep_line=TRUE;

		if(keep_line==TRUE)
			break;
	        }

	return result;
        }



/* get date/time string */
void get_time_string(time_t *raw_time,char *buffer,int buffer_length){
	time_t t;
	struct tm *tm_ptr;
	int day;
	int hour;
	int minute;
	int second;
	int year;
	char *weekdays[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	char *months[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sept","Oct","Nov","Dec"};
	char *timezone="";

	if(raw_time==NULL)
		time(&t);
	else 
		t=*raw_time;

	tm_ptr=localtime(&t);

	hour=tm_ptr->tm_hour;
	minute=tm_ptr->tm_min;
	second=tm_ptr->tm_sec;
	day=tm_ptr->tm_mday;
	year=tm_ptr->tm_year+1900;

#ifdef HAVE_TM_ZONE
	timezone=(char *)tm_ptr->tm_zone;
#else
	timezone=(char *)tzname[1];
#endif

	snprintf(buffer,buffer_length,"%s %s %d %02d:%02d:%02d %s %d",weekdays[tm_ptr->tm_wday],months[tm_ptr->tm_mon],day,hour,minute,second,timezone,year);
	buffer[buffer_length-1]='\x0';

	return;
        }


/* get time string for an interval of time */
void get_interval_time_string(int time_units,char *buffer,int buffer_length){
	unsigned long total_seconds;
	int hours=0;
	int minutes=0;
	int seconds=0;

	total_seconds=(unsigned long)(time_units*interval_length);
	hours=(int)total_seconds/3600;
	total_seconds%=3600;
	minutes=(int)total_seconds/60;
	total_seconds%=60;
	seconds=(int)total_seconds;
	snprintf(buffer,buffer_length,"%dh %dm %ds",hours,minutes,seconds);
	buffer[buffer_length-1]='\x0';

	return;
        }


/* get date/time string used in META tags for page expiration */
void get_expire_time_string(time_t *raw_time,char *buffer,int buffer_length){
	time_t t;
	struct tm *tm_ptr;
	int day;
	int hour;
	int minute;
	int second;
	int year;
	char *weekdays[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	char *months[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sept","Oct","Nov","Dec"};

	if(raw_time==NULL)
		time(&t);
	else 
		t=*raw_time;

	tm_ptr=gmtime(&t);

	hour=tm_ptr->tm_hour;
	minute=tm_ptr->tm_min;
	second=tm_ptr->tm_sec;
	day=tm_ptr->tm_mday;
	year=tm_ptr->tm_year+1900;

	snprintf(buffer,buffer_length,"%s, %d %s %d %02d:%02d:%02d GMT",weekdays[tm_ptr->tm_wday],day,months[tm_ptr->tm_mon],year,hour,minute,second);
	buffer[buffer_length-1]='\x0';

	return;
        }



/* fix the problem with strtok() skipping empty options between tokens */	
char *my_strtok(char *buffer,char *tokens){
	char *token_position;
	char *sequence_head;

	if(buffer!=NULL){
		if(original_my_strtok_buffer!=NULL)
			free(original_my_strtok_buffer);
		my_strtok_buffer=malloc(strlen(buffer)+1);
		if(my_strtok_buffer==NULL)
			return NULL;
		original_my_strtok_buffer=my_strtok_buffer;
		strcpy(my_strtok_buffer,buffer);
	        }
	
	sequence_head=my_strtok_buffer;

	if(sequence_head[0]=='\x0')
		return NULL;
	
	token_position=index(my_strtok_buffer,tokens[0]);

	if(token_position==NULL){
		my_strtok_buffer=index(my_strtok_buffer,'\x0');
		return sequence_head;
	        }

	token_position[0]='\x0';
	my_strtok_buffer=token_position+1;

	return sequence_head;

        }



/* get days, hours, minutes, and seconds from a raw time_t format or total seconds */
void get_time_breakdown(unsigned long raw_time,int *days,int *hours,int *minutes,int *seconds){
	unsigned long temp_time;
	int temp_days;
	int temp_hours;
	int temp_minutes;
	int temp_seconds;

	temp_time=raw_time;

	temp_days=temp_time/86400;
	temp_time-=(temp_days * 86400);
	temp_hours=temp_time/3600;
	temp_time-=(temp_hours * 3600);
	temp_minutes=temp_time/60;
	temp_time-=(temp_minutes * 60);
	temp_seconds=(int)temp_time;

	*days=temp_days;
	*hours=temp_hours;
	*minutes=temp_minutes;
	*seconds=temp_seconds;

	return;
	}



/* encodes a string in proper URL format */
char * url_encode(char *input){
	int len,output_len;
	int x,y;
	char temp_expansion[4];

	len=(int)strlen(input);
	output_len=(int)sizeof(encoded_url_string);

	encoded_url_string[0]='\x0';

	for(x=0,y=0;x<=len && y<output_len-1;x++){

		if((char)input[x]==(char)'\x0'){
			encoded_url_string[y]='\x0';
			break;
		        }
		else if((char)input[x]<=(char)' '){
			encoded_url_string[y]='+';
			y++;
		        }
		else if((char)input[x]>='0' && (char)input[x]<='9'){
			encoded_url_string[y]=input[x];
			y++;
		        }
		else if((char)input[x]>='A' && (char)input[x]<='Z'){
			encoded_url_string[y]=input[x];
			y++;
		        }
		else if((char)input[x]>=(char)'a' && (char)input[x]<=(char)'z'){
			encoded_url_string[y]=input[x];
			y++;
		        }
		else{
			encoded_url_string[y]='\x0';
			if((int)strlen(encoded_url_string)<(output_len-3)){
				sprintf(temp_expansion,"%%%02X",(unsigned int)input[x]);
				strcat(encoded_url_string,temp_expansion);
				y+=3;
			        }
		        }
	        }

	encoded_url_string[sizeof(encoded_url_string)-1]='\x0';

	return &encoded_url_string[0];
        }



/* determines the log file we should use */
void get_log_archive_to_use(int archive,char *buffer,int buffer_length){
	time_t current_time;
	struct tm *t;

	/* if we're not rotating the logs or if we want the current log, use the main one... */
	if(log_rotation_method==LOG_ROTATION_NONE || archive<=0){
		strncpy(buffer,log_file,buffer_length);
		buffer[buffer_length-1]='\x0';
		return;
	        }

	time(&current_time);

	determine_log_rotation_times(&current_time,archive);

	t=localtime(&this_scheduled_log_rotation);

	snprintf(buffer,buffer_length,"%snetsaint-%02d-%02d-%d-%02d.log",log_archive_path,t->tm_mon+1,t->tm_mday,t->tm_year+1900,t->tm_hour);
	buffer[buffer_length-1]='\x0';

	return;
        }


/* determines the log rotation times - past, present, future */
void determine_log_rotation_times(time_t *start_time,int archive){
	unsigned long past_hour;
	unsigned long past_day;
	unsigned long past_week;
	unsigned long past_month;
	struct tm *t;

	if(archive<=0)
		return;

	t=localtime(start_time);

	/* calculate the start of the hour */
	t->tm_sec=0;
	t->tm_min=0;
	past_hour=(unsigned long)mktime(t);

	/* calculate the start of the day (midnight, 00:00 hours) */
	t->tm_hour=0;
	past_day=(unsigned long)mktime(t);

	/* calculate the start of the day (midnight, 00:00 hours) for this past weekend */
	past_week=past_day-(60*60*24*t->tm_wday);

	/* calculate the start of the day (midnight, 00:00 hours) for the first day this month */
	t->tm_mday=1;
	past_month=(unsigned long)mktime(t);

	switch(log_rotation_method){
	case LOG_ROTATION_HOURLY:
		this_scheduled_log_rotation=(time_t)past_hour;
		last_scheduled_log_rotation=(time_t)(past_hour-(60*60));
		next_scheduled_log_rotation=(time_t)(past_hour+(60*60));
		break;
	case LOG_ROTATION_DAILY:
		this_scheduled_log_rotation=(time_t)past_day;
		last_scheduled_log_rotation=(time_t)(past_day-(60*60*24));
		next_scheduled_log_rotation=(time_t)(past_day+(60*60*24));
		break;
	case LOG_ROTATION_WEEKLY:
		this_scheduled_log_rotation=(time_t)past_week;
		last_scheduled_log_rotation=(time_t)(past_week-(60*60*24*7));
		next_scheduled_log_rotation=(time_t)(past_week+(60*60*24*7));
		break;
	case LOG_ROTATION_MONTHLY:
		this_scheduled_log_rotation=(time_t)past_month;
		if(t->tm_mon==0){
			t->tm_mon=11;
			t->tm_year--;
		        }
		else
			t->tm_mon--;
		last_scheduled_log_rotation=mktime(t);
		t=localtime((time_t *)&past_month);
		if(t->tm_mon==11){
			t->tm_mon=0;
			t->tm_year++;
		        }
		else
			t->tm_mon++;
		next_scheduled_log_rotation=mktime(t);
		break;
	default:
		break;
	        }

	/* recursively work our way backwards in time... */
	determine_log_rotation_times(&last_scheduled_log_rotation,archive-1);

	return;
        }


/* get the status of the netsaint process */
int get_netsaint_process_info(void){
	char input_buffer[MAX_INPUT_BUFFER];
	int process_state=STATE_OK;
	int result=STATE_OK;
	FILE *fp;

	/* if there is no check command defined, we don't know what's going on... */
	if(!strcmp(netsaint_check_command,"")){
		process_state=STATE_UNKNOWN;
		strcpy(netsaint_process_info,"No process check command defined");
	        }

	/* else run the check... */
	else{

		/* clear out the old check results */
		strcpy(netsaint_process_info,"");
		strcpy(input_buffer,"");

		/* run the process check command */
		fp=spopen(netsaint_check_command);
		if(fp==NULL){
			process_state=STATE_UNKNOWN;
			strncpy(netsaint_process_info,"Could not open pipe for process check command",sizeof(netsaint_process_info));
			netsaint_process_info[sizeof(netsaint_process_info)-1]='\x0';
	                }
		else{
			/* grab command output */
			fgets(input_buffer,MAX_INPUT_BUFFER-1,fp);
			result=spclose(fp);

			/* get the program return code */
			process_state=WEXITSTATUS(result);

			/* do some basic bounds checking on the return code */
			if(process_state<-1 || process_state>2)
				process_state=STATE_UNKNOWN;

			/* store output for later use */
			strncpy(netsaint_process_info,input_buffer,sizeof(netsaint_process_info));
			netsaint_process_info[sizeof(netsaint_process_info)-1]='\x0';
		        }
	        }

	netsaint_process_state=process_state;

	return process_state;
        }


/* calculates host state times */
void calculate_host_state_times(char *host_name,unsigned long *tmt, unsigned long *tup, float *ptup, unsigned long *tdown, float *ptdown, unsigned long *tunreachable, float *ptunreachable){
	time_t current_time;
	unsigned long total_program_time;
	unsigned long time_difference;
	unsigned long total_monitored_time;
	unsigned long time_up;
	unsigned long time_down;
	unsigned long time_unreachable;
	float percent_time_up;
	float percent_time_down;
	float percent_time_unreachable;
	hoststatus *temp_hoststatus;

	/* get host status info */
	temp_hoststatus=find_hoststatus(host_name);

	/* get the current time */
	time(&current_time);

	/* get total running time for NetSaint */
	total_program_time=(unsigned long)(current_time-program_start);

	if((unsigned long)temp_hoststatus->last_state_change==0L)
		time_difference=total_program_time;
	else
		time_difference=(unsigned long)(current_time-temp_hoststatus->last_state_change);

	
	time_up=temp_hoststatus->time_up;
	if(temp_hoststatus->status==HOST_UP)
		time_up+=time_difference;
	time_down=temp_hoststatus->time_down;
	if(temp_hoststatus->status==HOST_DOWN)
		time_down+=time_difference;
	time_unreachable=temp_hoststatus->time_unreachable;
	if(temp_hoststatus->status==HOST_UNREACHABLE)
		time_unreachable+=time_difference;

	total_monitored_time=time_up+time_down+time_unreachable;

	if(total_monitored_time==0L){
		percent_time_up=0.0;
		percent_time_down=0.0;
		percent_time_unreachable=0.0;
	        }
	else{
		percent_time_up=(float)(((float)time_up/(float)total_monitored_time)*100.0);
		percent_time_down=(float)(((float)time_down/(float)total_monitored_time)*100.0);
		percent_time_unreachable=(float)(((float)time_unreachable/(float)total_monitored_time)*100.0);
	        }

	*tmt=total_monitored_time;
	*tup=time_up;
	*ptup=percent_time_up;
	*tdown=time_down;
	*ptdown=percent_time_down;
	*tunreachable=time_unreachable;
	*ptunreachable=percent_time_unreachable;

	return;
        }


/* calculates service state times */
void calculate_service_state_times(char *host_name,char *service_desc,unsigned long *tmt, unsigned long *tok, float *ptok, unsigned long *twarn, float *ptwarn, unsigned long *tun, float *ptun, unsigned long *tcrit, float *ptcrit){
	time_t current_time;
	unsigned long total_program_time;
	unsigned long time_difference;
	unsigned long total_monitored_time;
	unsigned long time_ok;
	unsigned long time_warning;
	unsigned long time_unknown;
	unsigned long time_critical;
	float percent_time_ok;
	float percent_time_warning;
	float percent_time_unknown;
	float percent_time_critical;
	servicestatus *temp_svcstatus;

	/* get service status info */
	temp_svcstatus=find_servicestatus(host_name,service_desc);

	/* get the current time */
	time(&current_time);

	/* get total running time for NetSaint */
	total_program_time=(unsigned long)(current_time-program_start);

	if((unsigned long)temp_svcstatus->last_state_change==0L)
		time_difference=total_program_time;
	else
		time_difference=(unsigned long)(current_time-temp_svcstatus->last_state_change);

	
	time_ok=temp_svcstatus->time_ok;
	if(temp_svcstatus->status==SERVICE_OK || temp_svcstatus->status==SERVICE_RECOVERY)
	time_ok+=time_difference;

	time_warning=temp_svcstatus->time_warning;
	if(temp_svcstatus->status==SERVICE_WARNING)
		time_warning+=time_difference;

	time_unknown=temp_svcstatus->time_unknown;
	if(temp_svcstatus->status==SERVICE_UNKNOWN)
		time_unknown+=time_difference;

	time_critical=temp_svcstatus->time_critical;
	if(temp_svcstatus->status==SERVICE_CRITICAL || temp_svcstatus->status==SERVICE_UNREACHABLE || temp_svcstatus->status==SERVICE_HOST_DOWN)
		time_critical+=time_difference;

	total_monitored_time=time_ok+time_warning+time_unknown+time_critical;

	if(total_monitored_time==0L){
		percent_time_ok=0.0;
		percent_time_warning=0.0;
		percent_time_unknown=0.0;
		percent_time_critical=0.0;
	        }
	else{
		percent_time_ok=(float)(((float)time_ok/(float)total_monitored_time)*100.0);
		percent_time_warning=(float)(((float)time_warning/(float)total_monitored_time)*100.0);
		percent_time_unknown=(float)(((float)time_unknown/(float)total_monitored_time)*100.0);
		percent_time_critical=(float)(((float)time_critical/(float)total_monitored_time)*100.0);
	        }

	*tmt=total_monitored_time;
	*tok=time_ok;
	*twarn=time_warning;
	*tun=time_unknown;
	*tcrit=time_critical;
	*ptok=percent_time_ok;
	*ptwarn=percent_time_warning;
	*ptun=percent_time_unknown;
	*ptcrit=percent_time_critical;

	return;
        }


/**********************************************************
 ************** OBJECT SEARCH FUNCTIONS *******************
 **********************************************************/


/* find a specific contactgroup */
contactgroup * find_contactgroup(char *contactgroup_name){
	contactgroup *temp_contactgroup;

	for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){
		if(!strcmp(contactgroup_name,temp_contactgroup->group_name))
			return temp_contactgroup;
	        }
	
	return NULL;
        }


/* find a specific hostgroup */
hostgroup * find_hostgroup(char *hostgroup_name){
	hostgroup *temp_hostgroup;

	for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
		if(!strcmp(hostgroup_name,temp_hostgroup->group_name))
			return temp_hostgroup;
	        }
	
	return NULL;
        }


/* find the extended information for a given host */
hostextinfo * find_hostextinfo(char *host_name){
	hostextinfo *temp_hostextinfo;

	for(temp_hostextinfo=hostextinfo_list;temp_hostextinfo!=NULL;temp_hostextinfo=temp_hostextinfo->next){
		if(!strcmp(host_name,temp_hostextinfo->host_name))
			return temp_hostextinfo;
	        }
	
	return NULL;
        }


/* find a specific service */
service * find_service(char *host_name,char *svc_desc){
	service *temp_service;

	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
		if(!strcmp(temp_service->host_name,host_name) && !strcmp(temp_service->description,svc_desc))
			return temp_service;
	        }
	
	return NULL;
        }


/* find a specific host */
host *find_host(char *host_name){
	host *temp_host;

	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
		if(!strcmp(temp_host->name,host_name))
			return temp_host;
	        }

	return NULL;
        }


/* find a host status entry */
hoststatus *find_hoststatus(char *host_name){
	hoststatus *temp_hoststatus;

	for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next){
		if(!strcmp(temp_hoststatus->host_name,host_name))
			return temp_hoststatus;
	        }

	return NULL;
        }


/* find a service status entry */
servicestatus *find_servicestatus(char *host_name,char *svc_desc){
	servicestatus *temp_servicestatus;

	for(temp_servicestatus=servicestatus_list;temp_servicestatus!=NULL;temp_servicestatus=temp_servicestatus->next){
		if(!strcmp(temp_servicestatus->host_name,host_name) && !strcmp(temp_servicestatus->description,svc_desc))
			return temp_servicestatus;
	        }

	return NULL;
        }








