/***********************************************************************
 *
 * CONFIG.C - NetSaint Configuration CGI
 *
 * Written by Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 12-27-1999
 *
 * This CGI program will display various configuration information.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ***********************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include "../common/common.h"

#include "getcgi.h"
#include "cgiutils.h"
#include "auth.h"

extern char   main_config_file[MAX_FILENAME_LENGTH];
extern char   url_html_path[MAX_FILENAME_LENGTH];
extern char   url_docs_path[MAX_FILENAME_LENGTH];
extern char   url_images_path[MAX_FILENAME_LENGTH];
extern char   url_cgibin_path[MAX_FILENAME_LENGTH];

extern char html_table_bg_color[MAX_COLOR_LENGTH];
extern char html_table_fg_color[MAX_COLOR_LENGTH];
extern char html_table_bg_odd_color[MAX_COLOR_LENGTH];
extern char html_table_bg_even_color[MAX_COLOR_LENGTH];

extern host *host_list;
extern hostgroup *hostgroup_list;
extern contactgroup *contactgroup_list;
extern command *command_list;
extern timeperiod *timeperiod_list;
extern service *service_list;
extern contact *contact_list;


#define DISPLAY_NONE             0
#define DISPLAY_HOSTS            1
#define DISPLAY_HOSTGROUPS       2
#define DISPLAY_CONTACTS         3
#define DISPLAY_CONTACTGROUPS    4
#define DISPLAY_SERVICES         5
#define DISPLAY_TIMEPERIODS      6
#define DISPLAY_COMMANDS         7

void document_header(void);
void document_footer(void);
int process_cgivars(void);

void display_hosts(void);
void display_hostgroups(void);
void display_contacts(void);
void display_contactgroups(void);
void display_services(void);
void display_timeperiods(void);
void display_commands(void);

void unauthorized_message(void);


authdata current_authdata;

int display_type=DISPLAY_NONE;


int main(void){
	int error=FALSE;
	int result=OK;
	time_t t;
	char date_time[48];


	error=process_cgivars();

	document_header();

	reset_cgi_vars();

	result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
	if(result==ERROR){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
		printf("Error: Could not open CGI configuration file '%s' for reading!\n",DEFAULT_CGI_CONFIG_FILE);
		printf("</strong></font></div></p>\n");
		document_footer();
		return ERROR;
	        }

	result=read_main_config_file(main_config_file);
	if(result==ERROR){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
		printf("Error: Could not open main configuration file for reading!\n");
		printf("</strong></font></div></p>\n");
		document_footer();
		return ERROR;
	        }

	result=read_all_host_config_files(READ_HOSTGROUPS|READ_CONTACTGROUPS|READ_HOSTS);
	if(result==ERROR){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
		printf("Error: Could not open host configuration file(s) for reading!\n");
		printf("</strong></font></div></p>\n");
		document_footer();
		return ERROR;
                }

	/* begin top table */
	printf("<p>\n");
	printf("<table border=0>\n");
	printf("<tr>\n");
	printf("<td align=left valign=top>\n");

	/* left column of the first row */
	printf("<table border=1>\n");
	printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
	printf("<font size=+2><b>");
	printf("Configuration Information");
	printf("</b></font><br>\n");
	printf("<font size=-2>\n");
	time(&t);
	get_time_string(&t,date_time,sizeof(date_time));
	printf("Last Updated: %s<BR>",date_time);
	printf("NetSaint Network Monitor - <a href=\"http://www.netsaint.org\" target=\"_new\">www.netsaint.org</a><br>\n");
	printf("</font>\n");
	printf("</td></tr>\n");
	printf("</table>\n");
	printf("</td>\n");

	/* center column of top row */
	printf("<td align=left valign=center>\n");
	printf("</td>\n");

	/* right hand column of top row */
	printf("<td align=right valign=bottom>\n");
	printf("</td>\n");

	/* end of top table */
	printf("</tr>\n");
	printf("</table>\n");
	printf("</p>\n");


	/* get authentication information */
	get_authentication_information(&current_authdata);

	switch(display_type){
	case DISPLAY_HOSTS:
		display_hosts();
		break;
	case DISPLAY_HOSTGROUPS:
		display_hostgroups();
		break;
	case DISPLAY_CONTACTS:
		display_contacts();
		break;
	case DISPLAY_CONTACTGROUPS:
		display_contactgroups();
		break;
	case DISPLAY_SERVICES:
		display_services();
		break;
	case DISPLAY_TIMEPERIODS:
		display_timeperiods();
		break;
	case DISPLAY_COMMANDS:
		display_commands();
		break;
	default:
	        }

	document_footer();

	return OK;
        }




void document_header(void){
	char date_time[48];
	time_t t;

	time(&t);
	get_expire_time_string(&t,date_time,sizeof(date_time));

	printf("Cache-Control: no-store\n");
	printf("Pragma: no-cache\n");
	printf("Last-Modified: %s\n",date_time);
	printf("Expires: %s\n",date_time);
	printf("Content-type: text/html\r\n\r\n");

	printf("<html>\n");
	printf("<head>\n");
	printf("<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n");
	printf("<title>\n");
	printf("NetSaint Configuration\n");
	printf("</title>\n");
	printf("</head>\n");
	printf("<body>\n");

	return;
        }


void document_footer(void){

	printf("</body>\n");
	printf("</html>\n");

	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			x++;
			continue;
		        }

		/* we found the configuration type argument */
		else if(!strcmp(variables[x],"type")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* what information should we display? */
			if(!strcmp(variables[x],"hosts"))
				display_type=DISPLAY_HOSTS;
			else if(!strcmp(variables[x],"hostgroups"))
				display_type=DISPLAY_HOSTGROUPS;
			else if(!strcmp(variables[x],"contacts"))
				display_type=DISPLAY_CONTACTS;
			else if(!strcmp(variables[x],"contactgroups"))
				display_type=DISPLAY_CONTACTGROUPS;
			else if(!strcmp(variables[x],"services"))
				display_type=DISPLAY_SERVICES;
			else if(!strcmp(variables[x],"timeperiods"))
				display_type=DISPLAY_TIMEPERIODS;
			else if(!strcmp(variables[x],"commands"))
				display_type=DISPLAY_COMMANDS;
		        }

		/* we recieved an invalid argument */
		else
			error=TRUE;
	
	        }

	return error;
        }



void display_hosts(void){
	host *temp_host;
	int options=0;
	int odd=0;
	char time_string[16];
	int user_has_seen_something=FALSE;

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<H2>Host Definitions</H2>\n");
	printf("</div>\n");
	printf("</P>\n");

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=0>\n");
	printf("<TR>\n");
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Host Name</FONT></TH>",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Alias/Description</FONT></TH>",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Address</FONT></TH>",html_table_bg_color,html_table_fg_color);		
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Parent Host</FONT></TH>",html_table_bg_color,html_table_fg_color);		
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Max. Check Attempts</FONT></TH>",html_table_bg_color,html_table_fg_color);	
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Notification Interval</FONT></TH>",html_table_bg_color,html_table_fg_color);	
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Notification Options</FONT></TH>",html_table_bg_color,html_table_fg_color);	
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Notification Period</FONT></TH>",html_table_bg_color,html_table_fg_color);	
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Host Check Command</FONT></TH>",html_table_bg_color,html_table_fg_color);	
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Event Handler</FONT></TH>",html_table_bg_color,html_table_fg_color);	
	printf("</TR>\n");

	/* check all the hosts... */
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		/* check to see if user is authorized to view this host information... */
		if(is_authorized_for_host(temp_host,&current_authdata)==FALSE && is_authorized_for_configuration_information(&current_authdata)==FALSE)
			continue;

		user_has_seen_something=TRUE;

		if(odd){
			odd=0;
			printf("<TR bgcolor=%s>\n",html_table_bg_odd_color);
		        }
		else{
			odd=1;
			printf("<TR bgcolor=%s>\n",html_table_bg_even_color);
		        }

		printf("<TD>");
		printf("<table border=0 cellpadding=0 cellspacing=0 width=100%%>\n");
		printf("<tr>\n");
		printf("<TD align=left valign=top><a name=\"%s\"></a><font size=-1>%s&nbsp;</font></TD>\n",url_encode(temp_host->name),temp_host->name);
		printf("<TD align=right valign=top>\n");
		printf("<A HREF=\"%s?host=%s\"><IMG SRC=\"%s%s\" BORDER=\"0\" ALT=\"View Current Service Status Detail For This Host\"></A>\n",STATUS_CGI,url_encode(temp_host->name),url_images_path,STATUS_DETAIL_ICON);
		printf("<A HREF=\"%s?type=%d&host=%s\"><IMG SRC=\"%s%s\" BORDER=\"0\" ALT=\"View Extended Information For This Host\"></A>\n",EXTINFO_CGI,DISPLAY_HOST_INFO,url_encode(temp_host->name),url_images_path,XINFO_ICON);
		printf("<a href=\"%s?host=%s\"><img src=\"%s%s\" border=0 alt=\"View Status Map For This Host\"></a>\n",STATUSMAP_CGI,url_encode(temp_host->name),url_images_path,STATUSMAP_ICON);
		printf("<a href=\"%s?host=%s\"><img src=\"%s%s\" border=0 alt=\"View 3-D Status Map For This Host\"></a>\n",STATUSWORLD_CGI,url_encode(temp_host->name),url_images_path,STATUSWORLD_ICON);
		printf("</td>\n");
		printf("</tr>\n");
		printf("</table>\n");
		printf("</TD>\n");

		printf("<TD align=left valign=top><font size=-1>%s&nbsp;</font></TD>\n",temp_host->alias);
		printf("<TD align=left valign=top><font size=-1>%s&nbsp;</font></TD>\n",temp_host->address);
		printf("<TD align=left valign=top><font size=-1><a href=\"%s?type=hosts#%s\">%s</a>&nbsp;</font></TD>\n",CONFIG_CGI,url_encode(temp_host->parent_host),temp_host->parent_host);
		printf("<TD align=left valign=top><font size=-1>%d</font></TD>\n",temp_host->max_attempts);

		get_interval_time_string(temp_host->notification_interval,time_string,sizeof(time_string));
		printf("<TD align=left valign=\"top\"><FONT SIZE=-1>%s&nbsp;</FONT></TD>\n",time_string);

		printf("<TD align=left valign=top><font size=-1>");
		if(temp_host->notify_on_down){
			options=1;
			printf("<A HREF=\"%sstatuslevels.html#HOST DOWN\">Down</A>",url_docs_path);
		        }
		if(temp_host->notify_on_unreachable){
			printf("%s<A HREF=\"%sstatuslevels.html#HOST UNREACHABLE\">Unreachable</A>",(options)?", ":"",url_docs_path);
			options=1;
		        }
		if(temp_host->notify_on_recovery){
			printf("%s<A HREF=\"%sstatuslevels.html#RECOVERY\">Recovery</A>",(options)?", ":"",url_docs_path);
			options=1;
		        }
		if(!options)
			printf("None");
		printf("&nbsp;</font></TD>\n");
		printf("<TD align=left valign=top><font size=-1><a href=\"%s?type=timeperiods#%s\">%s</a>&nbsp;</font></TD>\n",CONFIG_CGI,url_encode(temp_host->notification_period),temp_host->notification_period);
		printf("<TD align=left valign=top><font size=-1><a href=\"%s?type=commands#%s\">%s</a>&nbsp;</font></TD>\n",CONFIG_CGI,url_encode(temp_host->host_check_command),temp_host->host_check_command);
		printf("<TD align=left valign=top><font size=-1><a href=\"%s?type=commands#%s\">%s</a>&nbsp;</font></TD>\n",CONFIG_CGI,url_encode(temp_host->event_handler),temp_host->event_handler);


		printf("</TR>\n");
	        }

	printf("</table>\n");
	printf("</div>\n");
	printf("</p>\n");

	/* display some helpful info if the user wasn't authorized for any hosts... */
	if(user_has_seen_something==FALSE && host_list!=NULL)
		unauthorized_message();

	return;
        }



void display_hostgroups(void){
	hostgroup *temp_hostgroup;
	int first_contactgroup;
	int first_host;
	char *current_contactgroup;
	char *current_host;
	char contact_members[MAX_CONTACTMEMBERS_LENGTH];
	char host_members[MAX_HOSTGROUPMEMBERS_LENGTH];
	int odd=0;

	/* see if user is authorized to view hostgroup information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<H2>Host Group Definitions</H2>\n");
	printf("</div>\n");
	printf("</P>\n");

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=0>\n");
	printf("<TR>\n");
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Group Name</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Description</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Contact Groups</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Hosts</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("</TR>\n");

	/* check all the hostgroups... */
	for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){

		if(odd){
			odd=0;
			printf("<TR bgcolor=%s>\n",html_table_bg_odd_color);
		        }
		else{
			odd=1;
			printf("<TR bgcolor=%s>\n",html_table_bg_even_color);
		        }

		printf("<TD valign=\"top\">");
		printf("<table border=0 cellspacing=3 cellpadding=0 width=100%%>");
		printf("<tr>");
		printf("<td align=left valign=top>");
		printf("<A NAME=\"%s\"></A><FONT SIZE=-1>%s</FONT></TD>",url_encode(temp_hostgroup->group_name),temp_hostgroup->group_name);
		printf("</td>");
		printf("<td align=right valign=center>");
		printf("<A HREF=\"%s%s?hostgroup=%s\"><IMG SRC=\"%s%s\" BORDER=\"0\" ALT=\"View Current Service Status Overview For This Host Group\"></A>\n",url_cgibin_path,STATUS_CGI,url_encode(temp_hostgroup->group_name),url_images_path,STATUS_OVERVIEW_ICON);
		printf("<A HREF=\"%s%s?hostgroup=%s&style=detail\"><IMG SRC=\"%s%s\" BORDER=\"0\" ALT=\"View Current Service Status Detail For This Host Group\"></A>\n",url_cgibin_path,STATUS_CGI,url_encode(temp_hostgroup->group_name),url_images_path,STATUS_DETAIL_ICON);
		printf("</td>");
		printf("</tr>");
		printf("</table>");
		printf("</TD>");

		printf("<TD valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_hostgroup->group_alias);
		printf("<TD valign=\"top\"><FONT SIZE=-1>");

		/* find all the contact groups for this hostgroup... */
		strncpy(contact_members,temp_hostgroup->contact_members,sizeof(contact_members));
		contact_members[sizeof(contact_members)-1]='\x0';
		current_contactgroup=strtok(contact_members,",");
		first_contactgroup=TRUE;
		while(current_contactgroup!=NULL){
			printf("%s<A HREF=\"%s?type=contactgroups#%s\"><FONT SIZE=-1>%s</FONT></A>\n",(first_contactgroup==TRUE)?"":", ",CONFIG_CGI,url_encode(current_contactgroup),current_contactgroup);
			first_contactgroup=FALSE;
			current_contactgroup=strtok(NULL,",");
		        }
		printf("</FONT></TD>\n");
		printf("<TD valign=\"top\"><FONT SIZE=-1>");

		/* find all the hosts that are members of this hostgroup... */
		strncpy(host_members,temp_hostgroup->members,sizeof(host_members));
		host_members[sizeof(host_members)-1]='\x0';
		current_host=strtok(host_members,",");
		first_host=TRUE;
		while(current_host!=NULL){
			printf("%s<A HREF=\"%s?type=hosts#%s\">%s</A>\n",(first_host==TRUE)?"":", ",CONFIG_CGI,url_encode(current_host),current_host);
			first_host=FALSE;
			current_host=strtok(NULL,",");
		        }

		printf("</FONT></TD>\n");
		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</div>\n");
	printf("</P>\n");

	return;
        }



void display_contacts(void){
	char commands[MAX_CONTACTCOMMANDS_LENGTH];
	char *current_command;
	contact *temp_contact;
	int odd=0;
	int found;
	int options;

	/* see if user is authorized to view contact information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	/* read in contact definitions... */
	read_all_host_config_files(READ_CONTACTS);

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<H2>Contact Definitions</H2>\n");
	printf("</div>\n");
	printf("</P>\n");

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=0>\n");
	printf("<TR>\n");
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Contact Name</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Alias</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Email Address</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Pager Address/Number</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Service Notification Options</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Host Notification Options</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Service Notification Period</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Host Notification Period</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Service Notification Commands</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Host Notification Commands</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("</TR>\n");

	/* check all contacts... */
	for(temp_contact=contact_list;temp_contact!=NULL;temp_contact=temp_contact->next){

		if(odd){
			odd=0;
			printf("<TR bgcolor=%s>\n",html_table_bg_odd_color);
		        }
		else{
			odd=1;
			printf("<TR bgcolor=%s>\n",html_table_bg_even_color);
		        }

		printf("<TD>\n");
		printf("<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0 WIDTH=100%%>\n");
		printf("<TR>\n");
		printf("<TD valign=\"top\" align=left><A NAME=\"%s\"></A><FONT SIZE=-1>%s</FONT></TD>\n",url_encode(temp_contact->name),temp_contact->name);
		printf("<TD valign=\"top\" align=right>");
		printf("<TD valign=\"top\"><A HREF=\"%s?contact=%s\"><IMG SRC=\"%s%s\" BORDER=\"0\" ALT=\"View All Notifications For This Contact\"></A></TD>\n",NOTIFICATIONS_CGI,url_encode(temp_contact->name),url_images_path,NOTIFICATION_ICON);
		printf("</TD>\n");
		printf("</TR>\n");
		printf("</TABLE>\n");
		printf("</TD>\n");

		printf("<TD valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_contact->alias);
		printf("<TD valign=\"top\"><FONT SIZE=-1><A HREF=\"mailto:%s\">%s</A>&nbsp;</FONT></TD>\n",temp_contact->email,temp_contact->email);
		printf("<TD valign=\"top\"><FONT SIZE=-1>%s&nbsp;</FONT></TD>\n",temp_contact->pager);

		printf("<TD align=left valign=\"top\"><FONT SIZE=-1>");
		options=0;
		if(temp_contact->notify_on_service_warning){
			options=1;
			printf("<A HREF=\"%sstatuslevels.html#WARNING\">Warning</A>",url_docs_path);
		        }
		if(temp_contact->notify_on_service_critical){
			printf("%s<A HREF=\"%sstatuslevels.html#CRITICAL\">Critical</A>",(options)?", ":"",url_docs_path);
			options=1;
		        }
		if(temp_contact->notify_on_service_recovery){
			printf("%s<A HREF=\"%sstatuslevels.html#RECOVERY\">Recovery</A>",(options)?", ":"",url_docs_path);
			options=1;
		        }
		if(!options)
			printf("None");
		printf("&nbsp;</FONT></TD>\n");

		printf("<TD align=left valign=\"top\"><FONT SIZE=-1>");
		options=0;
		if(temp_contact->notify_on_host_down){
			options=1;
			printf("<A HREF=\"%sstatuslevels.html#HOST DOWN\">Down</A>",url_docs_path);
		        }
		if(temp_contact->notify_on_host_unreachable){
			printf("%s<A HREF=\"%sstatuslevels.html#HOST UNREACHABLE\">Unreachable</A>",(options)?", ":"",url_docs_path);
			options=1;
		        }
		if(temp_contact->notify_on_host_recovery){
			printf("%s<A HREF=\"%sstatuslevels.html#RECOVERY\">Recovery</A>",(options)?", ":"",url_docs_path);
			options=1;
		        }
		if(!options)
			printf("None");
		printf("&nbsp;</FONT></TD>\n");

		printf("<TD valign=\"top\"><FONT SIZE=-1><A HREF=\"%s?type=timeperiods#%s\">%s</A>&nbsp;</FONT></TD>\n",CONFIG_CGI,url_encode(temp_contact->service_notification_period),temp_contact->service_notification_period);
		printf("<TD valign=\"top\"><FONT SIZE=-1><A HREF=\"%s?type=timeperiods#%s\">%s</A>&nbsp;</FONT></TD>\n",CONFIG_CGI,url_encode(temp_contact->host_notification_period),temp_contact->host_notification_period);

		printf("<TD valign=\"top\"><FONT SIZE=-1>");
		strncpy(commands,temp_contact->commands,sizeof(commands));
		commands[sizeof(commands)-1]='\x0';
		found=0;
		for(current_command=strtok(commands,",");current_command!=NULL;current_command=strtok(NULL,",")){
			printf("%s<A HREF=\"%s?type=commmands#%s\">%s</A>",(found)?", ":"",CONFIG_CGI,url_encode(current_command),current_command);
			found=1;
		        }
		printf("&nbsp;</FONT></TD>\n");

		printf("<TD valign=\"top\"><FONT SIZE=-1>");
		strncpy(commands,temp_contact->host_commands,sizeof(commands));
		commands[sizeof(commands)-1]='\x0';
		found=0;
		for(current_command=strtok(commands,",");current_command!=NULL;current_command=strtok(NULL,",")){
			printf("%s<A HREF=\"%s?type=commmands#%s\">%s</A>",(found)?", ":"",CONFIG_CGI,url_encode(current_command),current_command);
			found=1;
		        }
		printf("&nbsp;</FONT></TD>\n");

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</div>\n");
	printf("</P>\n");

	return;
        }



void display_contactgroups(void){
	int first_contact;
	contactgroup *temp_contactgroup;
	char *current_contact;
	char contact_members[MAX_CONTACTMEMBERS_LENGTH];
	int odd=0;

	/* see if user is authorized to view contactgroup information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<H2>Contact Group Definitions</H2>\n");
	printf("</div>\n");
	printf("</P>\n");

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=0>\n");
	printf("<TR>\n");
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Group Name</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Description</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Members</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("</TR>\n");

	/* check all the contact groups... */
	for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){

		if(odd){
			odd=0;
			printf("<TR bgcolor=%s>\n",html_table_bg_odd_color);
		        }
		else{
			odd=1;
			printf("<TR bgcolor=%s>\n",html_table_bg_even_color);
		        }

		printf("<TD valign=\"top\"><A NAME=\"%s\"></A><FONT SIZE=-1>%s</FONT></TD>\n",url_encode(temp_contactgroup->group_name),temp_contactgroup->group_name);
		printf("<TD valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_contactgroup->group_alias);
		printf("<TD valign=\"top\"><FONT SIZE=-1>");

		/* find all the contact who are members of this contact group... */
		strncpy(contact_members,temp_contactgroup->members,sizeof(contact_members));
		contact_members[sizeof(contact_members)-1]='\x0';
		current_contact=strtok(contact_members,",");
		first_contact=TRUE;
		while(current_contact!=NULL){
			printf("%s<A HREF=\"%s?type=contacts#%s\"><FONT SIZE=-1>%s</FONT></A>\n",(first_contact==TRUE)?"":", ",CONFIG_CGI,url_encode(current_contact),current_contact);
			first_contact=FALSE;
			current_contact=strtok(NULL,",");
		        }
		printf("</FONT></TD>\n");
		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</div>\n");
	printf("</P>\n");

	return;
        }



void display_services(void){
	char contact_members[MAX_CONTACTMEMBERS_LENGTH];
	char *current_contact;
	char command[MAX_COMMANDLINE_LENGTH];
	char *command_name;
	service *temp_service;
	int found;
	int options;
	int odd=0;
	char time_string[16];
	int user_has_seen_something=FALSE;

	/* read in service definitions... */
	read_all_host_config_files(READ_SERVICES);

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<H2>Service Definitions</H2>\n");
	printf("</div>\n");
	printf("</P>\n");

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=0>\n");
	printf("<TR>\n");
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Host</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Service Description</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Check Period</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Normal Check Interval</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Retry Check Interval</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Max. Check Attempts</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Contact Groups</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Notification Interval</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Notification Options</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Notification Period</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Event Handler</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Check Command</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("</TR>\n");

	/* check all the services... */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		/* check to see if user is authorized to view this service information... */
		if(is_authorized_for_service(temp_service,&current_authdata)==FALSE && is_authorized_for_configuration_information(&current_authdata)==FALSE)
			continue;

		user_has_seen_something=TRUE;

		if(odd){
			odd=0;
			printf("<TR bgcolor=%s>\n",html_table_bg_odd_color);
		        }
		else{
			odd=1;
			printf("<TR bgcolor=%s>\n",html_table_bg_even_color);
		        }

		printf("<TD valign=\"top\"><A NAME=\"%s;",url_encode(temp_service->host_name));
		printf("%s\"></A>",url_encode(temp_service->description));
		printf("<FONT SIZE=-1><A HREF=\"%s?type=hosts#%s\">%s</A></FONT></TD>\n",CONFIG_CGI,url_encode(temp_service->host_name),temp_service->host_name);
		
		printf("<TD valign=\"top\">\n");
		printf("<TABLE border=0 cellpadding=0 cellspacing=0 width=100%%>\n");
		printf("<TR>\n");
		printf("<TD align=left valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_service->description);
		printf("<TD align=right valign=\"top\">");
		printf("<A HREF=\"%s?type=%d&host=%s",EXTINFO_CGI,DISPLAY_SERVICE_INFO,url_encode(temp_service->host_name));
		printf("&service=%s\"><IMG SRC=\"%s%s\" BORDER=\"0\" ALT=\"View Extended Information For This Service\"></A>\n",url_encode(temp_service->description),url_images_path,XINFO_ICON);
		printf("</TD>\n");
		printf("</TR>\n");
		printf("</TABLE>\n");
		printf("</TD>\n");

		printf("<TD align=left valign=\"top\"><FONT SIZE=-1><A HREF=\"%s?type=timeperiods#%s\">%s</A></FONT></TD>\n",CONFIG_CGI,url_encode(temp_service->check_period),temp_service->check_period);

		get_interval_time_string(temp_service->check_interval,time_string,sizeof(time_string));
		printf("<TD align=left valign=\"top\"><FONT SIZE=-1>%s&nbsp;</FONT></TD>\n",time_string);
		get_interval_time_string(temp_service->retry_interval,time_string,sizeof(time_string));
		printf("<TD align=left valign=\"top\"><FONT SIZE=-1>%s&nbsp;</FONT></TD>\n",time_string);

		printf("<TD align=left valign=\"top\"><FONT SIZE=-1>%d</FONT></TD>\n",temp_service->max_attempts);

		printf("<TD align=left valign=\"top\"><FONT SIZE=-1>");
		found=0;
		strncpy(contact_members,temp_service->contact_members,sizeof(contact_members));
		contact_members[sizeof(contact_members)-1]='\x0';
		for(current_contact=strtok(contact_members,",");current_contact!=NULL;current_contact=strtok(NULL,",")){
			printf("%s<A HREF=\"%s?type=contactgroups#%s\">%s</A>",(found)?", ":"",CONFIG_CGI,url_encode(current_contact),current_contact);
			found=1;
		        }
		printf("&nbsp;</FONT></TD>\n");

		get_interval_time_string(temp_service->notification_interval,time_string,sizeof(time_string));
		printf("<TD align=left valign=\"top\"><FONT SIZE=-1>%s&nbsp;</FONT></TD>\n",time_string);

		printf("<TD align=left valign=\"top\"><FONT SIZE=-1>");
		options=0;
		if(temp_service->notify_on_warning){
			options=1;
			printf("<A HREF=\"%sstatuslevels.html#WARNING\">Warning</A>",url_docs_path);
		        }
		if(temp_service->notify_on_critical){
			printf("%s<A HREF=\"%sstatuslevels.html#CRITICAL\">Critical</A>",(options)?", ":"",url_docs_path);
			options=1;
		        }
		if(temp_service->notify_on_recovery){
			printf("%s<A HREF=\"%sstatuslevels.html#RECOVERY\">Recovery</A>",(options)?", ":"",url_docs_path);
			options=1;
		        }
		if(!options)
			printf("None");
		printf("&nbsp;</FONT></TD>\n");

		printf("<TD align=left valign=\"top\"><FONT SIZE=-1><A HREF=\"%s?type=timeperiods#%s\">%s</A></FONT></TD>\n",CONFIG_CGI,url_encode(temp_service->notification_period),temp_service->notification_period);

		printf("<TD align=left valign=\"top\"><FONT SIZE=-1><A HREF=\"%s?type=commands#%s\">%s</A>&nbsp;</FONT></TD>\n",CONFIG_CGI,url_encode(temp_service->event_handler),temp_service->event_handler);
		strncpy(command,temp_service->command,sizeof(command));
		command[sizeof(command)-1]='\x0';
		command_name=strtok(command,"!");
		printf("<TD align=left valign=\"top\"><FONT SIZE=-1><A HREF=\"%s?type=commands#%s\">%s</A>&nbsp;</FONT></TD>\n",CONFIG_CGI,url_encode(command_name),temp_service->command);

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</div>\n");
	printf("</P>\n");

	/* display some helpful info if the user wasn't authorized for any services... */
	if(user_has_seen_something==FALSE && service_list!=NULL)
		unauthorized_message();

	return;
        }



void display_timeperiods(void){
	char ranges[MAX_TIMEPERIODRANGE_LENGTH];
	timeperiod *temp_timeperiod;
	char *current_range;
	int found;
	int odd=0;

	/* see if user is authorized to view time period information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	/* read in time period definitions... */
	read_all_host_config_files(READ_TIMEPERIODS);

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<H2>Time Period Definitions</H2>\n");
	printf("</div>\n");
	printf("</P>\n");

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=0>\n");
	printf("<TR>\n");
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Time Period Name</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Alias Description</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Sunday Time Ranges</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Monday Time Ranges</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Tuesday Time Ranges</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Wednesday Time Ranges</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Thursday Time Ranges</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Friday Time Ranges</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Saturday Time Ranges</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("</TR>\n");

	/* check all the time periods... */
	for(temp_timeperiod=timeperiod_list;temp_timeperiod!=NULL;temp_timeperiod=temp_timeperiod->next){

		if(odd){
			odd=0;
			printf("<TR bgcolor=%s>\n",html_table_bg_odd_color);
		        }
		else{
			odd=1;
			printf("<TR bgcolor=%s>\n",html_table_bg_even_color);
		        }

		printf("<TD valign=\"top\"><A NAME=\"%s\"></A><FONT SIZE=-1>%s</FONT></TD>\n",url_encode(temp_timeperiod->name),temp_timeperiod->name);
		printf("<TD valign=\"top\"><FONT SIZE=-1>%s</FONT></TD>\n",temp_timeperiod->alias);

		printf("<TD valign=\"top\"><FONT SIZE=-1>");
		found=0;
		strncpy(ranges,temp_timeperiod->sun_range,sizeof(ranges));
		ranges[sizeof(ranges)-1]='\x0';
		for(current_range=strtok(ranges,",");current_range!=NULL;current_range=strtok(NULL,",")){
			printf("%s%s",(found)?", ":"",current_range);
			found=1;
		        }
		printf("&nbsp;</FONT></TD>\n");

		printf("<TD valign=\"top\"><FONT SIZE=-1>");
		found=0;
		strncpy(ranges,temp_timeperiod->mon_range,sizeof(ranges));
		ranges[sizeof(ranges)-1]='\x0';
		for(current_range=strtok(ranges,",");current_range!=NULL;current_range=strtok(NULL,",")){
			printf("%s%s",(found)?", ":"",current_range);
			found=1;
		        }
		printf("&nbsp;</FONT></TD>\n");

		printf("<TD valign=\"top\"><FONT SIZE=-1>");
		found=0;
		strncpy(ranges,temp_timeperiod->tue_range,sizeof(ranges));
		ranges[sizeof(ranges)-1]='\x0';
		for(current_range=strtok(ranges,",");current_range!=NULL;current_range=strtok(NULL,",")){
			printf("%s%s",(found)?", ":"",current_range);
			found=1;
		        }
		printf("&nbsp;</FONT></TD>\n");

		printf("<TD valign=\"top\"><FONT SIZE=-1>");
		found=0;
		strncpy(ranges,temp_timeperiod->wed_range,sizeof(ranges));
		ranges[sizeof(ranges)-1]='\x0';
		for(current_range=strtok(ranges,",");current_range!=NULL;current_range=strtok(NULL,",")){
			printf("%s%s",(found)?", ":"",current_range);
			found=1;
		        }
		printf("&nbsp;</FONT></TD>\n");

		printf("<TD valign=\"top\"><FONT SIZE=-1>");
		found=0;
		strncpy(ranges,temp_timeperiod->thu_range,sizeof(ranges));
		ranges[sizeof(ranges)-1]='\x0';
		for(current_range=strtok(ranges,",");current_range!=NULL;current_range=strtok(NULL,",")){
			printf("%s%s",(found)?", ":"",current_range);
			found=1;
		        }
		printf("&nbsp;</FONT></TD>\n");

		printf("<TD valign=\"top\"><FONT SIZE=-1>");
		found=0;
		strncpy(ranges,temp_timeperiod->fri_range,sizeof(ranges));
		ranges[sizeof(ranges)-1]='\x0';
		for(current_range=strtok(ranges,",");current_range!=NULL;current_range=strtok(NULL,",")){
			printf("%s%s",(found)?", ":"",current_range);
			found=1;
		        }
		printf("&nbsp;</FONT></TD>\n");

		printf("<TD valign=\"top\"><FONT SIZE=-1>");
		found=0;
		strncpy(ranges,temp_timeperiod->sat_range,sizeof(ranges));
		ranges[sizeof(ranges)-1]='\x0';
		for(current_range=strtok(ranges,",");current_range!=NULL;current_range=strtok(NULL,",")){
			printf("%s%s",(found)?", ":"",current_range);
			found=1;
		        }
		printf("&nbsp;</FONT></TD>\n");

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</div>\n");
	printf("</P>\n");

	return;
        }



void display_commands(void){
	command *temp_command;
	int odd=0;

	/* see if user is authorized to view command information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	/* read in command definitions... */
	read_all_host_config_files(READ_COMMANDS);

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<H2>Command Definitions</H2>\n");
	printf("</div>\n");
	printf("</P>\n");

	printf("<P>\n");
	printf("<div align=\"center\">\n");
	printf("<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=0>\n");
	printf("<TR>\n");
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Command Name</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Command Line</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("</TR>\n");

	/* check all commands */
	for(temp_command=command_list;temp_command!=NULL;temp_command=temp_command->next){

		if(odd){
			odd=0;
			printf("<TR bgcolor=%s>\n",html_table_bg_odd_color);
		        }
		else{
			odd=1;
			printf("<TR bgcolor=%s>\n",html_table_bg_even_color);
		        }

		printf("<TD valign=\"top\"><A NAME=\"%s\"></A><FONT SIZE=-1>%s</FONT></TD>\n",url_encode(temp_command->name),temp_command->name);
		printf("<TD valign=\"top\"><FONT SIZE=-1 COLOR=\"%s\">%s</FONT></TD>\n",HTML_COMMANDLINE_COLOR,temp_command->command_line);
		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</div>\n");
	printf("</P>\n");

	return;
        }



void unauthorized_message(void){

	printf("<p><div align=\"center\">\n");
	printf("<strong><font color=red>It appears as though you do not have permission to view the configuration information you requested...</font></strong><br><br>\n");
	printf("If you believe this is an error, check the HTTP server authentication requirements for accessing this CGI<br>");
	printf("and check the authorization options in your CGI configuration file.\n");
	printf("</div></p>\n");

	return;
	}
