/***********************************************************************
 *
 * HISTORY.C - NetSaint History CGI
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 11-05-1999
 *
 * This CGI program will display the history for the specified host.
 * If no host is specified, the history for all hosts will be displayed.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ***********************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include "../common/common.h"

#include "getcgi.h"
#include "cgiutils.h"
#include "auth.h"
#include "lifo.h"

#define DISPLAY_HOSTS			0

#define SERVICE_HISTORY			0
#define HOST_HISTORY			1

#define STATE_ALL			0
#define STATE_SOFT			1
#define STATE_HARD			2

void get_history(void);

void document_header(void);
void document_footer(void);
int process_cgivars(void);

extern char main_config_file[MAX_FILENAME_LENGTH];
extern char url_images_path[MAX_FILENAME_LENGTH];

extern char html_table_bg_odd_color[MAX_COLOR_LENGTH];
extern char html_table_bg_even_color[MAX_COLOR_LENGTH];

extern int log_rotation_method;

extern time_t last_scheduled_log_rotation;
extern time_t this_scheduled_log_rotation;

authdata current_authdata;

char log_file_to_use[MAX_FILENAME_LENGTH];
int log_archive=0;

int show_all_hosts=FALSE;
char host_name[MAX_HOSTNAME_LENGTH]="";
int display_type=DISPLAY_HOSTS;
int use_lifo=TRUE;

int history_options=HISTORY_ALL;
int state_options=STATE_ALL;


int main(void){
	int error=FALSE;
	int result=OK;
	time_t t;
	char date_time[48];

	/* get the variables passed to us */
	error=process_cgivars();

	/* if there was an error with the variables, print out an error message */
	if(error==TRUE){
		document_header();
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("Invalid argument supplied!");
		printf("</strong></font></div></p>\n");
		document_footer();
	        }

	else{

		document_header();

		reset_cgi_vars();

		result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open CGI configuration file '%s' for reading!\n",DEFAULT_CGI_CONFIG_FILE);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_main_config_file(main_config_file);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open main configuration file '%s' for reading!\n",main_config_file);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_all_host_config_files(READ_HOSTGROUPS|READ_CONTACTGROUPS|READ_HOSTS|READ_SERVICES);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open host configuration file(s) for reading!\n");
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
                        }

		get_log_archive_to_use(log_archive,log_file_to_use,(int)sizeof(log_file_to_use));

		/* begin top table */
		printf("<p>\n");
		printf("<table border=0 width=100%%>\n");
		printf("<tr>\n");
		printf("<td align=left valign=top>\n");

		printf("<table border=0>\n");
		printf("<tr><td>\n");

		/* left column of the first row */
		printf("<table border=1>\n");
		printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
		
		printf("<font size=+2><b>");
		if(log_rotation_method==LOG_ROTATION_NONE || log_archive==0)
			printf("Current Alert History");
		else
			printf("Archived Alert History");
		printf("</b></font><br>\n");
		printf("<font size=-2>\n");
		time(&t);
		get_time_string(&t,date_time,sizeof(date_time));
		printf("Last Updated: %s<BR>",date_time);
		printf("NetSaint Network Monitor - <a href=\"http://www.netsaint.org\" target=\"_new\">www.netsaint.org</a><br>\n");
		printf("</font>\n");
		printf("</td></tr>\n");

		if(show_all_hosts==FALSE){
			printf("<tr><td bgcolor=%s>\n",html_table_bg_even_color);
			printf("<font size=-2>\n");
			printf("<a href=\"%s?host=all\">View history for all hosts</a>\n",HISTORY_CGI);
			printf("</font>\n");
			printf("</td></tr>\n");
	                }

		printf("</table>\n");
		printf("</td>\n");

		printf("<td width=30></td>\n");

		/* middle column of top row */
		printf("<td align=left valign=center>\n");
		printf("<font size=+2><b>");
		if(show_all_hosts==TRUE)
			printf("History For All Hosts\n");
		else
			printf("History For Host \"%s\"\n",host_name);
		printf("</b></font>\n");
		printf("</td>\n");

		printf("</tr>\n");
		printf("</table>\n");
		printf("</td>\n");

		/* right hand column of top row */
		printf("<td align=right valign=bottom>\n");

		printf("<table border=0>\n");
		printf("<form method=\"GET\" action=\"%s\">\n",HISTORY_CGI);
		printf("<input type='hidden' name='host' value='%s'>\n",(show_all_hosts==TRUE)?"all":host_name);
		printf("<input type='hidden' name='archive' value='%d'>\n",log_archive);
		printf("<tr>\n");
		printf("<td align=left><font size=-2>State type options:</font></td>\n");
		printf("<td></td>\n");
		printf("</tr>\n");
		printf("<tr>\n");
		printf("<td align=left><font size=-1><select name='statetype'>\n");
		printf("<option value=%d %s>All state types\n",STATE_ALL,(state_options==STATE_ALL)?"selected":"");
		printf("<option value=%d %s>Soft states\n",STATE_SOFT,(state_options==STATE_SOFT)?"selected":"");
		printf("<option value=%d %s>Hard states\n",STATE_HARD,(state_options==STATE_HARD)?"selected":"");
		printf("</select></font></td>\n");
		printf("<td></td>\n");
		printf("</tr>\n");
		printf("<tr>\n");
		printf("<td align=left><font size=-2>History detail level for %s host%s:</font></td>",(show_all_hosts==TRUE)?"all":"this",(show_all_hosts==TRUE)?"s":"");
		printf("<td></td>\n");
		printf("</tr>\n");
		printf("<tr>\n");
		printf("<td align=left><font size=-1><select name='type'>\n");
		printf("<option value=%d %s>All alerts\n",HISTORY_ALL,(history_options==HISTORY_ALL)?"selected":"");
		printf("<option value=%d %s>All service alerts\n",HISTORY_SERVICE_ALL,(history_options==HISTORY_SERVICE_ALL)?"selected":"");
		printf("<option value=%d %s>All host alerts\n",HISTORY_HOST_ALL,(history_options==HISTORY_HOST_ALL)?"selected":"");
		printf("<option value=%d %s>Service warning\n",HISTORY_SERVICE_WARNING,(history_options==HISTORY_SERVICE_WARNING)?"selected":"");
		printf("<option value=%d %s>Service unknown\n",HISTORY_SERVICE_UNKNOWN,(history_options==HISTORY_SERVICE_UNKNOWN)?"selected":"");
		printf("<option value=%d %s>Service critical\n",HISTORY_SERVICE_CRITICAL,(history_options==HISTORY_SERVICE_CRITICAL)?"selected":"");
		printf("<option value=%d %s>Service recovery\n",HISTORY_SERVICE_RECOVERY,(history_options==HISTORY_SERVICE_RECOVERY)?"selected":"");
		printf("<option value=%d %s>Host down\n",HISTORY_HOST_DOWN,(history_options==HISTORY_HOST_DOWN)?"selected":"");
		printf("<option value=%d %s>Host unreachable\n",HISTORY_HOST_UNREACHABLE,(history_options==HISTORY_HOST_UNREACHABLE)?"selected":"");
		printf("<option value=%d %s>Host recovery\n",HISTORY_HOST_RECOVERY,(history_options==HISTORY_HOST_RECOVERY)?"selected":"");
		printf("</select></font></td>\n");
		printf("<td></td>\n");
		printf("</tr>\n");
		printf("<tr>\n");
		printf("<td align=left><font size=-2>Older Entries First:</font></td>\n");
		printf("<td></td>\n");
		printf("</tr>\n");
		printf("<tr>\n");
		printf("<td align=left valign=bottom><font size=-1><input type='checkbox' name='oldestfirst' %s></font></td>",(use_lifo==FALSE)?"checked":"");
		printf("<td align=left><font size=-1><input type='submit' value='Update'></font></td>\n");
		printf("</tr>\n");
		printf("</form>\n");
		printf("</table>\n");

		printf("</td>\n");

		/* end of top table */
		printf("</tr>\n");
		printf("</table>\n");
		printf("</p>\n");


		if(log_rotation_method!=LOG_ROTATION_NONE){
			printf("<p>\n");
			printf("<div align=\"center\">\n");
			printf("<table border=0>\n");
			printf("<tr>\n");
			printf("<td>\n");

			printf("<table border=0 cellspacing=20 cellpadding=0>\n");
			printf("<tr>\n");
			printf("<td align=center valign=bottom>\n");
			printf("<font size=-1>");
			if(log_archive==0){
				printf("<font size=-1 color=red>Latest Archive</font><br>");
				printf("<a href=\"%s?archive=1&host=%s&type=%d&statetype=%d%s\"><img src=\"%s%s\" border=0 alt=\"Latest Archive\"></a>",HISTORY_CGI,url_encode(host_name),history_options,state_options,(use_lifo==FALSE)?"&oldestfirst":"",url_images_path,LEFT_ARROW_ICON);
		                }
			else{
				printf("<font size=-1 color=red>Earlier Archive</font><br>");
				printf("<a href=\"%s?archive=%d&host=%s&type=%d&statetype=%d%s\"><img src=\"%s%s\" border=0 alt=\"Earlier Archive\"></a>",HISTORY_CGI,log_archive+1,url_encode(host_name),history_options,state_options,(use_lifo==FALSE)?"&oldestfirst":"",url_images_path,LEFT_ARROW_ICON);
		                }
			printf("</font>");
			printf("</td>\n");

			if(log_archive!=0){
				printf("<td align=center>\n");
				get_time_string((unsigned long*)&last_scheduled_log_rotation,date_time,sizeof(date_time));
				printf("<font size=-2><strong>%s</strong>",date_time);
				printf("<br>to<br>");
				get_time_string((unsigned long*)&this_scheduled_log_rotation,date_time,sizeof(date_time));
				printf("<strong>%s</strong></font>",date_time);

				printf("</td>\n");
				printf("<td align=center valign=bottom>\n");

				printf("<font size=-1>");
				if(log_archive==1){
					printf("<font size=-1 color=red>Current History</font><br>");
					printf("<a href=\"%s?host=%s&type=%d&statetype=%d%s\"><img src=\"%s%s\" border=0 alt=\"Current History\"></a>",HISTORY_CGI,url_encode(host_name),history_options,state_options,(use_lifo==FALSE)?"&oldestfirst":"",url_images_path,RIGHT_ARROW_ICON);
				        }
				else{
					printf("<font size=-1 color=red>More Recent Archive</font><br>");
					printf("<a href=\"%s?archive=%d&host=%s&type=%d&statetype=%d%s\"><img src=\"%s%s\" border=0 alt=\"More Recent Archive\"></a>",HISTORY_CGI,log_archive-1,url_encode(host_name),history_options,state_options,(use_lifo==FALSE)?"&oldestfirst":"",url_images_path,RIGHT_ARROW_ICON);
				        }
				printf("</font>");
				printf("</td>\n");
		                }

			printf("</tr>\n");
			printf("</table>\n");
	
			printf("</td>\n");
			printf("</tr>\n");
			printf("</table>\n");
			printf("</div>\n");
			printf("</p>\n");
	                }

		/* get authentication information */
		get_authentication_information(&current_authdata);

		get_history();

		document_footer();
	        }

	return OK;
        }



void document_header(void){
	char date_time[48];
	time_t current_time;
	time_t expire_time;


	printf("Cache-Control: no-store\n");
	printf("Pragma: no-cache\n");

	time(&current_time);
	get_expire_time_string(&current_time,date_time,sizeof(date_time));
	printf("Last-Modified: %s\n",date_time);

	expire_time=(time_t)0L;
	get_expire_time_string(&expire_time,date_time,sizeof(date_time));
	printf("Expires: %s\n",date_time);

	printf("Content-type: text/html\n\n");

	printf("<html>\n");
	printf("<head>\n");
	printf("<title>\n");
	printf("NetSaint History\n");
	printf("</title>\n");
	printf("</head>\n");
	printf("<body bgcolor=\"%s\">\n",HTML_BACKGROUND_COLOR);

	return;
        }


void document_footer(void){

	printf("</body>\n");
	printf("</html>\n");

	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1)
			continue;

		/* we found the host argument */
		else if(!strcmp(variables[x],"host")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_HOSTNAME_LENGTH-1){
				strcpy(host_name,"all");
			        }
			else{
				strncpy(host_name,variables[x],sizeof(host_name));
				host_name[sizeof(host_name)-1]='\x0';
			        }

			display_type=DISPLAY_HOSTS;

			if(!strcmp(host_name,"all"))
				show_all_hosts=TRUE;
			else
				show_all_hosts=FALSE;
		        }
	
	
		/* we found the history type argument */
		else if(!strcmp(variables[x],"type")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			history_options=atoi(variables[x]);
		        }
	
		/* we found the history state type argument */
		else if(!strcmp(variables[x],"statetype")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			state_options=atoi(variables[x]);
		        }
	
	
		/* we found the log archive argument */
		else if(!strcmp(variables[x],"archive")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			log_archive=atoi(variables[x]);
		        }

		/* we found the order argument */
		else if(!strcmp(variables[x],"oldestfirst")){
			use_lifo=FALSE;
		        }

	        }

	return error;
        }



void get_history(void){
	FILE *fp=NULL;
	char image[MAX_INPUT_BUFFER];
	char image_alt[MAX_INPUT_BUFFER];
	char input_buffer[MAX_INPUT_BUFFER];
	char input_buffer2[MAX_INPUT_BUFFER];
	char match1[MAX_INPUT_BUFFER];
	char match2[MAX_INPUT_BUFFER];
	int found_line=FALSE;
	int system_message=FALSE;
	int display_line=FALSE;
	time_t t;
	char date_time[48];
	char *temp_buffer;
	int history_type=SERVICE_HISTORY;
	int history_detail_type=HISTORY_SERVICE_CRITICAL;
	char entry_host_name[MAX_HOSTNAME_LENGTH];
	char entry_service_desc[MAX_SERVICEDESC_LENGTH];
	host *temp_host;
	service *temp_service;
	int result;
//
// agb 09/02/99
// Added last_message_date, current_message_date, and time pointer to display
// a break every hour in the history display.  This break would contain the
// current date and hour of all messages to follow.
//
	char last_message_date[MAX_INPUT_BUFFER]="";
	char current_message_date[MAX_INPUT_BUFFER]="";
	struct tm *time_ptr;
//


	if(use_lifo==TRUE){
		result=read_file_into_lifo(log_file_to_use);
		if(result!=LIFO_OK){
			if(result==LIFO_ERROR_MEMORY){
				printf("<p>\n");
				printf("<div align=\"center\">\n");
				printf("<font color=\"red\"><strong>");
				printf("<blink>Not enough memory to reverse log file - displaying history in natural order...</blink>");
				printf("</strong></font>\n");
				printf("</div>\n");
				printf("</p>\n");
			        }
			else if(result==LIFO_ERROR_FILE){
				printf("<p>\n");
				printf("<HR>\n");
				printf("<br clear=\"all\">\n");
				printf("<div align=\"center\"><font color=\"red\"><strong>");
				printf("Error: Cannot open log file '%s' for reading!",log_file_to_use);
				printf("</strong></font></div>\n");
				printf("<br>\n");
				printf("<HR>\n");
				printf("</p>\n");
				return;
			        }
			use_lifo=FALSE;
		        }
	        }

	if(use_lifo==FALSE){

		fp=fopen(log_file_to_use,"r");
		if(fp==NULL){
			printf("<p>\n");
			printf("<HR>\n");
			printf("<br clear=\"all\">\n");
			printf("<div align=\"center\"><font color=\"red\"><strong>");
			printf("Error: Cannot open log file '%s' for reading!",log_file_to_use);
			printf("</strong></font></div>\n");
			printf("<br>\n");
			printf("<HR>\n");
			printf("</p>\n");
			return;
		        }
	        }

	printf("<P>\n");
	printf("<font size=-1>\n");

	while(1){

		if(use_lifo==TRUE){
			if(pop_lifo(input_buffer,(int)sizeof(input_buffer))!=LIFO_OK)
				break;
		        }
		else{
			fgets(input_buffer,(int)(sizeof(input_buffer)-1),fp);
			if(feof(fp))
				break;
		        }

		strcpy(image,"");
		strcpy(image_alt,"");
		system_message=FALSE;

		strcpy(input_buffer2,input_buffer);

		if(strstr(input_buffer,"SERVICE ALERT:")){
			
			history_type=SERVICE_HISTORY;

			/* get host and service names */
			temp_buffer=my_strtok(input_buffer2,"]");
			temp_buffer=my_strtok(NULL,":");
			temp_buffer=my_strtok(NULL,";");
			strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
			entry_host_name[sizeof(entry_host_name)-1]='\x0';
			temp_buffer=my_strtok(NULL,";");
			strncpy(entry_service_desc,(temp_buffer==NULL)?"":temp_buffer,sizeof(entry_service_desc));
			entry_service_desc[sizeof(entry_service_desc)-1]='\x0';

			if(strstr(input_buffer,";CRITICAL;")){
				strncpy(image,CRITICAL_ICON,sizeof(image));
				strncpy(image_alt,CRITICAL_ICON_ALT,sizeof(image_alt));
				history_detail_type=HISTORY_SERVICE_CRITICAL;
                                }
			else if(strstr(input_buffer,";WARNING;")){
				strncpy(image,WARNING_ICON,sizeof(image));
				strncpy(image_alt,WARNING_ICON_ALT,sizeof(image_alt));
				history_detail_type=HISTORY_SERVICE_WARNING;
                                }
			else if(strstr(input_buffer,";UNKNOWN;")){
				strncpy(image,UNKNOWN_ICON,sizeof(image));
				strncpy(image_alt,UNKNOWN_ICON_ALT,sizeof(image_alt));
 				history_detail_type=HISTORY_SERVICE_UNKNOWN;
                               }
			else if(strstr(input_buffer,";RECOVERY;")){
				strncpy(image,RECOVERY_ICON,sizeof(image));
				strncpy(image_alt,RECOVERY_ICON_ALT,sizeof(image_alt));
				history_detail_type=HISTORY_SERVICE_RECOVERY;
                                }
		        }

		else if(strstr(input_buffer,"HOST ALERT:")){

			history_type=HOST_HISTORY;

			/* get host name */
			temp_buffer=my_strtok(input_buffer2,"]");
			temp_buffer=my_strtok(NULL,":");
			temp_buffer=my_strtok(NULL,";");
			strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
			entry_host_name[sizeof(entry_host_name)-1]='\x0';

			if(strstr(input_buffer,";DOWN")){
				strncpy(image,HOST_CRITICAL_ICON,sizeof(image));
				strncpy(image_alt,HOST_CRITICAL_ICON_ALT,sizeof(image_alt));
				history_detail_type=HISTORY_HOST_DOWN;
		                }
			else if(strstr(input_buffer,";UNREACHABLE")){
				strncpy(image,HOST_CRITICAL_ICON,sizeof(image));
				strncpy(image_alt,HOST_CRITICAL_ICON_ALT,sizeof(image_alt));
				history_detail_type=HISTORY_HOST_UNREACHABLE;
		                }
			else if(strstr(input_buffer,";RECOVERY")){
				strncpy(image,HOST_RECOVERY_ICON,sizeof(image));
				strncpy(image_alt,HOST_RECOVERY_ICON_ALT,sizeof(image_alt));
				history_detail_type=HISTORY_HOST_RECOVERY;
		                }
		        }

		else if(strstr(input_buffer," starting...")){
			strncpy(image,START_ICON,sizeof(image));
			strncpy(image_alt,START_ICON_ALT,sizeof(image_alt));
			system_message=TRUE;
		        }
		else if(strstr(input_buffer," shutting down...")){
			strncpy(image,STOP_ICON,sizeof(image));
			strncpy(image_alt,STOP_ICON_ALT,sizeof(image_alt));
			system_message=TRUE;
		        }
		else if(strstr(input_buffer," restarting...")){
			strncpy(image,RESTART_ICON,sizeof(image));
			strncpy(image_alt,RESTART_ICON_ALT,sizeof(image_alt));
			system_message=TRUE;
		        }

		image[sizeof(image)-1]='\x0';
		image_alt[sizeof(image_alt)-1]='\x0';

		/* get the timestamp */
		temp_buffer=strtok(input_buffer,"]");
		t=(temp_buffer==NULL)?0L:strtoul(temp_buffer+1,NULL,10);
//
// agb 09/02/99
// Added code that will convert the message time to a tm structure.  This
// time will be used to set the current_message_date to Month DD YYYY HH:00 of
// the current message.  If the current_message_date does not match the 
// last_message_date, the display will be seperated and the current_message_date
// will be printed in the break.
//
		time_ptr=localtime(&t);
		strftime(current_message_date,sizeof(current_message_date),"%B %d, %Y %H:00\n",time_ptr);
		current_message_date[sizeof(current_message_date)-1]='\x0';
//
		get_time_string(&t,date_time,sizeof(date_time));
		strip(date_time);

		temp_buffer=strtok(NULL,"\n");

		if(strcmp(image,"")){

			display_line=FALSE;

			if(system_message==TRUE)
				display_line=TRUE;

			else if(display_type==DISPLAY_HOSTS){

				sprintf(match1," HOST ALERT: %s;",host_name);
				sprintf(match2," SERVICE ALERT: %s;",host_name);

				if(show_all_hosts==TRUE)
					display_line=TRUE;
				else if(strstr(temp_buffer,match1))
					display_line=TRUE;
				else if(strstr(temp_buffer,match2))
					display_line=TRUE;

				if(display_line==TRUE){
					if(history_options==HISTORY_ALL)
						display_line=TRUE;
					else if(history_options==HISTORY_HOST_ALL && history_type==HOST_HISTORY)
						display_line=TRUE;
					else if(history_options==HISTORY_SERVICE_ALL && history_type==SERVICE_HISTORY)
						display_line=TRUE;
					else if(history_detail_type & history_options)
						display_line=TRUE;
					else 
						display_line=FALSE;
			                }

				if(display_line==TRUE){
					if(state_options==STATE_ALL)
						display_line=TRUE;
					else if((state_options & STATE_SOFT) && strstr(temp_buffer,";SOFT;"))
						display_line=TRUE;
					else if((state_options & STATE_HARD) && strstr(temp_buffer,";HARD;"))
						display_line=TRUE;
					else
						display_line=FALSE;
				        }
			        }


			/* make sure user is authorized to view this host or service information */
			if(system_message==FALSE){

				if(history_type==HOST_HISTORY){
					temp_host=find_host(entry_host_name);
					if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
						display_line=FALSE;
				        }
				else{
					temp_service=find_service(entry_host_name,entry_service_desc);
					if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
						display_line=FALSE;
				        }
			        }
			
			/* display the entry if we should... */
			if(display_line==TRUE){

//
// agb 09/02/99
// Added if then loop to break the history display every hour.
//
				if(strcmp(last_message_date,current_message_date) != 0){
					printf("<br clear=\"all\">\n");
					printf("<div align=\"center\">\n");
					printf("<table border=0 width=95%%><tr>");
					printf("<td width=40%%><hr width=100%%></td>");
					printf("<td align=center><font color=\"red\" size=-1><strong>%s</strong></font></td>",current_message_date);
					printf("<td width=40%%><hr width=100%%></td>");
					printf("</tr></table>\n");
					printf("</div>\n");
					printf("<br clear=\"all\">\n");
					strncpy(last_message_date,current_message_date,sizeof(last_message_date));
					last_message_date[sizeof(last_message_date)-1]='\x0';
				        }
//

				printf("<img align=\"left\" src=\"%s%s\" alt=\"%s\">[%s] %s<br clear=\"all\">\n",url_images_path,image,image_alt,date_time,temp_buffer);
				found_line=TRUE;
			        }
		        }

                }

	printf("</FONT>\n");
	printf("</P>\n");
	
	if(found_line==FALSE){
		printf("<HR>\n");
		printf("<p>\n");
		printf("<div align=\"center\">\n");
		printf("<font color=\"red\"><strong>");
		printf("No history information was found %s in %s log file",(show_all_hosts==TRUE)?"":"for this host",(log_archive==0)?"the current":"this archived");
		printf("</strong></font>\n");
		printf("</div>\n");
		printf("</p>\n");
	        }

	printf("<HR>\n");

	if(use_lifo==TRUE)
		free_lifo_memory();
	else
		fclose(fp);

	return;
        }
