/***********************************************************************
 *
 * SHOWLOG.C - NetSaint Log File CGI
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 11-05-1999
 *
 * This CGI program will display the contents of the NetSaint
 * log file.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ***********************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include "../common/common.h"

#include "getcgi.h"
#include "cgiutils.h"
#include "auth.h"
#include "lifo.h"

extern char   main_config_file[MAX_FILENAME_LENGTH];
extern char   url_images_path[MAX_FILENAME_LENGTH];

extern char html_table_bg_odd_color[MAX_COLOR_LENGTH];

extern int    log_rotation_method;
extern time_t this_scheduled_log_rotation;
extern time_t last_scheduled_log_rotation;
extern time_t next_scheduled_log_rotation;

void document_header(void);
void document_footer(void);
int process_cgivars(void);

authdata current_authdata;

int display_log();

char log_file_to_use[MAX_FILENAME_LENGTH]="";
int log_archive=0;

int use_lifo=TRUE;


int main(void){
	int error=FALSE;
	int result=OK;
	time_t t;
	char date_time[48];


	error=process_cgivars();

	document_header();

	reset_cgi_vars();

	result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
	if(result==ERROR){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
		printf("Error: Could not open CGI configuration file '%s' for reading!\n",DEFAULT_CGI_CONFIG_FILE);
		printf("</strong></font></div></p>\n");
		document_footer();
		return ERROR;
	        }

	result=read_main_config_file(main_config_file);
	if(result==ERROR){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
		printf("Error: Could not open main configuration file for reading!\n");
		printf("</strong></font></div></p>\n");
		document_footer();
		return ERROR;
	        }

	result=read_all_host_config_files(READ_HOSTGROUPS|READ_CONTACTGROUPS|READ_HOSTS|READ_SERVICES);
	if(result==ERROR){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
		printf("Error: Could not open host configuration file(s) for reading!\n");
		printf("</strong></font></div></p>\n");
		document_footer();
		return ERROR;
                }

	get_log_archive_to_use(log_archive,log_file_to_use,(int)sizeof(log_file_to_use));

	/* begin top table */
	printf("<p>\n");
	printf("<table border=0 width=100%%>\n");
	printf("<tr>\n");
	printf("<td align=left valign=top>\n");

	/* left column of the first row */
	printf("<table border=1>\n");
	printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
	printf("<font size=+2><b>");
	if(log_rotation_method==LOG_ROTATION_NONE || log_archive==0)
		printf("Current Log File");
	else
		printf("Archived Log File");
	printf("</b></font><br>\n");
	printf("<font size=-2>\n");
	time(&t);
	get_time_string(&t,date_time,(int)sizeof(date_time));
	printf("Last Updated: %s<BR>",date_time);
	printf("NetSaint Network Monitor - <a href=\"http://www.netsaint.org\" target=\"_new\">www.netsaint.org</a><br>\n");
	printf("</font>\n");
	printf("</td></tr>\n");
	printf("</table>\n");
	printf("</td>\n");

	/* center column of top row */
	printf("<td align=left valign=center>\n");
	printf("</td>\n");

	/* right hand column of top row */
	printf("<td align=right valign=bottom>\n");

	printf("<table border=0>\n");
	printf("<form method=\"GET\" action=\"%s\">\n",SHOWLOG_CGI);
	printf("<input type='hidden' name='archive' value='%d'>\n",log_archive);
	printf("<tr>");
	printf("<td align=left valign=bottom><font size=-2>Older Entries First:</font></td>\n");
	printf("<td></td>\n");
	printf("</tr>\n");
	printf("<tr>");
	printf("<td align=left valign=bottom><font size=-1><input type='checkbox' name='oldestfirst' %s></font></td>",(use_lifo==FALSE)?"checked":"");
	printf("<td align=left valign=bottom><font size=-1><input type='submit' value='Update'></font></td>\n");
	printf("</tr>\n");
	printf("</form>\n");
	printf("</table>\n");

	printf("</td>\n");

	/* end of top table */
	printf("</tr>\n");
	printf("</table>\n");
	printf("</p>\n");

	if(log_rotation_method!=LOG_ROTATION_NONE){
		printf("<p>\n");
		printf("<div align=\"center\">\n");
		printf("<table border=0>\n");
		printf("<tr>\n");
		printf("<td>\n");

		printf("<table border=0 cellspacing=20 cellpadding=0>\n");
		printf("<tr>\n");
		printf("<td align=center valign=center>\n");
		printf("<font size=-1>");
		if(log_archive==0){
			printf("<font size=-1 color=red>Latest Archive</font><br>");
			printf("<a href=\"%s?archive=1%s\"><img src=\"%s%s\" border=0 alt=\"Latest Archive\"></a>",SHOWLOG_CGI,(use_lifo==FALSE)?"?oldestfirst":"",url_images_path,LEFT_ARROW_ICON);
		        }
		else{
			printf("<font size=-1 color=red>Earlier Archive</font><br>");
			printf("<a href=\"%s?archive=%d%s\"><img src=\"%s%s\" border=0 alt=\"Earlier Archive\"></a>",SHOWLOG_CGI,log_archive+1,(use_lifo==FALSE)?"&oldestfirst":"",url_images_path,LEFT_ARROW_ICON);
		        }
		printf("</font>");
		printf("</td>\n");

		if(log_archive!=0){
			printf("<td align=center>\n");
			get_time_string((unsigned long*)&last_scheduled_log_rotation,date_time,(int)sizeof(date_time));
			printf("<font size=-2><strong>%s</strong>",date_time);
			printf("<br>to<br>");
			get_time_string((unsigned long*)&this_scheduled_log_rotation,date_time,(int)sizeof(date_time));
			printf("<strong>%s</strong></font>",date_time);

			printf("</td>\n");
			printf("<td align=center valign=center>\n");

			printf("<font size=-1>");
			if(log_archive==1){
				printf("<font size=-1 color=red>Current Log</font><br>");
				printf("<a href=\"%s%s\"><img src=\"%s%s\" border=0 alt=\"Current Log\"></a>",SHOWLOG_CGI,(use_lifo==FALSE)?"?oldestfirst":"",url_images_path,RIGHT_ARROW_ICON);
			        }
			else{
				printf("<font size=-1 color=red>More Recent Archive</font><br>");
				printf("<a href=\"%s?archive=%d%s\"><img src=\"%s%s\" border=0 alt=\"More Recent Archive\"></a>",SHOWLOG_CGI,log_archive-1,(use_lifo==FALSE)?"&oldestfirst":"",url_images_path,RIGHT_ARROW_ICON);
			        }
			printf("</font>");
			printf("</td>\n");
		        }

		printf("</tr>\n");
		printf("</table>\n");
	
		printf("</td>\n");
		printf("</tr>\n");
		printf("</table>\n");
		printf("</div>\n");
		printf("</p>\n");
	        }

	/* get authentication information */
	get_authentication_information(&current_authdata);

	display_log();

	document_footer();

	return OK;
        }




void document_header(void){
	char date_time[48];
	time_t current_time;
	time_t expire_time;


	printf("Cache-Control: no-store\n");
	printf("Pragma: no-cache\n");

	time(&current_time);
	get_expire_time_string(&current_time,date_time,(int)sizeof(date_time));
	printf("Last-Modified: %s\n",date_time);

	expire_time=(time_t)0L;
	get_expire_time_string(&expire_time,date_time,(int)sizeof(date_time));
	printf("Expires: %s\n",date_time);

	printf("Content-type: text/html\n\n");

	printf("<html>\n");
	printf("<head>\n");
	printf("<title>\n");
	printf("NetSaint Log File\n");
	printf("</title>\n");
	printf("</head>\n");
	printf("<body bgcolor=\"%s\">\n",HTML_BACKGROUND_COLOR);

	return;
        }


void document_footer(void){

	printf("</body>\n");
	printf("</html>\n");

	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			continue;
		        }

		/* we found the archive argument */
		else if(!strcmp(variables[x],"archive")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			log_archive=atoi(variables[x]);
		        }

		/* we found the order argument */
		else if(!strcmp(variables[x],"oldestfirst")){
			use_lifo=FALSE;
		        }

		/* we recieved an invalid argument */
		else
			error=TRUE;
	
	        }

	return error;
        }



int display_log(void){
	char input_buffer[MAX_INPUT_BUFFER];
	char image[MAX_INPUT_BUFFER];
	char image_alt[MAX_INPUT_BUFFER];
	time_t t;
	char *temp_buffer;
	char date_time[48];
	int error;
	FILE *fp=NULL;

//
// agb 09/02/99
// Added last_message_date, current_message_date, and time pointer to display
// a break every hour in the history display.  This break would contain the
// current date and hour of all messages to follow.
//
	char last_message_date[MAX_INPUT_BUFFER]="";
	char current_message_date[MAX_INPUT_BUFFER]="";
	struct tm *time_ptr;
//


	/* check to see if the user is authorized to view the log file */
	if(is_authorized_for_system_information(&current_authdata)==FALSE){
		printf("<hr>\n");
		printf("<p><div align=\"center\">\n");
		printf("<strong><font color=red>It appears as though you do not have permission to view the log file...</font></strong><br><br>\n");
		printf("If you believe this is an error, check the HTTP server authentication requirements for accessing this CGI<br>");
		printf("and check the authorization options in your CGI configuration file.\n");
		printf("</div></p>\n");
		printf("<hr>\n");
		return ERROR;
	        }

	error=FALSE;

	if(use_lifo==TRUE){
		error=read_file_into_lifo(log_file_to_use);
		if(error!=LIFO_OK){
			if(error==LIFO_ERROR_MEMORY){
				printf("<p>\n");
				printf("<div align=\"center\">\n");
				printf("<font color=\"red\"><strong>");
				printf("<blink>Not enough memory to reverse log file - displaying log in natural order...</blink>");
				printf("</strong></font>\n");
				printf("</div>\n");
				printf("</p>\n");
				error=FALSE;
			        }
			else
				error=TRUE;
			use_lifo=FALSE;
		        }
		else
			error=FALSE;
	        }

	if(use_lifo==FALSE){
  
		fp=fopen(log_file_to_use,"r");
		if(fp==NULL){
			printf("<hr>\n");
			printf("<p><div align=\"center\">\n");
			printf("<font color=\"red\"><strong>");
			printf("Error: Could not open log file '%s' for reading!",log_file_to_use);
			printf("</strong></font>\n");
			printf("</div></p>\n");
			printf("<hr>\n");
			error=TRUE;
	                }
	        }

	if(error==FALSE){

		printf("<p>\n");
		printf("<font size=-1>\n");
		
		while(1){

			if(use_lifo==TRUE){
				if(pop_lifo(input_buffer,(int)sizeof(input_buffer))!=LIFO_OK)
					break;
			        }
			else{
				fgets(input_buffer,(int)(sizeof(input_buffer)-1),fp);
				if(feof(fp))
					break;
			        }

			if(strstr(input_buffer,"Warning:")){
				strcpy(image,INFO_ICON);
				strcpy(image_alt,INFO_ICON_ALT);
			        }
			else if(strstr(input_buffer," starting...")){
				strcpy(image,START_ICON);
				strcpy(image_alt,START_ICON_ALT);
			        }
			else if(strstr(input_buffer," shutting down...")){
				strcpy(image,STOP_ICON);
				strcpy(image_alt,STOP_ICON_ALT);
			        }
			else if(strstr(input_buffer,"Bailing out due to errors...")){
				strcpy(image,STOP_ICON);
				strcpy(image_alt,STOP_ICON_ALT);
			        }
			else if(strstr(input_buffer," restarting...")){
				strcpy(image,RESTART_ICON);
				strcpy(image_alt,RESTART_ICON_ALT);
			        }
			else if(strstr(input_buffer,"HOST ALERT:") && strstr(input_buffer,";DOWN;")){
				strcpy(image,HOST_CRITICAL_ICON);
				strcpy(image_alt,HOST_CRITICAL_ICON_ALT);
			        }
			else if(strstr(input_buffer,"HOST ALERT:") && strstr(input_buffer,";UNREACHABLE;")){
				strcpy(image,HOST_CRITICAL_ICON);
				strcpy(image_alt,HOST_CRITICAL_ICON_ALT);
			        }
			else if(strstr(input_buffer,"HOST ALERT:") && strstr(input_buffer,";RECOVERY;")){
				strcpy(image,HOST_RECOVERY_ICON);
				strcpy(image_alt,HOST_RECOVERY_ICON_ALT);
			        }
			else if(strstr(input_buffer,"HOST NOTIFICATION:")){
				strcpy(image,HOST_NOTIFICATION_ICON);
				strcpy(image_alt,HOST_NOTIFICATION_ICON_ALT);
			        }
			else if(strstr(input_buffer,"SERVICE ALERT:") && strstr(input_buffer,";CRITICAL;")){
				strcpy(image,CRITICAL_ICON);
				strcpy(image_alt,CRITICAL_ICON_ALT);
			        }
			else if(strstr(input_buffer,"SERVICE ALERT:") && strstr(input_buffer,";WARNING;")){
				strcpy(image,WARNING_ICON);
				strcpy(image_alt,WARNING_ICON_ALT);
			        }
			else if(strstr(input_buffer,"SERVICE ALERT:") && strstr(input_buffer,";UNKNOWN;")){
				strcpy(image,UNKNOWN_ICON);
				strcpy(image_alt,UNKNOWN_ICON_ALT);
			        }
			else if(strstr(input_buffer,"SERVICE ALERT:") && strstr(input_buffer,";RECOVERY;")){
				strcpy(image,RECOVERY_ICON);
				strcpy(image_alt,RECOVERY_ICON_ALT);
			        }
			else if(strstr(input_buffer,"SERVICE NOTIFICATION:")){
				strcpy(image,NOTIFICATION_ICON);
				strcpy(image_alt,NOTIFICATION_ICON_ALT);
			        }
			else if(strstr(input_buffer,"SERVICE EVENT HANDLER:")){
				strcpy(image,SERVICE_EVENT_ICON);
				strcpy(image_alt,SERVICE_EVENT_ICON_ALT);
			        }
			else if(strstr(input_buffer,"HOST EVENT HANDLER:")){
				strcpy(image,HOST_EVENT_ICON);
				strcpy(image_alt,HOST_EVENT_ICON_ALT);
			        }
			else if(strstr(input_buffer,"EXTERNAL COMMAND:")){
				strcpy(image,EXTERNAL_COMMAND_ICON);
				strcpy(image_alt,EXTERNAL_COMMAND_ICON_ALT);
			        }
			else if(strstr(input_buffer,"LOG ROTATION:")){
				strcpy(image,LOG_ROTATION_ICON);
				strcpy(image_alt,LOG_ROTATION_ICON_ALT);
			        }
			else if(strstr(input_buffer,"active mode...")){
				strcpy(image,ACTIVE_ICON);
				strcpy(image_alt,ACTIVE_ICON_ALT);
			        }
			else if(strstr(input_buffer,"standby mode...")){
				strcpy(image,STANDBY_ICON);
				strcpy(image_alt,STANDBY_ICON_ALT);
			        }
			else if(strstr(input_buffer,"Zombie cleanup")){
				strcpy(image,ZOMBIE_CLEANUP_ICON);
				strcpy(image_alt,ZOMBIE_CLEANUP_ICON_ALT);
			        }
			else{
				strcpy(image,INFO_ICON);
				strcpy(image_alt,INFO_ICON_ALT);
			        }

			temp_buffer=strtok(input_buffer,"]");
			t=(temp_buffer==NULL)?0L:strtoul(temp_buffer+1,NULL,10);
//
// agb 09/02/99
// Added code that will convert the message time to a tm structure.  This
// time will be used to set the current_message_date to Month DD YYYY HH:00 of
// the current message.  If the current_message_date does not match the 
// last_message_date, the display will be seperated and the current_message_date
// will be printed in the break.
//
			time_ptr=localtime(&t);
			strftime(current_message_date,sizeof(current_message_date),"%B %d, %Y %H:00\n",time_ptr);
			current_message_date[sizeof(current_message_date)-1]='\x0';
//

//
// agb 09/02/99
// Added if then loop to break the history display every hour.
//
			if(strcmp(last_message_date,current_message_date) != 0){
				printf("<br clear=\"all\">\n");
				printf("<div align=\"center\">\n");
				printf("<table border=0 width=95%%><tr>");
				printf("<td width=40%%><hr width=100%%></td>");
				printf("<td align=center><font color=\"red\" size=-1><strong>%s</strong></font></td>",current_message_date);
				printf("<td width=40%%><hr width=100%%></td>");
				printf("</tr></table>\n");
				printf("</div>\n");
				printf("<br clear=\"all\">\n");
				strncpy(last_message_date,current_message_date,sizeof(last_message_date));
				last_message_date[sizeof(last_message_date)-1]='\x0';
				}
//

			get_time_string(&t,date_time,(int)sizeof(date_time));
			strip(date_time);

			temp_buffer=strtok(NULL,"\n");
			
			printf("<img align=\"left\" src=\"%s%s\" alt=\"%s\">[%s] %s<br clear=\"all\">\n",url_images_path,image,image_alt,date_time,(temp_buffer==NULL)?"":temp_buffer);

		        }

		printf("</font>\n");
		printf("</p>\n");

		printf("<hr>\n");

		if(use_lifo==FALSE)
			fclose(fp);
	        }

	document_footer();

	if(use_lifo==TRUE)
		free_lifo_memory();

	return OK;
        }















































